#include <assert.h>
#include <Xm/Xm.h>

static const char nullChar = '\0';
static const char newLineChar = '\n';

/*
//
// Convert an ASCII string to a compound string (XmString)
// 
*/
XmString strToXmString( const char* cp ) 
{
	XmString xms = NULL;	/* Result compound string */
	XmString txms;			/* Temporary compound string */
	char*	 ts;			/* Temporary string buffer */
	char*	 tsp;			/* Pointer for temporary string buffer */
	char	 c;				/* Current char */
        XmString xmstr ;
	
	/* Return null compound string if char* is null */

	if ( cp == 0 )
	{ 
		return XmStringCreateLocalized( "" );
    }

	/* Create temporary string buffer */

    ts = tsp = XtMalloc( strlen(cp) + 1 );

	/* Create a compound string from a ASCII string */

	do
	{
		c = *cp++;
		if ( c == nullChar || c == newLineChar )	/* is null or new line */
		{
			if ( ts != tsp )	/* is not empty string */
			{
				*tsp = nullChar;	/* Make null terminated string */
				txms =  XmStringCreateLocalized( ts );
				xmstr = XmStringConcat ( xms, txms );
 				XmStringFree ( xms ) ;
				xms = XmStringCopy ( xmstr ) ;
				XmStringFree ( xmstr ) ;
				XmStringFree( txms );
			}

			if ( c == newLineChar )		/* is new line code */
			{
				/* Concatenate a separator */

				txms =  XmStringSeparatorCreate();
				xmstr = XmStringConcat ( xms, txms );
                                XmStringFree ( xms ) ;
                                xms = XmStringCopy ( xmstr ) ;
                                XmStringFree ( xmstr ) ;
				XmStringFree( txms );
				tsp = ts;		/* Move pointer to start of temporary string */
			}
		}
		else
		{
			*tsp++ = c;
		}
	} while ( c != nullChar );

	/* Free temporary string buffer */

	XtFree( ts );

	return xms;
}
