#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include "sky_extern.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_type.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
char GetUpper ( char x ) ;
char GetLower ( char x ) ;
Sky_ReturnVal_E get_file_name_from_new_satellite ( char * name, char * filename ) ;
Sky_ReturnVal_E insert_new_satellite_to_initfile ( char * name, char * filename ) ;
void insert_satellite_name_into_entry ( void ) ;
Sky_ReturnVal_E DoSameSatelliteExist ( unsigned char * name ) ;
void ResetNetworkPIDs ( int tp_id ) ;
void ResetAllNetworkPIDs ( SI_STRUCTURE * si_p ) ;
void List_SelectedScrolled ( Widget w, int pos, int page ) ;
Sky_ReturnVal_E Get_NextSatelliteFileName ( char * outp ) ;
Sky_ReturnVal_E APP_Sky_ReleaseAllNetworkPID ( int tp_id ) ;

/**************************************************************************
 * Extern Global Variables
 **************************************************************************/
extern PixmapResources_t pixmap_resources ;

Sky_ReturnVal_E get_filename ( char * inp, char * outp ) 
{
   Sky_LastInfo_S   sky_p ;
   int              count ;
   int              i ;

   read_initfile ( & sky_p ) ;

   for ( i = 0, count = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
      if ( sky_p . sat_list [ i ] . flag == 1 ) {

         if ( ! strcmp ( inp, sky_p . sat_list [ i ] . satellite_name ) ) {
            strcpy ( outp, sky_p . sat_list [ i ] . satellite_file ) ;
            break ;
         }
         if ( count ++ < sky_p . num_of_satellite ) {
            continue ;
         } else {
            break ;
         }
      }
   }

   if ( count == sky_p . num_of_satellite ) {
      return SKY_FAIL ;
   } else {
      return SKY_SUCCESS ;
   }
}

Sky_ReturnVal_E Get_NextSatelliteFileName ( char * outp ) 
{
   Sky_LastInfo_S   sky_p ;
   int              count ;
   int              i ;

   read_initfile ( & sky_p ) ;

   if ( sky_p.num_of_satellite <= 0 ) {
     return SKY_FAIL ; /* no Satellite File */
   }

   /* Current Satellite  -> ο Satellite Set */
   for ( i = 0, count = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
      if ( sky_p . sat_list [ i ] . flag != 1 ) {
         continue ;
      }

      if ( ! strcmp ( sky_p.satellite_name,  sky_p.sat_list [ i ] . satellite_name ) ) {
         sky_p.num_of_satellite = sky_p.num_of_satellite - 1 ;
         memset ( &sky_p . sat_list [ i ], 0, sizeof ( Sky_SatelliteEntry_S ) ) ;
      }
   }

   if ( sky_p.num_of_satellite <= 0 ) {
     memset ( sky_p.satellite_name, 0 , sizeof ( sky_p.satellite_name ) ) ;
     memset ( sky_p.satellite_file, 0 , sizeof ( sky_p.satellite_file ) ) ;
     sky_p.transponder_id = 0 ;
     sky_p.num_of_satellite = 0 ;

     write_initfile ( & sky_p ) ;
     return SKY_FAIL ; /* no Satellite File */
   } 

   for ( i = 0, count = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
      if ( sky_p . sat_list [ i ] . flag != 1 ) {
         continue ;
      }

      strcpy ( outp, sky_p . sat_list [ i ] . satellite_file ) ;

      strcpy ( sky_p . satellite_name, sky_p . sat_list [ i ] . satellite_name ) ;
      strcpy ( sky_p . satellite_file, sky_p . sat_list [ i ] . satellite_file ) ;
      sky_p . transponder_id = 0 ;

      write_initfile ( & sky_p ) ;
      break ;
   }
   return SKY_SUCCESS ;
}

/* Make New Satellite FIle Name from Satellite Name */
Sky_ReturnVal_E get_file_name_from_new_satellite ( char * name, char * filename )
{
   int     i ;
   int     j ;
   int     flag ;
   int     namelen ;

   namelen = strlen ( name ) ;

   sprintf ( filename , "%s/", DATAFILE_PATH ) ;
   j = strlen ( filename ) ;

   filename [ j ++ ] = GetUpper ( name [ 0 ] ) ;

   flag = 0 ;
   for ( i = 1 ; i < namelen ; i ++ ) {
      if ( name [ i ] == ' ' ) {
         flag = 1 ;
      } else if ( flag ) {
         filename [ j ++ ] = GetUpper ( name [ i ] ) ;
         flag = 0 ;
      } else {
         filename [ j ++ ] = GetLower ( name [ i ] ) ;
      }
   }
   filename [ j ] = 0 ;
   strcat ( filename, "." ) ;
   strcat ( filename, SATELLITE_FILE_NAME_EXTENSION ) ;

   return SKY_SUCCESS ;
}

Sky_ReturnVal_E IsSatelliteNameValid ( char * name ) 
{
   char  buf [ 100 ] ;
   int   namelen ;
   int   i ;

   memset ( buf, 0, sizeof ( buf ) ) ;
   namelen = strlen ( name ) ;

   /* space || alphabet || digit -> ok */
   /* all spaces -> not ok */

   /*   ̽  */
   for ( i = 0 ; i < namelen ; i ++ ) {
      if ( name [ i ] != ' ' ) {
         break ;
      }
   }
   if ( i == namelen ) {
      #ifdef DEBUG_FAIL
      printf ( "Invalid Name = [%s]\n", name ) ;
      #endif

      return SKY_FAIL ;
   }
   memcpy ( buf, & name [ i ], namelen - i ) ;
   buf [ namelen - i ] = 0 ;

   namelen = strlen ( buf ) ;
   for ( i = 0 ;  i < namelen ; i ++ ) {
      if ( ( buf [ i ] == ' ' ) ||
           ( ( buf [ i ] >= 'A' ) && ( buf [ i ] <= 'Z' ) ) ||
           ( ( buf [ i ] >= 'a' ) && ( buf [ i ] <= 'z' ) ) ||
           ( ( buf [ i ] >= '0' ) && ( buf [ i ] <= '9' ) ) ) {
         continue ;
      } else {
         #ifdef DEBUG_FAIL
         printf ( "Invalid Name = [%s]\n", buf ) ;
         #endif

         return SKY_FAIL ;
      }
   }
   strcpy ( name, buf ) ;
   return SKY_SUCCESS ;
}

char GetUpper ( char x )
{
   if ( ( x >= 'a' ) && ( x <= 'z' ) ) {
      return ( x - 'a' + 'A' ) ;
   } else {
      return x ;
   }
}
char GetLower ( char x )
{
   if ( ( x >= 'A' ) && ( x <= 'Z' ) ) {
      return ( x - 'A' + 'a' ) ;
   } else {
      return x ;
   }
}

Sky_ReturnVal_E insert_new_satellite_to_initfile ( char * name, char * filename ) 
{
   int              i ;
   int              find = 0 ;
   int              position_t ;
   int              position_c ;
   Sky_LastInfo_S   sky_p ;
#if 0
   PM_INFORM        pmlist ;
#endif

   read_initfile ( & sky_p ) ;

   if ( sky_p . num_of_satellite >= MAX_SATELLITE_NUM ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Overflower Satellite [ Max = %d, Cur = %d ]\n", __FILE__, __LINE__, MAX_SATELLITE_NUM, sky_p . num_of_satellite ) ;
      #endif

      return SKY_FAIL ;
   }
   sky_p . sat_list [ sky_p . num_of_satellite ] . flag = 1 ;
   strcpy ( sky_p . sat_list [ sky_p . num_of_satellite ] . satellite_name, name ) ;
   strcpy ( sky_p . sat_list [ sky_p . num_of_satellite ] . satellite_file, filename ) ;
   sky_p . num_of_satellite ++ ;

   /*--------------------------------*/
   /*  ̸,  */
   /*--------------------------------*/
   strcpy ( sky_p . satellite_name, name ) ;
   strcpy ( sky_p . satellite_file, filename ) ;

   /*--------------------------------*/
   /* Setting Current Program Number */
   /*--------------------------------*/
#if 0
   if ( Sky_GetNumOfTransponder ( ) != 0 ) {
      if ( Sky_GetNumOfProgram ( 1 ) != 0 ) {

         sky_p . transponder_id = 1 ;
         sky_p . channel_id = 1 ;

         if ( Sky_GetInfoProgram ( sky_p . transponder_id * 100 + sky_p . channel_id, & pmlist ) != FALSE ) {

            #ifdef DEBUG
            printf ( "%s : %d : Sky_GetInfoProgram () Fail\n", __FILE__, __LINE__ ) ;
            #endif

         } else {
            strcpy ( sky_p . channel_name, ( char * ) pmlist . ProgramName ) ;
         }
      }
   }
#endif   
   write_initfile ( & sky_p ) ;

   return SKY_SUCCESS ;
}
/* Satellite List߿   ߰. */
void insert_satellite_name_into_entry ( void ) 
{
   XmString         xmstr ;
   Arg              args [ 2 ] ;
   int              ac ;

   Sky_LastInfo_S   sky_p ;
   int              idx ;

   read_initfile ( & sky_p ) ;

   if ( sky_p . num_of_satellite == 0 ) {
      return ;
   } else {
      idx = sky_p . num_of_satellite - 1 ;
   }
#if 0  
   xmstr = XmStringCreate ( sky_p . sat_list [ idx ] . satellite_name, XmFONTLIST_DEFAULT_TAG ) ;
#else
   xmstr = ( XmString ) strToXmString (  sky_p . sat_list [ idx ] . satellite_name ) ;
#endif

   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
   ac ++ ;

   switch ( idx ) {
      case 0 :
         XtSetValues ( tuner_satMenu_pdB1, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB1 ) ;
         break ;
      case 1 :
         XtSetValues ( tuner_satMenu_pdB2, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB2 ) ;
         break ;
      case 2 :
         XtSetValues ( tuner_satMenu_pdB3, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB3 ) ;
         break ;
      case 3 :
         XtSetValues ( tuner_satMenu_pdB4, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB4 ) ;
         break ;
      case 4 :
         XtSetValues ( tuner_satMenu_pdB5, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB5 ) ;
         break ;
      case 5 :
         XtSetValues ( tuner_satMenu_pdB6, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB6 ) ;
         break ;
      case 6 :
         XtSetValues ( tuner_satMenu_pdB7, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB7 ) ;
         break ;
      case 7 :
         XtSetValues ( tuner_satMenu_pdB8, args, ac ) ; 
         XtManageChild ( tuner_satMenu_pdB8 ) ;
         break ;
   }
   XmStringFree ( xmstr ) ;
}

Sky_ReturnVal_E DoSameSatelliteExist ( unsigned char * name )
{
  Sky_LastInfo_S  sky_r ;
  int             i ;

  read_initfile ( & sky_r ) ;

  for ( i = 0 ; i < sky_r . num_of_satellite ; i ++ ) {
    if ( !strcmp ( ( char * ) name, ( char * ) sky_r . sat_list [ i ] . satellite_name ) ) {
      return SKY_SUCCESS ;
    }
  }

  return SKY_FAIL ;
}
void ResetNetworkPIDs ( int tp_id )
{
  DATAPROGRAM_STRUCTURE  data_r ;
  SETDATA_STRUCTURE      setdata_r ;
  int                    numData ;
  int                    i ;

  numData = Sky_GetNumOfData ( tp_id ) ;
  memset ( & data_r, 0, sizeof ( data_r ) ) ;
  for ( i = 0 ; i < numData ; i ++ ) {
    data_r.DataNumber = tp_id * 100 + i + 1 ;
    if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
       #endif

       continue ;
     }

     if ( ! data_r . SetFlag ) {
       continue ;
     }

     /*-------------------------------------------------*/
     /* Make Message */
     /*-------------------------------------------------*/
     memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
     setdata_r . DataPID       = data_r . DataPID ;
     setdata_r . Encapsulation = data_r . Encapsulation ;
     setdata_r . pgnumber      = data_r . pgnumber ;
     setdata_r . PmtPID        = data_r . PmtPID ;

     /*-------------------------------------------------*/
     /* Set network PID */
     /*-------------------------------------------------*/
     if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
       #endif

     }
  }
}

#if 0
void ResetAllNetworkPIDs ( SI_STRUCTURE * si_p ) 
{
  DATAPROGRAM_STRUCTURE  data_r ;
  SETDATA_STRUCTURE      setdata_r ;
  int                    numData ;
  int                    i ;
  int                    j ;

  int                    si_numData = si_p -> NumOfData ;
  int                    iterate = 0 ;  

  numData = Sky_GetNumOfData ( si_p -> TrInfo . TransponderNumber ) ;

  #ifdef DEBUG
  printf ( "ResetAllNetworkPIDs () : org numData = %d, si numData = %d\n", numData, si_numData ) ;
  for ( i = 0 ; i < si_numData ; i ++ ) {
    printf ( "[ %d ] DataNumber = %d\n", i, si_p ->DataInfo [ i ].DataNumber ) ;
    printf ( "[ %d ] DataPID = 0x%x\n", i, si_p ->DataInfo [ i ].DataPID ) ;
    printf ( "[ %d ] Encapsulation = %d\n", i, si_p ->DataInfo [ i ].Encapsulation ) ;
  }
  #endif

  if ( numData >= si_numData ) {
    iterate = si_numData ;
  } else if ( numData < si_numData ) {
    iterate = numData ;
  } 

  #ifdef DEBUG
  printf ( "iterate = %d\n", iterate ) ;
  #endif

  /* ********************************************************************
   * i : old_data iterator
   * j : si_data iterator
   * *******************************************************************/
  memset ( & data_r, 0, sizeof ( data_r ) ) ;
  for ( i = 0, j = 0 ; i < iterate ; i ++ ) {
    data_r.DataNumber = si_p -> TrInfo . TransponderNumber * 100 + i + 1 ;

    #ifdef DEBUG
    printf ( "%s : %d : [ old %d ], [ new %d ] Before GetInfoData (). DataNumber = %d\n", 
             __FILE__, __LINE__, i, j, data_r.DataNumber ) ; 
    #endif

    if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
       #endif

       continue ;
    }

    #ifdef DEBUG
    printf ( "%s : %d : [ old %d ], [ new %d ] After GetInfoData (). DataNumber = %d, PID = %d, Type = %d\n", 
             __FILE__, __LINE__,  i, j, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation ) ; 
    #endif

     /*-------------------------------------------------*/
     /* Manual Add̸ skip */
     /*-------------------------------------------------*/
     if ( data_r . ManualAdd ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : [ old %d ], [ new %d ] Manual Add -> continue.\n", __FILE__, __LINE__, i, j ) ;
       #endif

       continue ;
     }

    #ifdef DEBUG
    printf ( "%s : %d : [ old %d ], [ new %d ] Not Manual Add\n", 
             __FILE__, __LINE__,  i, j ) ;
    #endif

     /*-------------------------------------------------*/
     /*  */
     /*-------------------------------------------------*/
     if ( ( data_r . DataPID != si_p -> DataInfo [ j ] . DataPID ) ||
          ( data_r . Encapsulation != si_p -> DataInfo [ j ] . Encapsulation ) ) {

       data_r . DataPID = si_p -> DataInfo [ j ] . DataPID ; 
       data_r . Encapsulation = si_p -> DataInfo [ j ] . Encapsulation ; 

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  i, j, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ j ] . DataPID, si_p -> DataInfo [ j ] . Encapsulation ) ; 
       #endif

       if ( Sky_UpdateData ( data_r ) != FALSE ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_UpdateData () Fail.\n", __FILE__, __LINE__ ) ;
         #endif

         j ++ ;
         continue ;
       } 

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] Sky_UpdateData () succ. DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  i, j, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ j ] . DataPID, si_p -> DataInfo [ j ] . Encapsulation ) ; 
       #endif
     }

     if ( data_r . SetFlag == 0 ) {

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] SetFlag == 0. DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  i, j, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ j ] . DataPID, si_p -> DataInfo [ j ] . Encapsulation ) ; 
       #endif

       j ++ ;
       continue ;
     }

     #ifdef DEBUG
     printf ( "%s : %d : [ old %d ], [ new %d ] SetFlag == 1 . DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
              __FILE__, __LINE__,  i, j, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
              si_p -> DataInfo [ j ] . DataPID, si_p -> DataInfo [ j ] . Encapsulation ) ; 
     #endif

     /*-------------------------------------------------*/
     /* Make Message */
     /*-------------------------------------------------*/
     memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
     setdata_r . DataPID       = data_r . DataPID ;
     setdata_r . Encapsulation = data_r . Encapsulation ;

     #ifdef DEBUG
     printf ( "%s : %d : [ old %d ], [ new %d ] Before Sky_SetNetworkPID (). DataNumber = %d, PID = %d, Type = %d\n",
              __FILE__, __LINE__,  i, j, data_r.DataNumber, setdata_r.DataPID, setdata_r.Encapsulation ) ;  
     #endif

     /*-------------------------------------------------*/
     /* Set network PID */
     /*-------------------------------------------------*/
     if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
       #endif

     }

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_SetNetworkPID () succ. Number = %d, PID = %d\n", __FILE__, __LINE__, data_r .DataNumber, data_r.DataPID ) ;
     #endif

     j ++ ;
  }

  if ( numData > si_numData ) { /* Delete */
    for ( i = iterate ; i < numData ; i ++ ) {
      data_r.DataNumber = si_p -> TrInfo . TransponderNumber * 100 + i + 1 ;

      #ifdef DEBUG
      printf ( "%s : %d : [ old %d ] Before GetInfoData (). DataNumber = %d\n", 
               __FILE__, __LINE__, i, data_r.DataNumber ) ; 
      #endif

      if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
         #endif

         continue ;
       }
  
       /*-------------------------------------------------*/
       /* Manual Add -> Not Delete */
       /*-------------------------------------------------*/
       if ( data_r . ManualAdd ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Manual Add Flag Set -> continue.\n", __FILE__, __LINE__ ) ;
         #endif

         /*-------------------------------------------------*/
         /* Set network PID */
         /*-------------------------------------------------*/
         if ( data_r . SetFlag ) {
           if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

             #ifdef DEBUG_FAIL
             printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
             #endif
           }
         }
         continue ;
       }
       
       /*-------------------------------------------------*/
       /* Delete Data PID */
       /*-------------------------------------------------*/
       if ( Sky_DeleteData ( data_r ) != FALSE ) {
  
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_DeleteData () fail\n", __FILE__, __LINE__ ) ;
         #endif

       }
    }
  } else if ( ( numData < si_numData ) || ( j < si_numData ) ) { /* Add */

    #ifdef DEBUG
    printf ( "Add : j = %d\n", j ) ;
    #endif

    for ( i = j ; i < si_numData ; i ++ ) {
       /*-------------------------------------------------*/
       /* Add Data PID */
       /*-------------------------------------------------*/
       if ( Sky_AddData ( si_p -> TrInfo . TransponderNumber, 
                          & si_p -> DataInfo [ i ] ) != FALSE ) {
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_AddData () fail\n", __FILE__, __LINE__ ) ;
         #endif

         continue ;
       }

       #ifdef DEBUG
       printf ( "%s : %d : Sky_AddData () Succ. DataNumber = %d\n", __FILE__, __LINE__, si_p -> DataInfo [ i ].DataNumber ) ;
       #endif

       if ( si_p -> DataInfo [ i ] . SetFlag == 0 ) {
         continue ;
       }

       /*-------------------------------------------------*/
       /* Make Message */
       /*-------------------------------------------------*/
       memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
       setdata_r . DataPID       = si_p -> DataInfo [ i ] . DataPID ;
       setdata_r . Encapsulation = si_p -> DataInfo [ i ] . Encapsulation ;
       setdata_r . pgnumber      = si_p -> DataInfo [ i ] . pgnumber ;
       setdata_r . PmtPID        = si_p -> DataInfo [ i ] . PmtPID ;

       /*-------------------------------------------------*/
       /* Set Data PID */
       /*-------------------------------------------------*/
       if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {
  
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
         #endif

       }
    }
  } 

}
#endif

void List_SelectedScrolled ( Widget w, int pos, int page )
{
  int  tmp ;

  if ( pos <= page ) {
    tmp = 1 ;
  } else {
    tmp = pos - page + 1 ;
  }
  XmListSelectPos ( w, pos, False ) ;
  XmListSetPos ( w, tmp ) ;
}

Sky_ReturnVal_E APP_Sky_ReleaseAllNetworkPID ( int tp_id )
{
  int                    numData ;
  int                    i ;
  DATAPROGRAM_STRUCTURE  data_r ;

  /* Release All Network Data PID */
  if ( Sky_ReleaseAllNetworkPID ( ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_ReleaseAllNetworkPID () Fail.\n", __FILE__, __LINE__ ) ;
    #endif

  }

  /* Update Data Info */
  numData = Sky_GetNumOfData ( tp_id ) ;
  for ( i = 0 ; i < numData ; i ++ ) {
    memset ( & data_r, 0, sizeof ( DATAPROGRAM_STRUCTURE ) ) ;
    data_r . DataNumber = tp_id * 100 + i + 1 ;
    if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      continue ;
    }
    if ( ! data_r . SetFlag ) {
      continue ;
    } 

    data_r . SetFlag = 0 ;
    if ( Sky_UpdateData ( data_r ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_UpdateData () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

    }
  }

  return SKY_SUCCESS ;
}

void ResetAllNetworkPIDs ( SI_STRUCTURE * si_p ) 
{
  DATAPROGRAM_STRUCTURE  data_r ;
  SETDATA_STRUCTURE      setdata_r ;
  int                    numData ;
  int                    old_iter ;
  int                    new_iter ;

  int                    k ;
  int                    si_numData ;
  APP_ScanResult_E       find = APP_SCAN_NOTHING_E ;

  numData    = Sky_GetNumOfData ( si_p -> TrInfo . TransponderNumber ) ;
  si_numData = si_p -> NumOfData ;

  #ifdef DEBUG
  printf ( "ResetAllNetworkPIDs () : org numData = %d, si numData = %d\n", numData, si_numData ) ;
  for ( k = 0 ; k < si_numData ; k ++ ) {
    printf ( "[ %d ] DataNumber = %d\n", k, si_p ->DataInfo [ k ].DataNumber ) ;
    printf ( "[ %d ] DataPID = 0x%x\n", k, si_p ->DataInfo [ k ].DataPID ) ;
    printf ( "[ %d ] Encapsulation = %d\n", k, si_p ->DataInfo [ k ].Encapsulation ) ;
  }
  #endif

  if ( !numData && !si_numData ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Nothing To Do. Old Data Num = %d, Si Data Num = %d.\n", __FILE__, __LINE__, numData, si_numData ) ;
    #endif

    return ;
  }

  /* ********************************************************************
   * old_iter : old_data iterator
   * new_iter : si_data iterator
   * *******************************************************************/
  for ( old_iter = 0, new_iter = 0 ; ; old_iter ++ ) {

    if ( ( old_iter == numData ) && ( new_iter == si_numData ) ) {
      find = APP_SCAN_NOTHING_E ;
      break ;
    } else if ( ( old_iter < numData ) && ( new_iter == si_numData ) ) {
      find = APP_SCAN_DELETE_E ;
      break ;
    } else if ( ( old_iter == numData ) && ( new_iter < si_numData ) ) {
      find = APP_SCAN_ADD_E ;
      break ;
    }

    /*--------------------------------------------------------------------*/
    /* old_iter < numData && new_iter < si_numData */
    /*--------------------------------------------------------------------*/
    memset ( & data_r, 0, sizeof ( data_r ) ) ;

    data_r.DataNumber = si_p -> TrInfo . TransponderNumber * 100 + old_iter + 1 ;

    #ifdef DEBUG
    printf ( "%s : %d : [ old%d ], [ new%d ] Before GetInfoData (). DataNumber = %d\n", 
             __FILE__, __LINE__, old_iter, new_iter, data_r.DataNumber ) ; 
    #endif

    if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
       #endif

       continue ;
    }

    #ifdef DEBUG
    printf ( "%s : %d : [ old %d ], [ new %d ] After GetInfoData (). DataNumber = %d, PID = %d, Type = %d\n", 
             __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation ) ; 
    #endif

     /*-------------------------------------------------*/
     /* Manual Add̸ skip */
     /*-------------------------------------------------*/
     if ( data_r . ManualAdd ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : [ old %d ], [ new %d ] Manual Add -> continue.\n", __FILE__, __LINE__, old_iter, new_iter ) ;
       #endif

       /*-------------------------------------------------*/
       /* Set network PID */
       /*-------------------------------------------------*/
       if ( data_r . SetFlag ) {
         if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {
           #ifdef DEBUG_FAIL
           printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
           #endif
         }
       }
       continue ;
     }

     #ifdef DEBUG
     printf ( "%s : %d : [ old %d ], [ new %d ] Not Manual Add\n", 
              __FILE__, __LINE__,  old_iter, new_iter ) ;
     #endif

     /*-------------------------------------------------*/
     /*  */
     /*-------------------------------------------------*/
     if ( ( data_r . DataPID != si_p -> DataInfo [ new_iter ] . DataPID ) ||
          ( data_r . Encapsulation != si_p -> DataInfo [ new_iter ] . Encapsulation ) ) {

       data_r . DataPID = si_p -> DataInfo [ new_iter ] . DataPID ; 
       data_r . Encapsulation = si_p -> DataInfo [ new_iter ] . Encapsulation ; 

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ new_iter ] . DataPID, si_p -> DataInfo [ new_iter ] . Encapsulation ) ; 
       #endif

       if ( Sky_UpdateData ( data_r ) != FALSE ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_UpdateData () Fail.\n", __FILE__, __LINE__ ) ;
         #endif

         new_iter ++ ;
         continue ;
       } 

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] Sky_UpdateData () succ. DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ new_iter ] . DataPID, si_p -> DataInfo [ new_iter ] . Encapsulation ) ; 
       #endif
     }

     if ( data_r . SetFlag == 0 ) {

       #ifdef DEBUG
       printf ( "%s : %d : [ old %d ], [ new %d ] SetFlag == 0. DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
                __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
                si_p -> DataInfo [ new_iter ] . DataPID, si_p -> DataInfo [ new_iter ] . Encapsulation ) ; 
       #endif

       new_iter ++ ;
       continue ;
     }

     #ifdef DEBUG
     printf ( "%s : %d : [ old %d ], [ new %d ] SetFlag == 1 . DataNumber = %d, Different Pid [ %d ] [ %d ] or Type [ %d ] [ %d ]\n",
              __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, data_r.DataPID, data_r.Encapsulation,  
              si_p -> DataInfo [ new_iter ] . DataPID, si_p -> DataInfo [ new_iter ] . Encapsulation ) ; 
     #endif

     /*-------------------------------------------------*/
     /* Make Message */
     /*-------------------------------------------------*/
     memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
     setdata_r . DataPID       = data_r . DataPID ;
     setdata_r . Encapsulation = data_r . Encapsulation ;

     #ifdef DEBUG
     printf ( "%s : %d : [ old %d ], [ new %d ] Before Sky_SetNetworkPID (). DataNumber = %d, PID = %d, Type = %d\n",
              __FILE__, __LINE__,  old_iter, new_iter, data_r.DataNumber, setdata_r.DataPID, setdata_r.Encapsulation ) ;  
     #endif

     /*-------------------------------------------------*/
     /* Set network PID */
     /*-------------------------------------------------*/
     if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
       #endif

     }

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_SetNetworkPID () succ. Number = %d, PID = %d\n", __FILE__, __LINE__, data_r .DataNumber, data_r.DataPID ) ;
     #endif

     new_iter ++ ;
  }

  if ( find == APP_SCAN_DELETE_E ) { /* numData > si_numData */

    #ifdef DEBUG
    printf ( "APP_SCAN_DELETE_E\n" );
    #endif

    for ( ; old_iter < numData ; old_iter ++ ) {

      data_r.DataNumber = si_p -> TrInfo . TransponderNumber * 100 + old_iter + 1 ;

      #ifdef DEBUG
      printf ( "%s : %d : [ old %d ] Before GetInfoData (). DataNumber = %d\n", 
               __FILE__, __LINE__, old_iter, data_r.DataNumber ) ; 
      #endif

      if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_GetInfoData () Fail.\n", __FILE__, __LINE__ ) ;
         #endif

         continue ;
       }
  
       /*-------------------------------------------------*/
       /* Manual Add -> Not Delete */
       /*-------------------------------------------------*/
       if ( data_r . ManualAdd ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Manual Add Flag Set -> continue.\n", __FILE__, __LINE__ ) ;
         #endif

         /*-------------------------------------------------*/
         /* Set network PID */
         /*-------------------------------------------------*/
         if ( data_r . SetFlag ) {
           if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

             #ifdef DEBUG_FAIL
             printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
             #endif
           }
         }
         continue ;
       }

       #ifdef DEBUG_FAIL
       printf ( "%s : %d : Not Manual Add.\n", __FILE__, __LINE__ ) ;
       #endif

       /*-------------------------------------------------*/
       /* Delete Data PID */
       /*-------------------------------------------------*/
       if ( Sky_DeleteData ( data_r ) != FALSE ) {
  
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_DeleteData () fail\n", __FILE__, __LINE__ ) ;
         #endif

       }
    }
  } else if ( find == APP_SCAN_ADD_E ) {

    #ifdef DEBUG
    printf ( "Add : new_iter = %d\n", new_iter ) ;
    #endif

    for ( ; new_iter < si_numData ; new_iter ++ ) {
       /*-------------------------------------------------*/
       /* Add Data PID */
       /*-------------------------------------------------*/
       if ( Sky_AddData ( si_p -> TrInfo . TransponderNumber, 
                          & si_p -> DataInfo [ new_iter ] ) != FALSE ) {
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_AddData () fail\n", __FILE__, __LINE__ ) ;
         #endif

         continue ;
       }

       #ifdef DEBUG
       printf ( "%s : %d : Sky_AddData () Succ. DataNumber = %d\n", __FILE__, __LINE__, si_p -> DataInfo [ new_iter ].DataNumber ) ;
       #endif

       if ( si_p -> DataInfo [ new_iter ] . SetFlag == 0 ) {
         continue ;
       }

       /*-------------------------------------------------*/
       /* Make Message */
       /*-------------------------------------------------*/
       memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
       setdata_r . DataPID       = si_p -> DataInfo [ new_iter ] . DataPID ;
       setdata_r . Encapsulation = si_p -> DataInfo [ new_iter ] . Encapsulation ;
       setdata_r . pgnumber      = si_p -> DataInfo [ new_iter ] . pgnumber ;
       setdata_r . PmtPID        = si_p -> DataInfo [ new_iter ] . PmtPID ;

       /*-------------------------------------------------*/
       /* Set Data PID */
       /*-------------------------------------------------*/
       if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {
  
         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
         #endif

       }
    }
  } 

}

