/* 
 * Source Name : sky_autoscan.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_autoscan.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void Trans3Lister_Reset ( void ) ;
rcl_return_code Trans3Lister_Add ( int newId, char * );
void AutoScanProgressBar_Reset ( void ) ;
void AutoScanSatelliteDisplay (  XtPointer x, XtIntervalId * id ) ;
void AutoScanResult_Reset ( void ) ;
void AutoScanResult_SetFlag ( void ) ;

void IntervalID_Reset ( void )  ;
void IntervalID_Set ( XtIntervalId newId )  ;
XtIntervalId IntervalID_Get ( void )  ;
void List_SelectedScrolled ( Widget w, int pos, int page ) ;

Sky_ReturnVal_E AutoScanResult_FileOpen ( void ) ;
Sky_ReturnVal_E AutoScanResult_FileClose ( void ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
autoscan_popupCallback ( Widget w, XtPointer client_data, XtPointer call_data ) ;
void
autoscan_popdownCallback ( Widget w, XtPointer client_data, XtPointer call_data ) ;
void
autoscan_cancelBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void AutoScanSatellite ( Widget w ) ;
void * ReallyAutoScan ( void * ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static  pid_t   g_childpid ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void AutoScanSatellite ( Widget w ) 
{
  char            satName [ MAX_SATELLITE_NAME_LEN + 1 ] ;
  Sky_LastInfo_S  sky_r ;

  LNB_STRUCTURE   lnb_r ;
  TR_STRUCTURE    tp_p [ MAX_TRANSPONDER_NUM ] ;
  int             numTp ;
  int             i ;
  int             rt ;
  char            txt [ 1024 ] ;

  XmString        xmStr ;
  Sky_AutoScan_Start_S  autoscan_p ;

  XtIntervalId    interval_id ;

  #ifdef DEBUG
  printf ( "%s : %d : AutoScanSatellite ()\n", __FILE__, __LINE__ ) ;
  #endif

  /*------------------------------------------------------------------------*/
  /* Progress Bar Reset */
  /*------------------------------------------------------------------------*/
  AutoScanProgressBar_Reset ( ) ;

  /*------------------------------------------------------------------------*/
  /* Get LNB Info */
  /*------------------------------------------------------------------------*/
  #ifdef DEBUG
  printf ( "%s : %d : before Sky_GetSatelliteInformation ()\n", __FILE__, __LINE__ ) ;
  #endif

  if ( Sky_GetSatelliteInformation ( & lnb_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  #ifdef DEBUG
  printf ( "%s : %d : autoscan satellite = %s\n", __FILE__, __LINE__, lnb_r.SatelliteName ) ;
  #endif

  /*------------------------------------------------------------------------*/
  /* set satellite name */
  /*------------------------------------------------------------------------*/
  /*
  xmStr = ( XmString ) strToXmString ( " " ) ;
  XtVaSetValues ( autoscan_lab, XmNlabelString, xmStr, NULL ) ;
  XmStringFree ( xmStr ) ;
  */
#if 0
  xmStr = ( XmString ) strToXmString ( ( char * ) lnb_r . SatelliteName ) ;
  XtVaSetValues ( autoscan_lab, XmNlabelString, xmStr, NULL ) ;
  XmStringFree ( xmStr ) ;
#else
  sprintf ( txt, "Auto Scan - %s", lnb_r . SatelliteName ) ;
  XtVaSetValues ( autoscan_shell, XmNtitle, txt, 0 ) ;
#endif

  /*------------------------------------------------------------------------*/
  /* Display Transponder List */
  /*------------------------------------------------------------------------*/
  memset ( tp_p, 0, sizeof ( tp_p ) ) ;
 
  #ifdef DEBUG
  printf ( "%s : %d : before transponder list display to autoscan panel\n", __FILE__, __LINE__ ) ;
  #endif

  Trans3Lister_Reset ( ) ;
  XmListDeleteAllItems ( autoscan_listLister ) ;
  numTp = Sky_GetNumOfTransponder ( ) ;

  #ifdef DEBUG
  printf ( "%s : %d : Number of Transponder = %d\n", __FILE__, __LINE__, numTp ) ;
  printf ( "----------------------------------------------------------------\n" ) ;
  #endif

  for ( i = 0 ; i < numTp ; i ++ ) {

    #ifdef DEBUG
    printf ( "%s : %d : <%d> before Sky_GetInfoTransponder () \n", __FILE__, __LINE__, i ) ;
    printf ( "----------------------------------------------------------------\n" ) ;
    #endif

    if ( Sky_GetInfoTransponder ( i + 1, & tp_p [ i ] ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
    }

    #ifdef DEBUG
    printf ( "%s : %d : <%d> after Sky_GetInfoTransponder () succ\n", __FILE__, __LINE__, i ) ;
    #endif

    /*----------------------------------------------------------------------*/
    /* Display Transponder Info to Import_shell */
    /*----------------------------------------------------------------------*/
    Trans3List_Format ( i + 1, tp_p [ i ] . DownlinkFrequency,
                        tp_p [ i ] . SymbolRate,
                        txt ) ;

    #ifdef DEBUG
    printf ( "%s : %d : <#%d> list format = %s\n", __FILE__, __LINE__, i, txt ) ;
    #endif

    Trans3Lister_Add ( i + 1, txt ) ;

    #ifdef DEBUG
    printf ( "%s : %d : <#%d> After Trans3Lister_Add ()\n", __FILE__, __LINE__, i ) ;
    #endif

  }

  #ifdef DEBUG
  printf ( "----------------------------------------------------------------\n" ) ;
  #endif

  /*------------------------------------------------------------------------*/
  /* AutoScan Result Global variable reset */
  /*------------------------------------------------------------------------*/
  /*AutoScanResult_Reset ( ) ;*/

  /*----------------------------------------------------------------------*/
  /* lock test   transponder display */
  /*----------------------------------------------------------------------*/
  if ( i ) {
    XmListSelectPos ( autoscan_listLister, 1, False ) ;
    Trans3List_Format_3 ( 1, 
                          tp_p [ 0 ] . DownlinkFrequency,
                          tp_p [ 0 ] . SymbolRate,
                          txt ) ;
    xmStr = ( XmString ) strToXmString ( txt ) ;
    XtVaSetValues ( autoscan_stsLab1, XmNlabelString, xmStr, NULL ) ; 
    XmStringFree ( xmStr ) ;
  }

  #ifdef DEBUG
  printf ( "%s : %d : Auto scan start\n", __FILE__, __LINE__ ) ;
  printf ( "Number of Transponder = %d\n", numTp ) ;
  #endif

  /*------------------------------------------------------------------------*/
  /* Lock Test & Scanning */
  /*------------------------------------------------------------------------*/
  autoscan_p.count = numTp ;
  memcpy ( & autoscan_p.lnb_r, & lnb_r, sizeof ( LNB_STRUCTURE ) ) ;
  memcpy ( & autoscan_p.tp_p, tp_p, sizeof ( tp_p ) ) ;

  /* ignore sig_child */
  signal ( SIGCLD, SIG_IGN ) ;

  g_childpid = fork () ;
  if ( g_childpid > 0 ) {

    #ifdef DEBUG
    printf ( "%s : %d : fork () succ. child pid = %d\n", __FILE__, __LINE__, g_childpid ) ;
    #endif

  } else if ( g_childpid == 0 ) {
    ReallyAutoScan ( ( void * ) & autoscan_p ) ;
    exit ( 0 ) ;
  } else {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : fork () fail. errno = %d\n", __FILE__, __LINE__, errno ) ;
    #endif

    return ;
  }

  /*------------------------------------------------------------------------*/
  /* Open AutoScan File */
  /*------------------------------------------------------------------------*/
  while ( 1 ) {
    if ( AutoScanResult_FileOpen ( ) != SKY_SUCCESS ) {
      sleep ( 1 ) ;
      continue ;
    } else {
      break ;
    }
  }

  /*------------------------------------------------------------------------*/
  /* SET TIMER */
  /*------------------------------------------------------------------------*/
  IntervalID_Reset ( )  ;
  interval_id = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), SKY_AUTOSCAN_TIMEOUT, & AutoScanSatelliteDisplay, NULL ) ;
  IntervalID_Set ( interval_id )  ;

}

void
autoscan_popupCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  XmString   xmStr ;

#if 0
  XtSetSensitive ( data_shell, False ) ;
#endif

  /*
  xmStr = ( XmString ) strToXmString ( "Ok" ) ;
  XtVaSetValues ( autoscan_cancelB, XmNlabelString, xmStr, NULL ) ;
  XmStringFree ( xmStr ) ;
  */

}
void
autoscan_popdownCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  Sky_LastInfo_S sky_r ;
  int            tmp ;

  XtIntervalId   id = IntervalID_Get ( )  ;

  #ifdef DEBUG
  printf ( "%s : %d : Send SIGUSR1 to %d\n", __FILE__, __LINE__, g_childpid ) ;
  #endif

  kill ( g_childpid, SIGUSR1 ) ;

  if ( id > 0 ) {

    #ifdef DEBUG
    printf ( "%s : %d : before XtRemoveTimeOut ( %d )\n", __FILE__, __LINE__, id ) ;
    #endif

    XtRemoveTimeOut ( id ) ;

    #ifdef DEBUG
    printf ( "%s : %d : after XtRemoveTimeOut ( %d )\n", __FILE__, __LINE__, id ) ;
    #endif

  }

  IntervalID_Reset ( )  ;

  AutoScanResult_FileClose ( ) ;
  AutoScanResult_Reset ( ) ;

  read_initfile ( & sky_r ) ;

#ifdef LINUX_VERSION
  List_SelectedScrolled ( tuner_satinfo_FqSyList_list, sky_r . transponder_id, 5 ) ;
#else
  List_SelectedScrolled ( tuner_satinfo_FqSyList_list, sky_r . transponder_id, 5 ) ;
#endif
}

void
autoscan_cancelBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  #ifdef DEBUG
  printf ( "%s : %d : autoscan_cancelBCallback ()\n", __FILE__, __LINE__ ) ;
  #endif
}

