/* 
 * Source Name : sky_autoscan_display.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_autoscan.h"
#include "sky_info.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void AutoScanProgressBar_Progress ( int cur_count, int tot_count ) ;
void AutoScanResult_Get ( Sky_AutoScan_Result_S * result_p ) ;
void List_SelectedScrolled ( Widget w, int pos, int page ) ;
void LockStatus_SetString ( char * txt ) ;
Sky_ReturnVal_E AutoScanResult_FileClose ( void ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void AutoScanSatelliteDisplay (  XtPointer x, XtIntervalId * id )  ;

void IntervalID_Reset ( void )  ;
void IntervalID_Set ( XtIntervalId newId )  ;
XtIntervalId IntervalID_Get ( void )  ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static XtIntervalId g_interval_id = 0 ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void AutoScanSatelliteDisplay (  XtPointer x, XtIntervalId * id )  
{
  int            i ;
  int            rt ;
  SI_STRUCTURE   si_r ;

  XmString       xmStr ;
  char           txt [ 1024 ] ;

  XtIntervalId           intervalId = 0 ;
  Sky_AutoScan_Result_S  result_r ;

  static  int            sequence = 0 ;

  AutoScanResult_Get ( & result_r ) ;

  if ( result_r . flag == AUTOSCAN_NOT_STARTED ) {

    intervalId = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), 
                                   SKY_AUTOSCAN_TIMEOUT, 
                                   & AutoScanSatelliteDisplay, NULL ) ;
    IntervalID_Set ( intervalId )  ;

    return ;

  } else if ( result_r . flag == AUTOSCAN_CONTINUED_E ) {

    intervalId = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), 
                                   SKY_AUTOSCAN_TIMEOUT, 
                                   & AutoScanSatelliteDisplay, NULL ) ;
    IntervalID_Set ( intervalId )  ;
  } 

  #ifdef DEBUG
  /*
  printf ( "%s : %d : selected transponder position = %d\n", 
           __FILE__, __LINE__, result_r . sts . next_selected_pos ) ;
  printf ( "%s : %d : result transponder position = %d\n", 
           __FILE__, __LINE__, result_r . sts . result_pos ) ;
  printf ( "%s : %d : result transponder string = %s\n", 
           __FILE__, __LINE__, result_r . sts . result_tp_str ) ;
  printf ( "%s : %d : next scanning string = %s\n", 
           __FILE__, __LINE__, result_r . sts . next_sc_str ) ;
  printf ( "%s : %d : total count = %d\n", 
           __FILE__, __LINE__, result_r . total_bar_count ) ;
  */
  #endif

  /*----------------------------------------------------------------------*/
  /*  Ͽ  ̸ panel update ʴ´. */
  /*----------------------------------------------------------------------*/
  if ( sequence == result_r . sequence ) {
    return ;
  } else {
     sequence = result_r . sequence ;
  }

  /*----------------------------------------------------------------------*/
  /* Lock Result */
  /*----------------------------------------------------------------------*/
  ResetAllNetworkPIDs ( & result_r.sts. si_info ) ;

  /*----------------------------------------------------------------------*/
  /* Display To Title */
  /*----------------------------------------------------------------------*/
  LockStatus_SetString ( result_r . sts . result_title_str ) ;

  /*----------------------------------------------------------------------*/
  /* Select i-th transponder */
  /*----------------------------------------------------------------------*/
  /* transponder list */
  xmStr = ( XmString ) strToXmString ( result_r . sts . result_tp_str ) ;
  XmListReplaceItemsPos ( autoscan_listLister, & xmStr, 1, result_r . sts . result_pos ) ;

  /* select transponder list */
#ifdef LINUX_VERSION
  List_SelectedScrolled ( autoscan_listLister, result_r . sts . next_selected_pos, 15 ) ;
  /*XmListSelectPos ( autoscan_listLister, result_r . sts . next_selected_pos, False ) ;*/
#else
  List_SelectedScrolled ( autoscan_listLister, result_r . sts . next_selected_pos, 13 ) ;
#endif

  /* scanning string */
  xmStr = ( XmString ) strToXmString ( result_r . sts . next_sc_str ) ;
  XtVaSetValues ( autoscan_stsLab1, XmNlabelString, xmStr, NULL ) ; 
  XmStringFree ( xmStr ) ;

  /*----------------------------------------------------------------------*/
  /* Progress Bar */
  /*----------------------------------------------------------------------*/
  AutoScanProgressBar_Progress ( result_r . sts . result_pos, result_r . total_bar_count ) ;

  /*----------------------------------------------------------------------*/
  /* Finish */
  /*----------------------------------------------------------------------*/
  if ( result_r . flag == AUTOSCAN_FINISHED_E ) {
    DisplayInform2Dialog ( autoscan_shell, SKY_INFORM_AUTOSCAN_FINISHED_E, "Auto Scan Finished" ) ;
    /*
    xmStr = ( XmString ) strToXmString ( "Ok" ) ;
    XtVaSetValues ( autoscan_cancelB, XmNlabelString, xmStr, NULL ) ;
    XmStringFree ( xmStr ) ;
    */
    sequence = 0 ; /* reset */

    AutoScanResult_FileClose ( ) ;
  }
}

void IntervalID_Reset ( void ) 
{
  g_interval_id = 0 ;

  #ifdef DEBUG
  printf ( "%s : %d : IntervalID_Reset () : Interval Id = %d\n", __FILE__, __LINE__, g_interval_id ) ;
  #endif
}
void IntervalID_Set ( XtIntervalId newId ) 
{
  g_interval_id = newId ;

  #ifdef DEBUG
  /*printf ( "%s : %d : Interval Id = %d\n", __FILE__, __LINE__, g_interval_id ) ;*/
  #endif
}
XtIntervalId IntervalID_Get ( void ) 
{
  return g_interval_id ;
}
