/* 
 * Source Name : sky_autoscan_msg.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_autoscan.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void AutoScanResult_Reset ( void ) ;
void AutoScanResult_SetFlag ( void ) ;
Sky_ReturnVal_E AutoScanResult_Set ( int seq, int result_pos, int next_pos, int total_count, char * trans_str, char * scan_str, char * title_str, SI_STRUCTURE * ) ;
void AutoScanResult_Get ( Sky_AutoScan_Result_S * result_p ) ;

Sky_ReturnVal_E AutoScanResult_FileCreate ( void ) ;
Sky_ReturnVal_E AutoScanResult_FileOpen ( void ) ;
Sky_ReturnVal_E AutoScanResult_FileClose ( void ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int                     g_fd ;
static Sky_AutoScan_Result_S * g_autoscan_p ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
Sky_ReturnVal_E AutoScanResult_FileOpen ( void )
{
  char * name = SKY_AUTOSCAN_RESULT_FILE_NAME ;
 
  g_fd = open ( name, O_RDWR ) ;
  if ( g_fd < 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s open fail. errno = %d\n", name, errno ) ;
    #endif

    return SKY_FAIL ;

  } 
  g_autoscan_p = ( Sky_AutoScan_Result_S * ) mmap ( 0, 
                                                    sizeof ( Sky_AutoScan_Result_S ),  
                                                    PROT_READ | PROT_WRITE, 
                                                    MAP_SHARED, 
                                                    g_fd, 
                                                    0 ) ;
  if ( g_autoscan_p == ( Sky_AutoScan_Result_S * ) MAP_FAILED ) {
 
    #ifdef DEBUG_FAIL
    printf ( "%s Mmap fail. errno = %d\n", name, errno ) ;
    #endif

    close ( g_fd ) ;
    return SKY_FAIL ;
  } else {
    return SKY_SUCCESS ;
  }
}
Sky_ReturnVal_E AutoScanResult_FileCreate ( void )
{
  char * name = SKY_AUTOSCAN_RESULT_FILE_NAME ;
  char * buf ; 
 
  g_fd = open ( name, O_CREAT | O_WRONLY, 0644 ) ;
  if ( g_fd < 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s creat fail. errno = %d\n", name, errno ) ;
    #endif

    return SKY_FAIL ;

  } else {
    buf = ( char * ) malloc ( sizeof ( Sky_AutoScan_Result_S ) ) ;
    memset ( buf, 0, sizeof ( Sky_AutoScan_Result_S ) ) ;
    write ( g_fd, buf, sizeof ( Sky_AutoScan_Result_S ) ) ;
    close ( g_fd ) ;
  }

  return AutoScanResult_FileOpen () ;
}
Sky_ReturnVal_E AutoScanResult_FileClose ( void )
{
  munmap ( ( char * ) g_autoscan_p, sizeof ( Sky_AutoScan_Result_S ) ) ;
  close ( g_fd ) ;

  g_autoscan_p = NULL ;
  g_fd = 0 ;

  return SKY_SUCCESS ;
}

void AutoScanResult_Reset ( void )
{
  if ( ( g_autoscan_p != NULL ) && g_fd ) {
    memset ( & g_autoscan_p, 0, sizeof ( Sky_AutoScan_Result_S ) ) ;
  }
}
Sky_ReturnVal_E AutoScanResult_Set ( 
  int    seq,
  int    result_pos,
  int    next_pos,
  int    total_count,
  char * trans_str,
  char * scan_str,
  char * title_str,
  SI_STRUCTURE * si_p )
{
  if ( g_autoscan_p -> read_flag != AUTOSCAN_READ_E ) {
    return SKY_FAIL ;
  }

  if ( result_pos >= total_count ) {
    g_autoscan_p -> flag = AUTOSCAN_FINISHED_E ;
  } else {
    g_autoscan_p -> flag = AUTOSCAN_CONTINUED_E ;
  }
  g_autoscan_p -> sequence = seq ;
  g_autoscan_p -> sts .result_pos         = result_pos ;
  strcpy ( g_autoscan_p -> sts . result_tp_str, trans_str ) ;
  strcpy ( g_autoscan_p -> sts . next_sc_str, scan_str ) ;
  strcpy ( g_autoscan_p -> sts . result_title_str, title_str ) ;
  g_autoscan_p -> sts .next_selected_pos  = next_pos ;
  memcpy ( & g_autoscan_p -> sts .si_info, si_p, sizeof ( SI_STRUCTURE ) ) ;
  g_autoscan_p -> total_bar_count = total_count ;
  g_autoscan_p -> read_flag = AUTOSCAN_NOT_READ_E ;

  #ifdef DEBUG
  printf ( "sequence          = %d\n", g_autoscan_p -> sequence ) ;
  printf ( "flag              = %d\n", g_autoscan_p -> flag ) ;
  printf ( "result_pos        = %d\n", g_autoscan_p -> sts .result_pos ) ;
  printf ( "result_tp_str     = %s\n", g_autoscan_p -> sts .result_tp_str ) ;
  printf ( "next_sc_str       = %s\n", g_autoscan_p -> sts .next_sc_str ) ;
  printf ( "result_title_str  = %s\n", g_autoscan_p -> sts .result_title_str ) ;
  printf ( "next_selected_pos = %d\n", g_autoscan_p -> sts .next_selected_pos ) ;
  printf ( "total_bar_count   = %d\n", g_autoscan_p -> total_bar_count ) ;
  printf ( "read_flag         = %d\n", g_autoscan_p -> read_flag ) ;
  #endif

  return SKY_SUCCESS ;
}
void AutoScanResult_Get ( Sky_AutoScan_Result_S * result_p )
{
  memcpy ( result_p, g_autoscan_p, sizeof ( Sky_AutoScan_Result_S ) ) ;
  g_autoscan_p -> read_flag = AUTOSCAN_READ_E ;
}
void AutoScanResult_SetFlag ( void ) 
{
  g_autoscan_p -> flag = AUTOSCAN_FINISHED_E ;
}
