/* 
 * Source Name : sky_autoscan_thread.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif
#include <stdio.h>
#include <errno.h>
#include <signal.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_autoscan.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void ResetAllNetworkPIDs ( SI_STRUCTURE * si_p ) ;
void Trans3List_Format_2 ( int num, unsigned int freq, unsigned int symRate, int lock, char * txt ) ;
void Trans3List_Format_3 ( int num, unsigned int freq, unsigned int symRate, char * txt ) ;
Sky_ReturnVal_E AutoScanResult_Set ( int seq, int result_pos, int next_pos, int total_count, char * trans_str, char * scan_str, char * title_str, SI_STRUCTURE * ) ;
void LockStatus_GetString ( char * name, TR_STRUCTURE * tp_p, int sts, char * txt ) ;

Sky_ReturnVal_E AutoScanResult_FileCreate ( void ) ;
Sky_ReturnVal_E AutoScanResult_FileClose ( void ) ;

void APP_Sky_ReleaseAllNetworkPID ( int tp_id ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void * ReallyAutoScan ( void * ) ;
void signal_handler ( int sig_no ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void * ReallyAutoScan ( void * arg ) 
{
  int            sequence = 1 ;
  int            i ;
  int            rt ;
  int            next_pos ;
  SI_STRUCTURE   si_r ;

  char           txt1 [ 1024 ] ;
  char           txt2 [ 1024 ] ;
  char           titleStr [ 1024 ] ;

  Sky_LastInfo_S sky_r ;
  Sky_AutoScan_Start_S  * autoscan_p = ( Sky_AutoScan_Start_S * ) arg ;

  #ifdef DEBUG
  printf ( "1\n" ) ; fflush ( stdout ) ;
  printf ( "%s : %d : ReallyAutoScan () . my pid = %d\n", __FILE__, __LINE__, getpid () ) ;
  #endif

  /*------------------------------------------------------------------------*/
  /* Signal Setting */
  /*------------------------------------------------------------------------*/
  signal ( SIGUSR1, signal_handler ) ;

  if ( AutoScanResult_FileCreate ( ) != SKY_SUCCESS ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : AutoScanResult_FileCreate () Fail.\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  for ( i = 0 ; i < autoscan_p -> count ; i ++ ) {
   
    /*----------------------------------------------------------------------*/
    /* Release All -> UpdateData Info */
    /*----------------------------------------------------------------------*/
    /*APP_Sky_ReleaseAllNetworkPID ( autoscan_p -> tp_p [ i ] . TransponderNumber ) ;*/
    if ( Sky_ReleaseAllNetworkPID ( ) != FALSE ) {
      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ReleaseAllNetworkPID () Fail.\n", __FILE__, __LINE__ ) ;
      #endif
    }


    /*----------------------------------------------------------------------*/
    /* Initfile read */
    /*----------------------------------------------------------------------*/
    read_initfile ( & sky_r ) ;

    /*----------------------------------------------------------------------*/
    /* Lock Test */
    /*----------------------------------------------------------------------*/
    rt = Sky_TunerLockTest ( autoscan_p -> lnb_r, autoscan_p -> tp_p [ i ] ) ;

    sky_r . transponder_id = autoscan_p -> tp_p [ i ] . TransponderNumber ;
    write_initfile ( & sky_r ) ;

    if ( rt == SKY_OK ) { /* scanning */

      /*--------------------------------------------------------------------*/
      /* Scanning */
      /*--------------------------------------------------------------------*/
      memset ( & si_r, 0, sizeof ( si_r ) ) ;
      if ( Sky_SearchChannel ( _WAIT, & si_r ) != FALSE ) {

        #ifdef DEBUG_FAIL
        printf ( "%s : %d : Sky_SearchChannel () Fail.\n", __FILE__, __LINE__ ) ;
        #endif

        continue ;

      } else {

        #ifdef DEBUG
        printf ( "%s : %d : before ResetNetworkPIDs().\n", __FILE__, __LINE__ ) ;
        #endif

        /*ResetAllNetworkPIDs ( & si_r ) ;*/
      }
    } else {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_TunerLockTest () Fail. ret = %d\n", __FILE__, __LINE__, rt ) ;
      #endif

    } 

    /*----------------------------------------------------------------------*/
    /*   */
    /*----------------------------------------------------------------------*/
    /*  ġ,  ġ, ü , tranponder list ,  label  */
    /* XmList */
    Trans3List_Format_2 ( i + 1, 
                          autoscan_p -> tp_p [ i ] . DownlinkFrequency,
                          autoscan_p -> tp_p [ i ] . SymbolRate,
                          rt, 
                          txt1 ) ;

    if ( i < ( autoscan_p -> count - 1 ) ) {
      next_pos = i + 2 ;
    } else {
      next_pos = i + 1 ;
    }

    Trans3List_Format_3 ( next_pos,
                          autoscan_p -> tp_p [ next_pos - 1 ] . DownlinkFrequency,
                          autoscan_p -> tp_p [ next_pos - 1 ] . SymbolRate,
                          txt2 ) ;

    /*----------------------------------------------------------------------*/
    /*   to Title */
    /*----------------------------------------------------------------------*/
    LockStatus_GetString ( ( char * ) autoscan_p -> lnb_r.SatelliteName, & autoscan_p -> tp_p [ i ], rt, titleStr ) ;

    while ( 1 ) {
      if ( AutoScanResult_Set ( sequence ++, i + 1, next_pos, autoscan_p -> count, txt1, txt2, titleStr, & si_r ) != SKY_SUCCESS ) {
        sleep ( 1 ) ;
      } else {
        break ;
      }
    }

  }

  AutoScanResult_FileClose ( ) ;
  return ;
}
void signal_handler ( int sig_no ) 
{
  #ifdef DEBUG
  printf ( "%s : %d : signal # %d caught\n",__FILE__, __LINE__, sig_no ) ;
  #endif

  exit ( 0 ) ;
}
