#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>

#include "sky_extern.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void AvailableDataLister_Reset ( void ) ;
rcl_return_code AvailableDataLister_Add ( int newId, char * );
void DataList_Format ( unsigned short pid, unsigned short type, char * txt ) ;
Sky_ReturnVal_E LocalIpAddressInfo_Read ( int * count, unsigned long * osc_val ) ;
void IpAddressLister_Reset ( void ) ;
rcl_return_code IpAddressLister_Add ( int newId );
void DisplayIpAddressList ( int count, unsigned long * ipAddr ) ;

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void InitializeDataSetup ( void ) ;
void set_firmwareVersion ( void ) ;
void set_macAddress ( void ) ;
void set_ipAddress ( unsigned long * ) ;
void set_ipOrMacUsing ( void ) ;
/*void set_dataPid ( void ) ;*/
void set_InitialDataSetupToCard ( unsigned long ipaddress ) ;
void set_driverVersion ( void )  ;
void clear_inDataPid ( void ) ;
void ClearListerItem ( void ) ;

void UnSetDataValChanged ( void )  ;
Sky_ReturnVal_E CheckDataPidValue ( char * str, int * val )  ;
char * getServiceTypeString ( u16 NetworkType ) ;
#if 0
Sky_ReturnVal_E SetDataInfoInBuffer ( NETWORK_PID * data ) ;
#else
Sky_ReturnVal_E SetDataInfoInBuffer ( SETDATA_STRUCTURE * data ) ;
#endif
Sky_ReturnVal_E GetPidFromXmString ( int * pid ) ;
Sky_ReturnVal_E GetIpAddrFromString ( char * txt, IP_ADDRESS * ip ) ;


/**************************************************************************
 * Global Variables
 **************************************************************************/
/*Sky_DataSetup_S    DataInfo_p ;*/
int                DataValChg_g = 0 ;
XmString           SelectedXmString_g = NULL ;
int                SelectedItemLength_g = 0 ;
int                SelectedItemPosition_g = 0 ;

/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/

void InitializeDataSetup ( void ) 
{
   unsigned long   ip ;

   /* ʱ ȭ  */
   
   set_firmwareVersion ( ) ;
   set_driverVersion ( ) ;
   set_macAddress ( ) ;
   set_ipAddress ( & ip ) ;
   set_ipOrMacUsing ( ) ;
   /*set_dataPid ( ) ;     /* read from initfile & Display */
   /*clear_inDataPid ( ) ;*/

   set_InitialDataSetupToCard ( ip ) ; /* set to pc-card */

   UnSetDataValChanged ( ) ;
}

void ReInitializeDataSetup ( void ) 
{
   clear_inDataPid ( ) ;
   set_ipOrMacUsing ( ) ;
   /*set_dataPid ( ) ;     /* read from initfile & Display */
}
void clear_inDataPid ( void ) 
{
   /* DATA PID CLEAR */
   XmTextSetString ( data_PE_form1_DPTxt, "" ) ;

   /* RADIO BOX CLEAR */
   XmToggleButtonGadgetSetState ( data_PE_form1_STRadB1, False, False ) ;
   XmToggleButtonGadgetSetState ( data_PE_form1_STRadB2, False, False ) ;
   XmToggleButtonGadgetSetState ( data_PE_form1_STRadB3, False, False ) ;
   XmToggleButtonGadgetSetState ( data_PE_form1_STRadB4, False, False ) ;
}
void set_firmwareVersion ( void ) 
{
   VERSION_NUM      ver_p ;
   char             txt [ 100 ] ;
#if 1
   XmString         str ;
#endif

   /*-----------------------------------*/
   /* Firmware Version                  */
   /*-----------------------------------*/
   if ( Sky_GetFirmwareVersion ( & ver_p ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetFirmwareVersion () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }

   /* make string */ 
   sprintf ( txt, "%d.%02d", ver_p . HighVersion, ver_p . LowVersion ) ;
   
#if 0
   /* display string */
   XmTextSetString ( data_NC_form1_FVTxt, txt ) ;
#else
   str = ( XmString ) strToXmString ( txt ) ;
   XtVaSetValues ( data_NC_form1_FVLab2, XmNlabelString, str, NULL ) ;
   XmStringFree ( str ) ;
#endif

}
void set_driverVersion ( void ) 
{
   VERSION_NUM      ver_p ;
   char             txt [ 100 ] ;
#if 1
   XmString         str ;
#endif

   /*-----------------------------------*/
   /* Driver Version                    */
   /*-----------------------------------*/
   if ( Sky_GetDriverVersion ( & ver_p ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetDriverVersion () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }

   /* make string */ 
   sprintf ( txt, "%d.%02d", ver_p . HighVersion, ver_p . LowVersion ) ;
   
#if 0
   /* display string */
   XmTextSetString ( data_NC_form1_DVTxt, txt ) ;
#else
   str = ( XmString ) strToXmString ( txt ) ;
   XtVaSetValues ( data_NC_form1_DVLab2, XmNlabelString, str, NULL ) ;
   XmStringFree ( str ) ;
#endif

}
void set_macAddress ( void ) 
{
   MAC_ADDRESS      mac_p ;
   char             txt [ 100 ] ;

#if 1
   XmString         str ;
#endif

   /*-------------------------------------*/
   /* Permanent Address , Station Address */
   /*-------------------------------------*/
   if ( Sky_GetMacAddress ( & mac_p ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetMacAddress () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }

   /* make string */
   memset ( txt, 0, sizeof ( txt ) ) ;
   sprintf ( txt, "%02X-%02X-%02X-%02X-%02X-%02X", mac_p . Macaddress1,
                                                   mac_p . Macaddress2,
                                                   mac_p . Macaddress3,
                                                   mac_p . Macaddress4,
                                                   mac_p . Macaddress5,
                                                   mac_p . Macaddress6 ) ;
#if 0
   /* display string */
   XmTextSetString ( data_NC_form2_PATxt, txt ) ;
   XmTextSetString ( data_NC_form2_SATxt, txt ) ;
#else
   str = ( XmString ) strToXmString ( txt ) ;
   XtVaSetValues ( data_NC_form2_PALab2, XmNlabelString, str, NULL ) ;
   XtVaSetValues ( data_NC_form2_SALab2, XmNlabelString, str, NULL ) ;
   XmStringFree ( str ) ;
#endif
   
}
void set_ipAddress ( unsigned long * ip ) 
{
   struct hostent * HOSTENT ;
   unsigned long    ipaddress ;
   unsigned long    ipList [ MAX_IP_ADDRESS_NUM ] ;
   char             txt [ 100 ] ;
   int              count ;
   int              i ;

   /*-------------------------------------*/
   /* Get HOST NAME                       */
   /*-------------------------------------*/
   if ( gethostname ( txt, sizeof ( txt ) ) < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : gethostname () Fail. Errno = %d\n", __FILE__, __LINE__, errno ) ;
      #endif

      return ;
   }

   #ifdef DEBUG_FAIL
   printf ( "%s : %d : hostname = %s\n", __FILE__, __LINE__, txt ) ;
   #endif

   /*-------------------------------------*/
   /* Get IP Address                      */
   /*-------------------------------------*/
   HOSTENT = ( struct hostent * ) gethostbyname ( txt ) ;
   endhostent ( ) ;
   if ( HOSTENT == NULL ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : gethostbyname () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }
   memcpy ( & ipaddress, HOSTENT -> h_addr, 4 ) ;

   #ifdef DEBUG
   printf ( "IP Address = %08x\n", ipaddress ) ;
   #endif

   /*-------------------------------------*/
   /* make string */
   /*-------------------------------------*/
#ifdef LINUX_VERSION 	/* chhs : 06/09 */ /* hochoi : 99/10/5 */
   sprintf ( txt, "%d.%d.%d.%d", ( int ) ( ( ipaddress >>  0 ) & 0xff ), 
                                 ( int ) ( ( ipaddress >>  8 ) & 0xff ),
                                 ( int ) ( ( ipaddress >> 16 ) & 0xff ),
                                 ( int ) ( ( ipaddress >> 24 ) & 0xff ) ) ;
#else
   sprintf ( txt, "%d.%d.%d.%d", ( int ) ( ( ipaddress >> 24 ) & 0xff ), 
                                 ( int ) ( ( ipaddress >> 16 ) & 0xff ),
                                 ( int ) ( ( ipaddress >> 8 ) & 0xff ),
                                 ( int ) ( ipaddress & 0xff ) ) ;
#endif   

   #ifdef DEBUG
   printf ( "IP Address = %s\n", txt ) ;
   #endif

   /*-------------------------------------*/
   /* display string */
   /*-------------------------------------*/
   XmTextSetString ( data_NC_form2_ipTxt, txt ) ;

   * ip = ipaddress ; /* add 1999.11.23 */

   if ( LocalIpAddressInfo_Read ( & count, ipList ) != SKY_SUCCESS ) {
     ipList [ 0 ] = ipaddress ;
     count = 1 ;
   } else {

     for ( i = 0 ; i < count ; i ++ ) {
       if ( ipaddress == ipList [ i ] ) {
         break ;
       }
     }
     if ( i == count ) {
       ipList [ count ] = ipaddress ;
       count = count + 1 ;
     }
   }

   DisplayIpAddressList ( count, ipList ) ;

}
void set_ipOrMacUsing ( void ) 
{
#if 0
   Sky_LastInfo_S   sky_p ;

   /*-------------------------------------*/
   /* IP or MAC                           */
   /*-------------------------------------*/
   read_initfile ( & sky_p ) ;

   switch ( sky_p . ip_or_mac ) {
      case SKY_NETCARD_MAC :
         XmToggleButtonGadgetSetState ( data_NC_form2_RadB2, True, True ) ;
         break ;
      case SKY_NETCARD_IP :
      default :
         XmToggleButtonGadgetSetState ( data_NC_form2_RadB1, True, True ) ;
         break ;
   }
#endif
}

void set_InitialDataSetupToCard ( unsigned long ipaddress )
{
   struct hostent * HOSTENT ;
   /*unsigned long    ipaddress ;*/
   char             txt [ 100 ] ;
   IP_ADDRESS       ip ;
   Sky_LastInfo_S   sky_p ;
   u8               filter ;
   int              i ;

   int              numData ;
   DATAPROGRAM_STRUCTURE  data ;
   SETDATA_STRUCTURE      setdata ;
   TR_STRUCTURE           tpinfo ;

   #ifdef DEBUG
   printf ( "%s : %d : IP Address = %08x\n", __FILE__, __LINE__, ipaddress ) ;
   #endif

   ip . Ipaddress1 = ( u8 ) ( ipaddress & 0xff ) ;
   ip . Ipaddress2 = ( u8 ) ( ( ipaddress >> 8 ) & 0xff ) ;
   ip . Ipaddress3 = ( u8 ) ( ( ipaddress >> 16 ) & 0xff ) ;
   ip . Ipaddress4 = ( u8 ) ( ( ipaddress >> 24 ) & 0xff ) ;
   
   #ifdef DEBUG
   printf ( "IP Address = %d.%d.%d.%d\n", ip . Ipaddress1,
                                          ip . Ipaddress2,
                                          ip . Ipaddress3,
                                          ip . Ipaddress4 ) ;
   #endif

   
   /*-------------------------------------*/
   /* Set IP Address                      */
   /*-------------------------------------*/
   if ( Sky_SetIPAddress ( ip ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_SetIPAddress () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }
   
   /*-------------------------------------*/
   /* IP or MAC */
   /*-------------------------------------*/
   /* read_initfile ( & sky_p ) ; */
   /* filter = sky_p . ip_or_mac ; */
#if 0
   filter = 1 ;
#else /* For IHUG. add 1999.11.23 */
   filter = 0 ;
#endif
   if ( Sky_SetIPorMACfiltering ( & filter ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_SetIPorMACfiltering () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }
}
/******************************************************************
 * Gloval Value Change
 ******************************************************************/
void SetDataValChanged ( void ) 
{
   DataValChg_g = 1 ;
}
void UnSetDataValChanged ( void ) 
{
   DataValChg_g = 0 ;
}
int IsDataValChanged ( void ) 
{
   return DataValChg_g ;
}

/******************************************************************
 * ADD
 ******************************************************************/
void AddDataPid ( void )
{
#if 0
   NETWORK_PID  data ;
#else
   SETDATA_STRUCTURE  data ;
#endif
   char         buf [ 100 ] ;
   int          pid ;
   
   char       * txt ;
   Widget       children [ 5 ] ;
   Arg          args [ 2 ] ;
   int          ac ;
   XmString     xmstr ;

   /*--------------*/
   /* Get Data Pid */
   /*--------------*/
   txt = ( char * ) XmTextGetString ( data_PE_form1_DPTxt ) ;
   if ( *txt == ( char )NULL ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Null String\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }
   strcpy ( buf, txt ) ;
   XtFree ( txt ) ;

   #ifdef DEBUG_FAIL
   printf ( "%s : %d : Add PID = %s\n", __FILE__, __LINE__, buf ) ;
   #endif

   /*-------------------*/
   /* Check Valid PID   */
   /*-------------------*/
   if ( CheckDataPidValue ( buf, & pid ) != SKY_SUCCESS ) {

#if 0
      xmstr = ( XmString ) strToXmString ( "Warning!\nData PID should be given as Hexadecimal value.\nTry again!" ) ;
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
      ac ++ ;
      XtSetValues ( msgWarning_txtlabel, args, ac ) ;
      XmStringFree ( xmstr ) ;

      /* Display Warning Shell */
      ac = 0 ;
      children [ ac ++ ] = msgWarning_pixmapLabel ; 
      children [ ac ++ ] = msgWarning_txtlabel ; 
      children [ ac ++ ] = msgWarning_separ ; 
      children [ ac ++ ] = msgWarning_okB ; 
      XtManageChildren ( children, ac ) ;
      XtManageChild ( msgWarning_board ) ;
#endif
      return ;
   }
   data . DataPID = pid ;
 
   #ifdef DEBUG
   printf ( "%s : %d : Add PID = %d\n", __FILE__, __LINE__, pid ) ;
   #endif

   /*-------------------*/
   /*  Get Service Type */
   /*-------------------*/
   if ( XmToggleButtonGadgetGetState ( data_PE_form1_STRadB1 ) == True ) {
#if 0
      data . NetworkType = SERVICE_TYPE_INT_1 ;
#else
      data . Encapsulation = DATA_PIPING ;
#endif
   } else if ( XmToggleButtonGadgetGetState ( data_PE_form1_STRadB2 ) == True ) {
#if 0
      data . NetworkType = SERVICE_TYPE_INT_2 ;
#else
      data . Encapsulation = DATA_STREAMING ;
#endif
   } else if ( XmToggleButtonGadgetGetState ( data_PE_form1_STRadB3 ) == True ) {
#if 0
      data . NetworkType = SERVICE_TYPE_INT_3 ;
#else
      data . Encapsulation = MULTIPROTOCOL_ENCAPSULATION ;
#endif
   } else if ( XmToggleButtonGadgetGetState ( data_PE_form1_STRadB4 ) == True ) {
#if 0
      data . NetworkType = SERVICE_TYPE_INT_4 ;
#else
      data . Encapsulation = DATA_STREAMING_PES_1 ;
#endif
   } else {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : No Selected Service Type\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   } 

   /*-------------------*/
   /* Add to Buffer     */
   /*-------------------*/
   /* check same pid : fixed ... or variabled */
   /* if exist same pid, return SKY_FAIL */
   if ( SetDataInfoInBuffer ( & data ) != SKY_SUCCESS ) {

      #ifdef DEBUG_FAIL
      printf ( "Invalid Pid = %d\n", data . DataPID ) ; 
      #endif

      return ;
   }

   #ifdef DEBUG
   printf ( "%s : %d : Add PID = %d\n", __FILE__, __LINE__, data . DataPID ) ;
#if 0
   printf ( "%s : %d : Add TYPE = %d\n", __FILE__, __LINE__, data . NetworkType ) ;
#else
   printf ( "%s : %d : Add TYPE = %d\n", __FILE__, __LINE__, data . Encapsulation ) ;
#endif
   #endif
   /*-------------------*/
   /* Add to Lister     */
   /*-------------------*/
   /* 123456789012345678 */
                /* 1234          2345678 */
#if 0
   sprintf ( buf, " %4X          %s", data . DataPID, getServiceTypeString ( data . NetworkType ) ) ;
#else
   sprintf ( buf, " %4X          %s", data . DataPID, getServiceTypeString ( data . Encapsulation ) ) ;
#endif

   xmstr = ( XmString ) strToXmString ( buf ) ;
   XmListAddItem ( data_PE_form3_list_lister, xmstr, 0 ) ;
   
}
Sky_ReturnVal_E CheckDataPidValue ( char * str, int * val ) 
{
   int    size ;
   int    i ;

   size = strlen ( str ) ;

   if ( size > 4 ) {
      #ifdef DEBUG_FAIL
      printf ( "%s : So Long Pid\n", str ) ;
      #endif

      return SKY_FAIL ;
   }

   for ( i = 0 ; i < size ; i ++ ) {
      if ( ( ( str [ i ] >= '0' ) && ( str [ i ] <= '9' ) ) ||
           ( ( str [ i ] >= 'A' ) && ( str [ i ] <= 'F' ) ) ||
           ( ( str [ i ] >= 'a' ) && ( str [ i ] <= 'f' ) ) ) {
         continue ;
      } else {

         #ifdef DEBUG_FAIL
         printf ( "%s : Invalid Character(s)\n", str ) ;
         #endif

         return SKY_FAIL ;
      }
   }

   sscanf ( str, "%x", val ) ;
   if ( * val > MAX_DATA_PID_VAL ) {
      #ifdef DEBUG_FAIL
      printf ( "%d : So Big Pid\n", * val ) ;
      #endif

      return SKY_FAIL ;
   }

   #ifdef DEBUGL
   printf ( "%s : %d : Add PID = %d\n", __FILE__, __LINE__, * val ) ;
   #endif

   return SKY_SUCCESS ;
}
#if 0
Sky_ReturnVal_E SetDataInfoInBuffer ( NETWORK_PID * data )
#else
Sky_ReturnVal_E SetDataInfoInBuffer ( SETDATA_STRUCTURE * data )
#endif
{
   int    i ;
   int    find = 0 ;

#ifdef DATA
   for ( i = 0 ; i < DataInfo_p . num_of_fixed ; i ++ ) {

      if ( data -> DataPID == DataInfo_p . data_list [ i ]. DataPID ) {

         if ( DataInfo_p . data_flag [ i ] == SKY_FAV_CH_DELETE ) {

#if 0
            if (  data -> NetworkType == DataInfo_p . data_list [ i ].NetworkType ) {
#else
            if (  data -> Encapsulation == DataInfo_p . data_list [ i ].Encapsulation ) {
#endif

               DataInfo_p . data_flag [ i ] = SKY_FAV_NOT_CHG ;

               #ifdef DEBUG
               printf ( "---DATA INFO 1 VALUE [ %d ]------------------------------------------\n", i ) ;
               printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
               printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
               printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
               printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
               #endif

               return SKY_SUCCESS ;   /* display into lister */

            } else {  /* insert or update */

               #ifdef DEBUG
               printf ( "---DATA INFO 2 VALUE [ %d ]------------------------------------------\n", i ) ;
               printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
               printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
               printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
               printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
               #endif

               break ;
            }
         } else {

            #ifdef DEBUG
            printf ( "---DATA INFO 3 VALUE [ %d ]------------------------------------------\n", i ) ;
            printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
            printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
            printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
            printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
            printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
            printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
            #endif

            return SKY_FAIL ; /* Not display */
         }
      } else {

         #ifdef DEBUG
         printf ( "---DATA INFO 4 VALUE [ %d ]------------------------------------------\n", i ) ;
         printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
         printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
         printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
         printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
         #endif

         continue ;
      }

   }
   for ( i = DataInfo_p . num_of_fixed ; i < DataInfo_p . num_of_data ; i ++ ) {

      if ( data -> DataPID == DataInfo_p . data_list [ i ]. DataPID ) {

         #ifdef DEBUG
         printf ( "---DATA INFO 5 VALUE [ %d ]------------------------------------------\n", i ) ;
         printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
         printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
         printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
         printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
         #endif

         return SKY_FAIL ; /* Not display */
      }
   }

   DataInfo_p . data_flag [ i ] = SKY_FAV_CH_ADD ;
   DataInfo_p . data_list [ i ] . DataPID     = data -> DataPID ;
#if 0
   DataInfo_p . data_list [ i ] . NetworkType = data -> NetworkType ;
#else
   DataInfo_p . data_list [ i ] . Encapsulation = data -> Encapsulation ;
#endif
   DataInfo_p . num_of_data ++ ;

   #ifdef DEBUG
   printf ( "---DATA INFO 6 VALUE [ %d ]------------------------------------------\n", i ) ;
   printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
   printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
   printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
   printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
   printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
   printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
   #endif
#endif

   return SKY_SUCCESS ;
}
char * getServiceTypeString ( u16 NetworkType ) 
{
   static char txt [ 100 ] ;

#if 0
   switch ( NetworkType ) {
      case SERVICE_TYPE_INT_2 :
         strcpy ( txt, SERVICE_TYPE_2 ) ; break ;
      case SERVICE_TYPE_INT_3 :
         strcpy ( txt, SERVICE_TYPE_3 ) ; break ;
      case SERVICE_TYPE_INT_4 :
         strcpy ( txt, SERVICE_TYPE_4 ) ; break ;
      case SERVICE_TYPE_INT_1 :
      default :
         strcpy ( txt, SERVICE_TYPE_1 ) ; break ;
   }
#else
   switch ( NetworkType ) {
      case DATA_STREAMING :
         strcpy ( txt, SERVICE_TYPE_2 ) ; break ;
      case MULTIPROTOCOL_ENCAPSULATION :
         strcpy ( txt, SERVICE_TYPE_3 ) ; break ;
      case DATA_STREAMING_PES_1 :
         strcpy ( txt, SERVICE_TYPE_4 ) ; break ;
      case DATA_PIPING :
      default :
         strcpy ( txt, SERVICE_TYPE_1 ) ; break ;
   }
#endif
   return txt ;
}
/******************************************************************
 * REMOVE
 ******************************************************************/
void RemoveDataPid ( void ) 
{
   int  position ;
   int  i ;
   int  j ;
   int  pid ;
   int  find = 0 ;
#ifdef DATA
   /* Get Data PID */
   if ( GetPidFromXmString ( & pid ) != SKY_SUCCESS ) {
      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Invalid PID\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   } 

   /* Delete from Buffer */
   for ( i = 0 ; i < DataInfo_p . num_of_fixed ; i ++ ) {
      if ( ( pid == DataInfo_p . data_list [ i ] . DataPID ) && 
           ( DataInfo_p . data_flag [ i ] != SKY_FAV_CH_DELETE ) ) {
         DataInfo_p . data_flag [ i ] = SKY_FAV_CH_DELETE  ;
         find = 1 ;

         #ifdef DEBUG
         printf ( "---DATA INFO 7 VALUE [ %d ]------------------------------------------\n", i ) ;
         printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
         printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
         printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
         printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
         #endif

         break ;
      }
   }
   
   for ( i = DataInfo_p . num_of_fixed ; i < DataInfo_p . num_of_data ; i ++ ) {
      if ( ( pid == DataInfo_p . data_list [ i ] . DataPID ) && 
           ( DataInfo_p . data_flag [ i ] == SKY_FAV_CH_ADD ) ) {
         DataInfo_p . data_flag [ i ] = 0 ;
         DataInfo_p . data_list [ i ] . DataPID= 0 ;
#if 0
         DataInfo_p . data_list [ i ] . NetworkType= 0 ;
#else
         DataInfo_p . data_list [ i ] . Encapsulation = 0 ;
#endif

         #ifdef DEBUG
         printf ( "---DATA INFO 8 VALUE [ %d ]------------------------------------------\n", i ) ;
         printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
         printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
         printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
         printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
         printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
         #endif

         if ( i < ( DataInfo_p . num_of_data - 1 ) ) {

            #ifdef DEBUG
            for ( j = 0 ; j < DataInfo_p . num_of_data ; j ++ ) {
               printf ( "---DATA INFO 9 VALUE [ %d ]------------------------------------------\n", i ) ;
               printf ( "[ %d ].flag      = %d\n", i, DataInfo_p . data_flag [ i ] ) ;
#if 0
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
               printf ( "[ %d ].type      = %d\n", i, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
               printf ( "[ %d ].pid       = 0x%x\n", i, DataInfo_p . data_list [ i ] . DataPID ) ;
               printf ( "[ %d ].num_fixed = %d\n", i, DataInfo_p . num_of_fixed ) ;
               printf ( "[ %d ].num_data  = %d\n", i, DataInfo_p . num_of_data ) ;
            }
            #endif

#if 0
            memmove ( & DataInfo_p . data_list [ i ], 
                      & DataInfo_p . data_list [ i + 1 ],
                      sizeof ( NETWORK_PID ) * ( DataInfo_p . num_of_data - i - 1 ) ) ;
#else
            memmove ( & DataInfo_p . data_list [ i ], 
                      & DataInfo_p . data_list [ i + 1 ],
                      sizeof ( SETDATA_STRUCTURE ) * ( DataInfo_p . num_of_data - i - 1 ) ) ;
#endif

            #ifdef DEBUG
            for ( j = 0 ; j < DataInfo_p . num_of_data ; j ++ ) {
               printf ( "---DATA INFO 10 VALUE [ %d ]------------------------------------------\n", i ) ;
               printf ( "[ %d ].flag      = %d\n", j, DataInfo_p . data_flag [ i ] ) ;
#if 0
               printf ( "[ %d ].type      = %d\n", j, DataInfo_p . data_list [ i ] . NetworkType ) ;
#else
               printf ( "[ %d ].type      = %d\n", j, DataInfo_p . data_list [ i ] . Encapsulation ) ;
#endif
               printf ( "[ %d ].pid       = 0x%x\n", j, DataInfo_p . data_list [ i ] . DataPID ) ;
               printf ( "[ %d ].num_fixed = %d\n", j, DataInfo_p . num_of_fixed ) ;
               printf ( "[ %d ].num_data  = %d\n", j, DataInfo_p . num_of_data ) ;
            }
            #endif
         }
         find = 1 ;
         break ;
      }
   }

   if ( find ) {

      /* delete from lister */   
      if ( ( position = GetSelectedItemPosition () ) != 0 ) {
         XmListDeletePos ( data_PE_form3_list_lister, position ) ;
      }
   }
   ClearListerItem ( ) ;
#endif
}
/******************************************************************
 * Save Lister Item
 ******************************************************************/
void SaveListerItem ( int position, int len, XmString xmstr ) 
{
   SelectedItemPosition_g = position ;
   SelectedItemLength_g = len ;
   SelectedXmString_g = XmStringNCopy ( xmstr, len ) ; 
}
void ClearListerItem ( void )
{
   SelectedItemPosition_g = 0 ;
   SelectedItemLength_g = 0 ;
   if ( SelectedXmString_g != NULL ) {
      XmStringFree ( SelectedXmString_g ) ;
      SelectedXmString_g = NULL ;
   }
}
int GetSelectedItemPosition ( void )
{
   return SelectedItemPosition_g ;
}
void GetSelectedItemString ( XmString * xmstr )
{
   * xmstr = XmStringNCopy ( SelectedXmString_g, SelectedItemLength_g ) ;
}
Sky_ReturnVal_E GetPidFromXmString ( int * pid ) 
{
   char   *   txt ;
   Boolean    ret ;
   char       buf [ 100 ] ;

   ret = XmStringGetLtoR ( SelectedXmString_g, XmFONTLIST_DEFAULT_TAG, & txt ) ;
   if ( ret != True ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : XmStringGetLtoR () fail\n", __FILE__, __LINE__ ) ;
      #endif

      return SKY_FAIL ;
   }
    
   sscanf ( txt, "%x %s", pid, buf ) ;
   XtFree ( txt ) ;

   #ifdef DEBUG
   printf ( "%s : %d : DataPid = 0x%X, Data = %s\n", __FILE__, __LINE__, * pid, buf ) ;
   #endif

   return SKY_SUCCESS ;
}
Sky_ReturnVal_E GetIpAddrFromString ( char * txt, IP_ADDRESS * ip ) 
{
   int    ipaddr [ 4 ] ;
#if 0	/* chhs : 06/09 */
   char   buf [ 100 ] ;
#else
   char   buf [ 50 ] ;
   char   buf2 [ 50 ] ;
#endif
   int    x ;
   int    i ;
   int    j ;
   

#if 1	/* chhs : 06/09 */
   strcpy ( buf2, txt );
   strcat ( buf2, "." );
#endif

   #ifdef DEBUG
   printf("[GetIpAddrFromString] buf2 = %s\n", buf2);
   #endif

   memset ( ipaddr, 0, sizeof ( ipaddr ) ) ;

   i = 0 ;
   j = 0 ;
   x = 0 ;
#if 0	/* chhs : 06/09 */
   for ( i = 0 ; txt [ i ] != 0 ; i ++ ) { 
#else
   for ( i = 0 ; buf2 [ i ] != 0 ; i ++ ) { 
#endif

      if ( x >= 4 ) {
         break ;
      }
#if 0	/* chhs : 06/09 */
      if ( txt [ i ] == '.' ) {
         buf [ j ] = 0 ;
         ipaddr [ x ++ ] = atoi ( buf ) ;
         j = 0 ;
      } else if ( ( txt [ i ] >= '0' ) && ( txt [ i ] <= '9' ) ) {
         buf [ j ++ ] = txt [ i ] ;
      } else {
         return SKY_FAIL ;
      } 
#else
      if ( buf2 [ i ] == '.' ) {
         buf [ j ] = 0 ;
         ipaddr [ x ++ ] = atoi ( buf ) ;
         j = 0 ;
      } else if ( ( buf2 [ i ] >= '0' ) && ( buf2 [ i ] <= '9' ) ) {
         buf [ j ++ ] = buf2 [ i ] ;
      } else {
         return SKY_FAIL ;
      } 
#endif
   }

   ip -> Ipaddress1 = ipaddr [ 0 ] ;
   ip -> Ipaddress2 = ipaddr [ 1 ] ;
   ip -> Ipaddress3 = ipaddr [ 2 ] ;
   ip -> Ipaddress4 = ipaddr [ 3 ] ;

   return SKY_SUCCESS ;
}

/* Cancel BUTTON */
void RecoverDataInfo ( void )
{
   ClearListerItem ( ) ;
   UnSetDataValChanged ( )  ;
#ifdef DATA
   memset ( & DataInfo_p, 0, sizeof ( DataInfo_p ) ) ;
#endif
}
