/* 
 * Source Name : sky_data_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_data.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void AvailableData_SetCurrentAvailableDataId ( int newId ) ;
int AvailableData_GetCurrentAvailableDataId ( void ) ;

void AvailableDataLister_Reset ( void ) ;
rcl_return_code AvailableDataLister_Add ( int newId, char * );
rcl_return_code AvailableDataLister_Remove ( int newId );
rcl_return_code AvailableDataLister_GetPosById ( int newId, int * pos );
rcl_return_code AvailableDataLister_GetIdByPos ( int pos, int * newId ) ;
void DataList_Format ( unsigned short pid, unsigned short type, char * txt ) ;

rcl_return_code AvailableDataList_SetNotSelected ( int numPos, int * pos_list ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int               current_availabledata_id_g = 0 ;
static AvailableDataList_S     * availabledataList_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentAvailableDataId ( int newId )
{
   current_availabledata_id_g = newId ;
}
int GetCurrentAvailableDataId ( void )
{
   return current_availabledata_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void AvailableDataLister_Reset ( void ) 
{
   AvailableDataList_S  *  curr ;
   AvailableDataList_S  *  next ;

   if ( availabledataList_g == NULL ) {
      return ;
   }

   curr = availabledataList_g ;
   next = availabledataList_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   availabledataList_g = NULL ;
}
rcl_return_code AvailableDataLister_Add ( int newId, char * txt )
{
   AvailableDataList_S  ** curr ;
   XmString        xmstr ;

   #ifdef DEBUG
   int             count = 0 ;
   #endif

   #ifdef DEBUG
   printf ( "%s : %d : AvailableDataLister_Add () : new id = %d\n", __FILE__, __LINE__, newId ) ;
   #endif

   xmstr = ( XmString ) strToXmString ( txt ) ;
 
   if ( availabledataList_g == NULL ) {
      availabledataList_g = ( AvailableDataList_S * ) malloc ( sizeof ( AvailableDataList_S ) ) ;
      availabledataList_g -> id = newId ;
      availabledataList_g -> next = NULL ;

      XmListAddItem ( data_PE_form1_listLister , xmstr, 0 ) ;
      XmStringFree ( xmstr ) ;

      #ifdef DEBUG
      printf ( "%s : %d : #1 Add  () : new id = %d\n", __FILE__, __LINE__, newId ) ;
      #endif

      return RCL_SUCCESS ;
   }

   curr = & availabledataList_g ;
   while ( 1 ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( AvailableDataList_S * ) malloc ( sizeof ( AvailableDataList_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         XmListAddItem (  data_PE_form1_listLister, xmstr, 0 ) ;
         XmStringFree ( xmstr ) ;

         #ifdef DEBUG
         printf ( "%s : %d : #%d Add  () : new id = %d\n", __FILE__, __LINE__, count, newId ) ;
         #endif

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;

      #ifdef DEBUG
      count ++  ;
      #endif
   }
   return RCL_SUCCESS ;
}
rcl_return_code AvailableDataLister_Remove ( int newId )
{
   AvailableDataList_S  ** prev ;
   AvailableDataList_S  ** curr ;
   
   if ( availabledataList_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( availabledataList_g -> next == NULL ) {
      if ( availabledataList_g -> id == newId ) {
         free ( availabledataList_g ) ;
         availabledataList_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & availabledataList_g ;
   curr = & availabledataList_g ;
   while ( ( *curr ) ) {
      if ( ( * curr ) -> id == newId ) {
         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;
         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code AvailableDataLister_GetPosById ( int newId, int * pos )
{
   int               i ;
   AvailableDataList_S  * curr ;
   
   if ( availabledataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = availabledataList_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code AvailableDataLister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   AvailableDataList_S  * curr ;
   
   if ( availabledataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = availabledataList_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}
rcl_return_code AvailableDataList_SetNotSelected ( int numPos, int * pos_list ) 

{
   int               i ;
   AvailableDataList_S  * curr ;
   
   if ( availabledataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   for ( i = 0 ; i < numPos ; i ++ ) {
     XmListDeselectPos (  data_PE_form1_listLister, pos_list [ i ] + 1 ) ; 
   }

   return RCL_SUCCESS ;
}
void DataList_Format ( unsigned short pid, unsigned short type, char * txt )
{
  char   buf [ 100 ] ;

  switch ( type ) {
    case DATA_PIPING : /* data piping */
      strcpy ( buf, SERVICE_TYPE_1 ) ;
      break ;
    case DATA_STREAMING : /* data streaming */
      strcpy ( buf, SERVICE_TYPE_2 ) ;
      break ;
    case MULTIPROTOCOL_ENCAPSULATION : /* multiprotocol encapsulation */
      strcpy ( buf, SERVICE_TYPE_3 ) ;
      break ;
    case DATA_STREAMING_PES_1 : /* data streaming pes 1 */
      strcpy ( buf, SERVICE_TYPE_4 ) ;
      break ;
    case DATA_CAROUSEL : 
      strcpy ( buf, SERVICE_TYPE_5 ) ;
      break ;
    default :
      strcpy ( buf, SERVICE_TYPE_1 ) ;
      break ;
  }

  sprintf ( txt, " %7X   %s", pid, buf ) ;

}

