/* 
 * Source Name : sky_import.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void Trans2Lister_Reset ( void ) ;
rcl_return_code Trans2Lister_Add ( int newId, char * );

void ConvertInt2Str_LocalOscillator ( unsigned int x, char * txt ) ;
void ConvertInt2Str_ToneBurst ( unsigned int ToneBurst, char * txt ) ;
void ConvertInt2Str_SwitchOnOff ( LNB_HILOW SwitchOnOff , char * txt ) ;
void ConvertInt2Str_DiseqcPort ( unsigned char DiseqcPort , char * txt ) ;
void ConvertInt2Str_LNBPower ( LNB_POWER LNBPower , char * txt ) ;

void
data_menu_SATBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
import_okBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
import_cancelBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void ClearImportShell ( void ) ;
void DisplayImportShell ( char * )  ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
void Import_Display_LNB ( char * filename, LNB_STRUCTURE * lnb_p ) ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void DisplayImportShell ( char * filename ) 
{
  LNB_STRUCTURE   lnb_r ;
  TR_STRUCTURE    tp_r ;
  int             numTp ;
  int             i ;
  char            txt [ 1024 ] ;

  /*------------------------------------------------------------------------*/
  /* Clear Import_SHell */
  /*------------------------------------------------------------------------*/
  ClearImportShell ( ) ;

  /*------------------------------------------------------------------------*/
  /* Get LNB information */
  /*------------------------------------------------------------------------*/
  if ( Sky_GetSatelliteInformation ( & lnb_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_GetSatelliteInformation () Fail.\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  Import_Display_LNB ( filename, & lnb_r ) ;

  /*------------------------------------------------------------------------*/
  /* Get Transponder information */
  /*------------------------------------------------------------------------*/
  Trans2Lister_Reset ( ) ;
  numTp = Sky_GetNumOfTransponder ( ) ;
  for ( i = 0 ; i < numTp ; i ++ ) { 
  
    if ( Sky_GetInfoTransponder ( i + 1, & tp_r ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
    }

    /*----------------------------------------------------------------------*/
    /* Display Transponder Info to Import_shell */
    /*----------------------------------------------------------------------*/
    Trans2List_Format ( i + 1, tp_r . DownlinkFrequency,
                        tp_r . SymbolRate,
                        tp_r . SignalType, 
                        txt ) ;
    Trans2Lister_Add ( i + 1, txt ) ;
  }
  if ( i ) {
    XmListSelectPos ( import_transListLister, 1, False ) ;
  }
}

void ClearImportShell ( void ) 
{
  XmTextSetString ( import_satNameT, "" ) ;
  XmTextSetString ( import_fileT, "" ) ;
  XmTextSetString ( import_oscT, "" ) ;
  XmTextSetString ( import_toneT, "" ) ;
  XmTextSetString ( import_22kT, "" ) ;
  XmTextSetString ( import_diseqcT, "" ) ;
  XmTextSetString ( import_lnbT, "" ) ;

  XmListDeleteAllItems ( import_transListLister ) ;
}
void Import_Display_LNB ( char * filename, LNB_STRUCTURE * lnb_p ) 
{
  char     txt [ 100 ] ;

  /* satellite Name */
  XmTextSetString ( import_satNameT, lnb_p -> SatelliteName ) ;

  /* satellite file */
  XmTextSetString ( import_fileT, filename ) ;

  /* local oscillator */
  ConvertInt2Str_LocalOscillator ( lnb_p -> LocalOscillator / 1000, txt ) ;
  XmTextSetString ( import_oscT, txt ) ;

  /* tone burst */
  ConvertInt2Str_ToneBurst ( lnb_p -> ToneBurst, txt ) ;
  XmTextSetString ( import_toneT, txt ) ;

  /* Switch On/off */
  if ( lnb_p -> LocalOscillator == 0 ) {
    XmTextSetString ( import_22kT, NULL ) ;
  } else {
    ConvertInt2Str_SwitchOnOff ( lnb_p -> SwitchOnOff, txt ) ;
    XmTextSetString ( import_22kT, txt ) ;
  }

  /* diseqc */
  ConvertInt2Str_DiseqcPort ( lnb_p -> DiseqcPort, txt ) ;
  XmTextSetString ( import_diseqcT, txt ) ;

  /* lnb power */
  ConvertInt2Str_LNBPower ( lnb_p -> LNBPower, txt ) ;
  XmTextSetString ( import_lnbT, txt ) ;

}

void
import_okBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  Sky_LastInfo_S   sky_r ;
  int              i ;
  char             satName [ MAX_SATELLITE_NAME_LEN ] ;
  char             satFile [ MAX_FILE_NAME_LEN ] ;
  char           * txt ;

  read_initfile ( & sky_r ) ;

  /*-----------------------------------------------------------------------*/
  /* Get Satellite Name */
  /*-----------------------------------------------------------------------*/
  txt = ( char * ) XmTextGetString ( import_satNameT ) ;
  if ( *txt == ( char )NULL ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : XmTextGetString () is null\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  strcpy ( satName, txt ) ;
  XtFree ( txt ) ;

  /*-----------------------------------------------------------------------*/
  /* Get Satellite file */
  /*-----------------------------------------------------------------------*/
  txt = ( char * ) XmTextGetString ( import_fileT ) ;
  if ( *txt == ( char )NULL ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : XmTextGetString () is null\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  strcpy ( satFile, txt ) ;
  XtFree ( txt ) ;

  /*-----------------------------------------------------------------------*/
  /* compare satellite name */
  /*-----------------------------------------------------------------------*/
  for ( i = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
    if ( sky_r . sat_list [ i ] . flag ) {
      if ( ! strcmp ( satName, sky_r . sat_list [ i ] . satellite_name ) ) {
        break ;
      } 
    }
  }
  if ( i == MAX_SATELLITE_NUM ) { /* New Satellite */
    sky_r . num_of_satellite = sky_r . num_of_satellite + 1 ;
    for ( i = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
      if ( sky_r . sat_list [ i ] . flag == 0 ) {
        sky_r . sat_list [ i ] . flag = 1 ;
        strcpy ( sky_r . sat_list [ i ] . satellite_name, satName ) ;
        strcpy ( sky_r . sat_list [ i ] . satellite_file, satFile ) ;
        break ;
      }
    }
  }
  strcpy ( sky_r . satellite_name, satName ) ;
  strcpy ( sky_r . satellite_file, satFile ) ;
  sky_r . transponder_id = XmListGetKbdItemPos ( import_transListLister ) ;

  write_initfile ( & sky_r ) ;

  /*-------------------------------------------------------------------*/
  /* COMback Main Panel */
  /*-------------------------------------------------------------------*/
  data_menu_SATBCallback ( NULL, NULL, NULL ) ;
}

void
import_cancelBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  /* Original Satellite File Open */
  LNB_STRUCTURE   lnb_r ;
  TR_STRUCTURE    tp_r ;
  char            filename [ MAX_FILE_NAME_LEN + 1 ] ;

  /*------------------------------------------------------------------------*/
  /* Get File name */
  /*------------------------------------------------------------------------*/
  if ( GetFileNameFromFileSelection ( filename ) != SKY_SUCCESS ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : GetFileNameFromFileSelection () Fail\n", __FILE__, __LINE__ ) ;
      #endif

      return ;

  }
  /*------------------------------------------------------------------------*/
  /* Open File */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileOpen ( filename ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ChannelFileOpen () Fail. File = %s\n", __FILE__, __LINE__, filename
 ) ;
      #endif

      return ;

   }

  /*------------------------------------------------------------------------*/
  /* Open Read */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileRead ( ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ChannelFileRead () Fail. \n", __FILE__, __LINE__ ) ;
      #endif

      Sky_ChannelFileClose () ;
      return ;
  }

  /*------------------------------------------------------------------------*/
  /* Close */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_ChannelFileClose () Fail.\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------------------------*/
  /* COMback Main Panel */
  /*-------------------------------------------------------------------*/
  data_menu_SATBCallback ( NULL, NULL, NULL ) ;

}


