#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sky_extern.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/

#include "sky_skysub.h"

/* ----------------------------------------------------------------- */
/*                      FUNCTION PROTOTYPES                          */
/* ----------------------------------------------------------------- */
Sky_ReturnVal_E open_initfile ( void ) ;
Sky_ReturnVal_E read_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
Sky_ReturnVal_E write_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
Sky_ReturnVal_E close_initfile ( void ) ;

/* ----------------------------------------------------------------- */
/*                      GLOBAL VARIABLES                             */
/* ----------------------------------------------------------------- */
static int              fd ;
static Sky_LastInfo_S * info_p = ( Sky_LastInfo_S * ) MAP_FAILED ;

Sky_ReturnVal_E open_initfile ( void )
{
   int            i ;


   int            rt ;
   int            fd_tmp ;
   char         * name = INITIAL_INFO_FILE_NAME ;
   struct stat    fileSts ;

   Sky_LastInfo_S * tmp_p ;

   rt = stat ( name, & fileSts ) ;
   if ( rt < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : stat ( %s ) fail. errno = %d\n", __FILE__, __LINE__, name, errno ) ;
      #endif

      /* create file */
      fd_tmp = open ( name, O_CREAT | O_WRONLY, 0644 ) ;
      if ( fd < 0 ) {

        #ifdef DEBUG_FAIL
        printf ( "%s creat fail. errno = %d\n", name, errno ) ;
        #endif

        return SKY_FAIL ;

      } else {
        tmp_p = ( Sky_LastInfo_S * ) malloc ( sizeof ( Sky_LastInfo_S ) ) ;
        memset ( tmp_p, 0, sizeof ( Sky_LastInfo_S ) ) ;
        write ( fd_tmp, ( char * ) tmp_p, sizeof ( Sky_LastInfo_S ) ) ;
        close ( fd_tmp ) ;
        free ( tmp_p ) ;
      }

   } 

   /* datafile open */
   fd = open ( name, O_RDWR ) ;
   if ( fd < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s Open Fail -> Set Default Value. Errno = %d\n", name, errno ) ;
      #endif

      return SKY_FAIL ;

   } else {

      #ifdef DEBUG_SUCC
      printf ( "%s Open Succ -> Set Default Value. Errno = %d\n", name, errno ) ;
      #endif

   }

   /* datafile mmap */
   info_p = ( Sky_LastInfo_S * ) mmap ( 0, sizeof ( Sky_LastInfo_S ), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0 ) ;
   if ( info_p == ( Sky_LastInfo_S * ) MAP_FAILED ) {

      #ifdef DEBUG_FAIL
      printf ( "%s Mmap Fail -> Set Default Value. Errno = %d\n", name, errno ) ;
      #endif

      close ( fd ) ; 
      return SKY_FAIL ;

   } else {

      #ifdef DEBUG
      printf ( "----LAST INFORMATION----\n" ) ;
      printf ( "satellite_name = %s\n", info_p -> satellite_name ) ;
      printf ( "satellite_file = %s\n", info_p -> satellite_file ) ;
      printf ( "transponder_id = %d\n", info_p -> transponder_id ) ;
#if 0
      printf ( "channel_id     = %d\n", info_p -> channel_id     ) ;
      printf ( "channel_name   = %s\n", info_p -> channel_name   ) ;
#endif
      printf ( "num_of_satellite = %d\n", info_p -> num_of_satellite   ) ;
      for ( i = 0 ; i < info_p -> num_of_satellite ; i ++ ) {
         printf ( "[ %d ] . satellite_name= %s\n", i, info_p -> sat_list [ i ] . satellite_name ) ;
         printf ( "[ %d ] . satellite_file= %s\n", i, info_p -> sat_list [ i ] . satellite_file ) ;
      }
      #endif

      return SKY_SUCCESS ;
   }
}

Sky_ReturnVal_E read_initfile ( Sky_LastInfo_S * lastinfo_p )
{
   if ( info_p != ( Sky_LastInfo_S * ) MAP_FAILED ) {
      memcpy ( lastinfo_p, info_p, sizeof ( Sky_LastInfo_S ) ) ;
      return SKY_SUCCESS ;
   } else {

      #ifdef DEBUG_FAIL
      printf ( "%s Read Fail -> Set Default Value. Errno = %d\n", INITIAL_INFO_FILE_NAME, errno ) ;
      #endif

      return SKY_FAIL ;
   }

}
Sky_ReturnVal_E write_initfile ( Sky_LastInfo_S * lastinfo_p )
{
   if ( info_p != ( Sky_LastInfo_S * ) MAP_FAILED ) {
      memcpy ( info_p, lastinfo_p, sizeof ( Sky_LastInfo_S ) ) ;
      return SKY_SUCCESS ;
   } else {

      #ifdef DEBUG_FAIL
      printf ( "%s Write Fail\n", INITIAL_INFO_FILE_NAME ) ;
      #endif
     
      return SKY_FAIL ;
   }
}
Sky_ReturnVal_E close_initfile ( void )
{
   if ( info_p != ( Sky_LastInfo_S * ) MAP_FAILED ) {
      munmap ( ( char * ) info_p, sizeof ( Sky_LastInfo_S ) ) ;
      close ( fd ) ;

      return SKY_SUCCESS ;
   } else {

      #ifdef DEBUG_FAIL
      printf ( "%s Close Fail\n", INITIAL_INFO_FILE_NAME ) ;
      #endif
     
      return SKY_FAIL ;
   }
}
Sky_ReturnVal_E save_skymedia_last_status ( void )
{
   close_initfile ( ) ;
}

