/* 
 * Source Name : sky_ip_chg.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif
#include <errno.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_autoscan.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
data_IpSelBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void SaveIpAddressInform ( void ) ;
void DisplayIpAddress ( int pos, unsigned long ipaddress ) ;
void DisplayIpAddressList ( int count, unsigned long * ipAddr ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
data_IpSelBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  XmString     xmStr ;
  char       * text ;

  #ifdef DEBUG
  printf ( "IP change ...\n" ) ;
  #endif

  XtVaGetValues ( w, XmNlabelString, & xmStr, NULL ) ; 
  XmStringGetLtoR ( xmStr, XmFONTLIST_DEFAULT_TAG, & text ) ;
  XmStringFree ( xmStr ) ;

  XmTextSetString ( data_NC_form2_ipTxt, text ) ;
  XtFree ( text ) ;
}

void SaveIpAddressInform ( void ) 
{
   IP_ADDRESS       ip ;
   char           * txt ;
   unsigned long    ipaddress ;
   int              pos ;

   txt = ( char * ) XmTextGetString ( data_NC_form2_ipTxt ) ;
   if ( *txt == 0 ) {
     #ifdef DEBUG
     printf ( "%s : %d : Empty Ip Address\n", __FILE__, __LINE__ ) ;
     #endif

     return ;
   }

   if ( GetIpAddrFromString ( txt, & ip ) != SKY_SUCCESS ) {

     #ifdef DEBUG
     printf ( "%s : %d : Invalid Ip Address\n", __FILE__, __LINE__ ) ;
     #endif

     return ;
   }
   XtFree ( txt ) ;

   #ifdef DEBUG
   printf ( "IP Address = %d.%d.%d.%d\n", ip . Ipaddress1,
                                          ip . Ipaddress2,
                                          ip . Ipaddress3,
                                          ip . Ipaddress4 ) ;
   #endif


   /*-------------------------------------*/
   /* Set IP Address                      */
   /*-------------------------------------*/
   if ( Sky_SetIPAddress ( ip ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_SetIPAddress () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

   } else {

      #ifdef DEBUG
      printf ( "%s : %d : Sky_SetIPAddress () Succ.\n", __FILE__, __LINE__ ) ;
      #endif

   }

   /*-------------------------------------*/
   /* Insert Into List */
   /*-------------------------------------*/
   ipaddress = ( ( ip.Ipaddress1 & 0xff ) << 24 ) |
               ( ( ip.Ipaddress2 & 0xff ) << 16 ) |
               ( ( ip.Ipaddress3 & 0xff ) << 8 ) |
                 ( ip.Ipaddress4 & 0xff ) ;

   if ( IpAddressLister_GetPosById ( ipaddress, & pos ) == RCL_SUCCESS ) {

     #ifdef DEBUG
     printf ( "%s : %d : IpAddressLister_GetPosById () succ -> already exist\n", __FILE__, __LINE__ ) ;
     #endif

     return ; 
   }

   if ( IpAddressLister_Add ( ipaddress ) != RCL_SUCCESS ) {
     #ifdef DEBUG
     printf ( "%s : %d : IpAddressLister_Add () fail\n", __FILE__, __LINE__ ) ;
     #endif

     return ;
   }

  /* display */
   if ( IpAddressLister_GetPosById ( ipaddress, & pos ) != RCL_SUCCESS ) {

     #ifdef DEBUG
     printf ( "%s : %d : IpAddressLister_GetPosById () fail\n", __FILE__, __LINE__ ) ;
     #endif

     return ; 
   }

   DisplayIpAddress ( pos, ipaddress ) ;
}
void DisplayIpAddress ( int pos, unsigned long ipaddress ) 
{
   Arg        args [ 2 ] ;
   int        ac ;
   XmString   xmstr ;

   char       ipAddr_s [ 100 ] ;

   #ifdef DEBUG
   printf ( "DisplayIpAddress ( )\n" ) ;
   #endif

   sprintf ( ipAddr_s, "%d.%d.%d.%d", 
                       ( ( ipaddress >> 24 ) & 0xff ),
                       ( ( ipaddress >> 16 ) & 0xff ),
                       ( ( ipaddress >> 8 ) & 0xff ),
                         ( ipaddress & 0xff ) ) ;
   #ifdef DEBUG
   printf ( "ipAddr = %s\n", ipAddr_s ) ;
   #endif

   xmstr = XmStringCreate ( ipAddr_s, XmFONTLIST_DEFAULT_TAG ) ; 
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
   ac ++ ;

   switch ( pos ) {
      case 0 :
         XtSetValues ( data_NC_form2_B1, args, ac ) ;
         XtManageChild ( data_NC_form2_B1 ) ;
         break ;
      case 1 :
         XtSetValues ( data_NC_form2_B2, args, ac ) ;
         XtManageChild ( data_NC_form2_B2 ) ;
         break ;
      case 2 :
         XtSetValues ( data_NC_form2_B3, args, ac ) ;
         XtManageChild ( data_NC_form2_B3 ) ;
         break ;
      case 3 :
         XtSetValues ( data_NC_form2_B4, args, ac ) ;
         XtManageChild ( data_NC_form2_B4 ) ;
         break ;
      case 4 :
         XtSetValues ( data_NC_form2_B5, args, ac ) ;
         XtManageChild ( data_NC_form2_B5 ) ;
         break ;
      case 5 :
         XtSetValues ( data_NC_form2_B6, args, ac ) ;
         XtManageChild ( data_NC_form2_B6 ) ;
         break ;
      case 6 :
         XtSetValues ( data_NC_form2_B7, args, ac ) ;
         XtManageChild ( data_NC_form2_B7 ) ;
         break ;
      case 7 :
         XtSetValues ( data_NC_form2_B8, args, ac ) ;
         XtManageChild ( data_NC_form2_B8 ) ;
         break ;
   }
   XmStringFree ( xmstr ) ;
}
void DisplayIpAddressList ( int count, unsigned long * ipAddr ) 
{
   Arg        args [ 2 ] ;
   int        ac ;
   XmString   xmstr ;

   int        i ;
   int        j ;

   char       ipAddr_s [ 100 ] ;

   #ifdef DEBUG
   printf ( "DisplayIpAddressList ( )\n" ) ;
   #endif

   IpAddressLister_Reset ( ) ;

   /* Clear */
   xmstr = XmStringCreate ( "          ", XmFONTLIST_DEFAULT_TAG ) ; 
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
   ac ++ ;

   for ( i = 0 ; i < MAX_IP_ADDRESS_NUM ; i ++ ) {
      switch ( i ) {
         case 0 :
            XtSetValues ( data_NC_form2_B1, args, ac ) ;
            break ;
         case 1 :
            XtSetValues ( data_NC_form2_B2, args, ac ) ;
            break ;
         case 2 :
            XtSetValues ( data_NC_form2_B3, args, ac ) ;
            break ;
         case 3 :
            XtSetValues ( data_NC_form2_B4, args, ac ) ;
            break ;
         case 4 :
            XtSetValues ( data_NC_form2_B5, args, ac ) ;
            break ;
         case 5 :
            XtSetValues ( data_NC_form2_B6, args, ac ) ;
            break ;
         case 6 :
            XtSetValues ( data_NC_form2_B7, args, ac ) ;
            break ;
         case 7 :
            XtSetValues ( data_NC_form2_B8, args, ac ) ;
            break ;
      }
   }

   /* Setting */
   for ( i = 0 ; i < count ; i ++ ) {

      sprintf ( ipAddr_s, "%d.%d.%d.%d", 
                          ( ( ipAddr [ i ] >> 24 ) & 0xff ),
                          ( ( ipAddr [ i ] >> 16 ) & 0xff ),
                          ( ( ipAddr [ i ] >> 8 ) & 0xff ),
                            ( ipAddr [ i ] & 0xff ) ) ;
      #ifdef DEBUG
      printf ( "ipAddr [ %d ] = %s\n", i, ipAddr_s ) ;
      #endif

      if ( IpAddressLister_Add ( ipAddr [ i ] ) != RCL_SUCCESS ) {

        #ifdef DEBUG_FAIL
        printf ( "%s : %d : IpAddressLister_Add () Fail\n", __FILE__, __LINE__ ) ;
        #endif

      }

      xmstr = XmStringCreate ( ipAddr_s, XmFONTLIST_DEFAULT_TAG ) ; 
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
      ac ++ ;

      switch ( i ) {
         case 0 :
            XtSetValues ( data_NC_form2_B1, args, ac ) ;
            XtManageChild ( data_NC_form2_B1 ) ;
            break ;
         case 1 :
            XtSetValues ( data_NC_form2_B2, args, ac ) ;
            XtManageChild ( data_NC_form2_B2 ) ;
            break ;
         case 2 :
            XtSetValues ( data_NC_form2_B3, args, ac ) ;
            XtManageChild ( data_NC_form2_B3 ) ;
            break ;
         case 3 :
            XtSetValues ( data_NC_form2_B4, args, ac ) ;
            XtManageChild ( data_NC_form2_B4 ) ;
            break ;
         case 4 :
            XtSetValues ( data_NC_form2_B5, args, ac ) ;
            XtManageChild ( data_NC_form2_B5 ) ;
            break ;
         case 5 :
            XtSetValues ( data_NC_form2_B6, args, ac ) ;
            XtManageChild ( data_NC_form2_B6 ) ;
            break ;
         case 6 :
            XtSetValues ( data_NC_form2_B7, args, ac ) ;
            XtManageChild ( data_NC_form2_B7 ) ;
            break ;
         case 7 :
            XtSetValues ( data_NC_form2_B8, args, ac ) ;
            XtManageChild ( data_NC_form2_B8 ) ;
            break ;
      }
      XmStringFree ( xmstr ) ;
   }
   for ( i = count ; i < MAX_IP_ADDRESS_NUM ; i ++ ) {
      switch ( i ) {
         case 0 :
            XtUnmanageChild ( data_NC_form2_B1 ) ;
            break ;
         case 1 :
            XtUnmanageChild ( data_NC_form2_B2 ) ;
            break ;
         case 2 :
            XtUnmanageChild ( data_NC_form2_B3 ) ;
            break ;
         case 3 :
            XtUnmanageChild ( data_NC_form2_B4 ) ;
            break ;
         case 4 :
            XtUnmanageChild ( data_NC_form2_B5 ) ;
            break ;
         case 5 :
            XtUnmanageChild ( data_NC_form2_B6 ) ;
            break ;
         case 6 :
            XtUnmanageChild ( data_NC_form2_B7 ) ;
            break ;
         case 7 :
            XtUnmanageChild ( data_NC_form2_B8 ) ;
            break ;
      }
      
   }
}
