/* 
 * Source Name : sky_ip_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_app.h"
#include "sky_ip.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void IpAddress_SetCurrentIpAddressId ( int newId ) ;
int IpAddress_GetCurrentIpAddressId ( void ) ;

void IpAddressLister_Reset ( void ) ;
rcl_return_code IpAddressLister_Add ( int newId );
rcl_return_code IpAddressLister_Remove ( int newId );
rcl_return_code IpAddressLister_GetPosById ( int newId, int * pos );
rcl_return_code IpAddressLister_GetIdByPos ( int pos, int * newId ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int                   current_ipAddress_id_g = 0 ;
static IpAddressList_S     * ipAddressList_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentIpAddressId ( int newId )
{
   current_ipAddress_id_g = newId ;
}
int GetCurrentIpAddressId ( void )
{
   return current_ipAddress_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void IpAddressLister_Reset ( void ) 
{
   IpAddressList_S  *  curr ;
   IpAddressList_S  *  next ;

   XmListDeleteAllItems ( addsat_oscListLister ) ;

   if ( ipAddressList_g == NULL ) {
      return ;
   }

   curr = ipAddressList_g ;
   next = ipAddressList_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   ipAddressList_g = NULL ;
}
rcl_return_code IpAddressLister_Add ( int newId )
{
   IpAddressList_S  ** curr ;
   int                 i ;

   if ( ipAddressList_g == NULL ) {
      ipAddressList_g = ( IpAddressList_S * ) malloc ( sizeof ( IpAddressList_S ) ) ;
      ipAddressList_g -> id = newId ;
      ipAddressList_g -> next = NULL ;

      return RCL_SUCCESS ;
   }

   curr = & ipAddressList_g ;
   for ( i = 0 ; i < MAX_IP_ADDRESS_NUM ; i ++ ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( IpAddressList_S * ) malloc ( sizeof ( IpAddressList_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;
   }

   if ( i == MAX_IP_ADDRESS_NUM ) {
     return RCL_FAILURE ;
   } else {
     return RCL_SUCCESS ;
   }
}
rcl_return_code IpAddressLister_Remove ( int newId )
{
   IpAddressList_S  ** prev ;
   IpAddressList_S  ** curr ;
   int                  pos = 0 ;
   
   if ( ipAddressList_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( ipAddressList_g -> next == NULL ) {
      if ( ipAddressList_g -> id == newId ) {
         free ( ipAddressList_g ) ;
         ipAddressList_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & ipAddressList_g ;
   curr = & ipAddressList_g ;
   while ( ( *curr ) ) {

      if ( ( * curr ) -> id == newId ) {

         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;

         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;

      pos ++ ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code IpAddressLister_GetPosById ( int newId, int * pos )
{
   int               i ;
   IpAddressList_S  * curr ;
   
   if ( ipAddressList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = ipAddressList_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code IpAddressLister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   IpAddressList_S  * curr ;
   
   if ( ipAddressList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = ipAddressList_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}

