#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>

#include "sky_extern.h"
#include "sky_type.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void ResetNetworkPIDs ( int tp_id ) ;
void LockStatus_DisplayToTitle ( LNB_STRUCTURE * lnb_p, TR_STRUCTURE * tp_p, int sts ) ;

void my_SetWatchCursor ( Widget w1 ) ;
void my_ResetCursor ( Widget w1 ) ;

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void set_lockStatus ( XtPointer x, XtIntervalId * id ) ;
void set_sat_status ( XtPointer x, XtIntervalId * id ) ;
void remove_timer ( void ) ;
void display_satellite_signal_status ( void ) ;
void ReallyLockTest ( int );

/**************************************************************************
 * GLOBAL VARIABLES
 **************************************************************************/
extern PixmapResources_t pixmap_resources;

extern Widget * agc_stick_g [ MAX_VOLUME_NUM ] ;
extern Widget * ber_stick_g [ MAX_VOLUME_NUM ] ;

static XtIntervalId  interval = 0 ;  /* for Satellite Whole Status */
static XtIntervalId  interval2 = 0 ; /* for Satellite Signal */

/* 1. read initfile -> transponder_id, channel_id */
/* 2. display Tuner Lock Status */
void set_lockStatus ( XtPointer x, XtIntervalId * id ) 
{
   int                ret1 ;
   int                i ;
   int                agc_num ;
   char               buf [ 100 ] ;
   TUNER_RESULT       tunerResult_p ;

   Arg                args [ 2 ] ;
   Arg                args2 [ 2 ] ;
   int                ac ;
   int                ac2 ;
   XmString           xmstr ;

   int                lockStatus ;

   /* Set Timer : 1 Second */
   interval = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), LOCK_TEST_TIME_OUT, & set_lockStatus, x ) ;

   memcpy ( & lockStatus, x, sizeof ( int ) ) ;

   #ifdef DEBUG
   printf ( "%s : %d : set_lockStatus () : %ld\n", __FILE__, __LINE__, time ( NULL ) ) ;
   #endif

   ret1 = Sky_GetTunerInformation ( & tunerResult_p ) ;
   if ( ret1 != FALSE ) {
      return ; ;
   }

   /*------------------------------------------------------------------*/
   /* Display LOCK STATUS                                              */
   /*------------------------------------------------------------------*/
   if ( lockStatus == SKY_OK ) {
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Green ) ;
      ac ++ ;

      XtSetValues ( lock_tuner, args, ac ) ;
      XtSetValues ( lock_qpsk, args, ac ) ;
      XtSetValues ( lock_viterbi, args, ac ) ;
      XtSetValues ( lock_frame, args, ac ) ;

      ac = 0 ;
      switch ( tunerResult_p . FECRate ) {
         case TUNER_RATE12 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N311 ) ; 
            break ;
         case TUNER_RATE23 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N312 ) ; 
            break ;
         case TUNER_RATE34 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N313 ) ; 
            break ;
         case TUNER_RATE45 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N314 ) ; 
            break ;
         case TUNER_RATE56 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N315 ) ; 
            break ;
         case TUNER_RATE67 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N316 ) ; 
            break ;
         case TUNER_RATE78 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N317 ) ; 
            break ;
         case TUNER_RATE89 :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . N318 ) ; 
            break ;
         default :
            XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . FecNone ) ; 
            break ;
      }
      ac ++  ;
      XtSetValues ( lock_fecVal, args, ac ) ;  

      /* AGC */
      sprintf ( buf, "%03d", tunerResult_p . AGCValue ) ;
      xmstr = XmStringCreate ( buf, XmFONTLIST_DEFAULT_TAG ) ;
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
      ac ++ ;
      XtSetValues ( lock_agcValTxt, args, ac ) ;  
      XmStringFree ( xmstr ) ;

      if ( tunerResult_p . AGCValue <= 50 ) {
         agc_num = 0 ;
      } else if ( tunerResult_p . AGCValue >= 150 ) {
         agc_num = 20 ;
      } else {
         agc_num = ( tunerResult_p . AGCValue - 50 ) / 5 ;
      }

      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Vol3 ) ;
      ac ++ ;

      ac2 = 0 ;
      XtSetArg ( args2 [ ac2 ], XmNlabelPixmap, pixmap_resources . Vol1 ) ;
      ac2 ++ ;
      for ( i = 0 ; i < agc_num ; i ++ ) {
         if ( i < 10 ) {
            XtSetValues ( * agc_stick_g [ i ], args, ac ) ;
         } else {
            XtSetValues ( * agc_stick_g [ i ], args2, ac2 ) ;
         }
      }
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . vol2 ) ;
      ac ++ ;
      for ( i = agc_num ; i < MAX_AGC_NUM ; i ++ ) {
         XtSetValues ( * agc_stick_g [ i ], args, ac ) ;
      }

      /* BER */
      if ( tunerResult_p . Error1 == 0 ) {
         agc_num = 0 ;
      } else {
         switch ( tunerResult_p . Error2 ) {
            case 0 :
            case 1 :
            case 2 :
            case 3 :
               agc_num = 20 ;
               break ;
            case 4 :
               agc_num = 15 + ( tunerResult_p . Error1 ) / 2 ;
               break ;
            case 5 :
               agc_num = 10 + tunerResult_p . Error1 / 2 ;
               break ;
            case 6 :
               agc_num = 5 + tunerResult_p . Error1 / 2 ;
               break ;
            case 7 :
               agc_num = tunerResult_p . Error1 / 2 ;
               break ;
            case 8 :
            case 9 :
            default :
               agc_num = 0 ;
               break ;
         }
      }
      
      sprintf ( buf, "%dE-%d", tunerResult_p . Error1, tunerResult_p . Error2 ) ;

      xmstr = XmStringCreate ( buf, XmFONTLIST_DEFAULT_TAG ) ;
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
      ac ++ ;
      XtSetValues ( lock_berValTxt, args, ac ) ;
      XmStringFree ( xmstr ) ;

      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Vol3 ) ;
      ac ++ ;

      ac2 = 0 ;
      XtSetArg ( args2 [ ac2 ], XmNlabelPixmap, pixmap_resources . vol2 ) ;
      ac2 ++ ;

      for ( i = 0 ; i < agc_num ; i ++ ) {
         XtSetValues ( * ber_stick_g [ i ], args, ac ) ;
      }
      for ( i = agc_num ; i < MAX_BER_NUM ; i ++ ) {
         XtSetValues ( * ber_stick_g [ i ], args2, ac2 ) ;
      }
     
      /*   : Default Status -> Sat1 -> Sat2 -> Sat3 -> Sat4 -> Sat5 */
      display_satellite_signal_status ( ) ;

   /*------------------------------------------------------------------*/
   /* Display UNLOCK STATUS                                            */
   /*------------------------------------------------------------------*/
   } else {

      /* TUNNING STATUS */
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Green ) ;
      ac ++ ;

      ac2 = 0 ;
      XtSetArg ( args2 [ ac2 ], XmNlabelPixmap, pixmap_resources . Red ) ;
      ac2 ++ ;

      if ( tunerResult_p . TunerLock == SKY_OK ) {
         XtSetValues ( lock_tuner, args, ac ) ;
      } else {
         XtSetValues ( lock_tuner, args2, ac2 ) ;
      }

      if ( tunerResult_p . QpskLock == SKY_OK ) {
         XtSetValues ( lock_qpsk, args, ac ) ;
      } else {
         XtSetValues ( lock_qpsk, args2, ac2 ) ;
      }

      if ( tunerResult_p . ViterbiLock == SKY_OK ) {
         XtSetValues ( lock_viterbi, args, ac ) ;
      } else {
         XtSetValues ( lock_viterbi, args2, ac2 ) ;
      }

      if ( tunerResult_p . FrameLock == SKY_OK ) {
         XtSetValues ( lock_frame, args, ac ) ;
      } else {
         XtSetValues ( lock_frame, args2, ac2 ) ;
      }

      /* AGC, BER, FEC ---> CLEAR */
      /* FEC */
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . FecNone ) ; 
      ac ++  ;
      XtSetValues ( lock_fecVal, args, ac ) ;  

      /* AGC */
      strcpy ( buf, "" ) ;
      xmstr = XmStringCreate ( buf, XmFONTLIST_DEFAULT_TAG ) ;
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
      ac ++ ;
      XtSetValues ( lock_agcValTxt, args, ac ) ;  
      XmStringFree ( xmstr ) ;

      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . vol2 ) ;
      ac ++ ;
      for ( i = 0 ; i < MAX_AGC_NUM ; i ++ ) {
         XtSetValues ( * agc_stick_g [ i ], args, ac ) ;
      }

      /* BER */
      strcpy ( buf, "" ) ;

      xmstr = XmStringCreate ( buf, XmFONTLIST_DEFAULT_TAG ) ;
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
      ac ++ ;
      XtSetValues ( lock_berValTxt, args, ac ) ;
      XmStringFree ( xmstr ) ;

      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . vol2 ) ;
      ac ++ ;
      for ( i = 0 ; i < MAX_BER_NUM ; i ++ ) {
         XtSetValues ( * ber_stick_g [ i ], args, ac ) ;
      }

      /*   : Default Status -> Sat1*/
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat1 ) ;
      ac ++ ;
      XtSetValues ( lock_sat, args, ac ) ;

   }
}
void remove_timer ( void ) 
{
   if ( interval != 0 ) {
      XtRemoveTimeOut ( interval ) ;
   }
   if ( interval2 != 0 ) {
      XtRemoveTimeOut ( interval2 ) ;
   }

   interval = 0 ;
}
void display_satellite_signal_status ( void ) 
{
   if ( interval2 != 0 ) {
      XtRemoveTimeOut ( interval2 ) ;
   }
   /*interval2 = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), LOCK_TEST_TIME_OUT / 5, & set_sat_status, NULL ) ;*/
   interval2 = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), LOCK_TEST_TIME_OUT / 20, & set_sat_status, NULL ) ;
}
void set_sat_status ( XtPointer x, XtIntervalId * id )
{
   Arg          args [ 2 ] ;
   int          ac ;

   static int   count = 0 ;

   interval2 = XtAppAddTimeOut ( XtWidgetToApplicationContext ( data_shell ), LOCK_TEST_TIME_OUT / 5, & set_sat_status, NULL ) ;

   ac = 0 ;
   switch ( count % 5 ) {
      case 1 :
         XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat2 ) ;
         break ;
      case 2 :
         XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat3 ) ;
         break ;
      case 3 :
         XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat4 ) ;
         break ;
      case 4 :
         XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat5 ) ;
         break ;
      case 0 :
      default :
         XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Sat1 ) ;
         break ;
         
   } 
   ac ++ ;
   XtSetValues ( lock_sat, args, ac ) ;

   count ++ ;
}

void
lock_shell_popupCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
   my_SetWatchCursor ( data_shell ) ;
}

void
lock_shell_popdownCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
   #ifdef DEBUG
   printf ( "lock_shell_popdownCallback\n" ) ;
   #endif

#if 0
   EnableWidget ( data_shell ) ;
#else
   my_ResetCursor ( data_shell ) ;
#endif

   /* remove timer */
   remove_timer ( ) ;
}

void
tuner_lockBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
   Sky_ReturnVal_E   ret ;
   int               numTp ;
   int             * position ;
   int               count ;
   Boolean           rt ;
   int               tp_id = 0 ;

   #ifdef DEBUG
   printf ( "tuner_lockBCallback\n" ) ;
   #endif

   if ( ! Sky_GetNumOfTransponder () ) {
     DisplayErrorDialog ( w, "Warning !!\nThere is No Transponder\n" ) ;
     return ;
   }

#if 1
   rt = XmListGetSelectedPos ( tuner_satinfo_FqSyList_list, & position, & count ) ;
   if ( rt != True ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : XmListGetSelectedPos () Fail\n", __FILE__, __LINE__ ) ;
     #endif

     DisplayErrorDialog ( w, "Warning !!\nSelect a Transponder" ) ;
     return ;

   } else if ( count == 0 ) {
     DisplayErrorDialog ( w, "Warning !!\nSelect a Transponder" ) ;
     return ;
   } 
   tp_id = *position ;
#else
   tp_id = XmListGetKbdItemPos ( data_PE_form1_listLister ) ;
   if ( tp_id <= 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Unselected Transponder\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }
#endif

#if 0
   DisableWidget ( data_shell ) ;
#endif
   XtVaSetValues ( lock_shell, XmNmwmDecorations, NULL, NULL ) ;
   XtManageChild ( lock_board ) ;

   /* Lock Test */
   ReallyLockTest ( tp_id ) ;
}

void ReallyLockTest ( int tr_id )
{
   Sky_LastInfo_S     sky_p ;
   int                ret ;
   int                ret1 ;
   int                i ;
   int                agc_num ;
   char               buf [ 100 ] ;
   TUNER_RESULT       tunerResult_p ;

   Arg                args [ 2 ] ;
   Arg                args2 [ 2 ] ;
   int                ac ;
   int                ac2 ;
   XmString           xmstr ;

   LNB_STRUCTURE      lnbinfo ;
   TR_STRUCTURE       tpinfo ;

   static int         g_lockStatus ;

   if ( Sky_GetSatelliteInformation ( & lnbinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetSatelliteInformation () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( lock_close, "Warning!!\nFail to get Satellte information" ) ;
      return ;
   }

   if ( Sky_GetInfoTransponder ( tr_id , & tpinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( lock_close, "Warning!!\nFail to get Transponder information" ) ;
      return ;
   }

   if ( Sky_ReleaseAllNetworkPID ( ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ReleaseAllNetworkPID () Fail.\n", __FILE__, __LINE__ ) ;
      #endif
   }

   /*----------------------------------------------------------------------*/
   /* Initfile read */
   /*----------------------------------------------------------------------*/
   read_initfile ( & sky_p ) ;

   g_lockStatus = Sky_TunerLockTest ( lnbinfo, tpinfo ) ;

   sky_p . transponder_id = tpinfo . TransponderNumber ;
   write_initfile ( & sky_p ) ;

   /*------------------------------------------------------------------*/
   /* Display Lock Status to Title */
   /*------------------------------------------------------------------*/
   LockStatus_DisplayToTitle ( & lnbinfo, & tpinfo, ret ) ;

   /*---------------------------------------------------------------*/
   /* Reset Network PID */
   /*---------------------------------------------------------------*/
   ResetNetworkPIDs ( tpinfo . TransponderNumber ) ;

   /*---------------------------------------------------------------*/
   /* Display Lock status to Lock Panel */
   /*---------------------------------------------------------------*/
   set_lockStatus ( ( void * ) & g_lockStatus, NULL ) ;
}
