/* 
 * Source Name : sky_sat_add_edit.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <stdio.h>
#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat.h"
#include "sky_sat_osc.h"
#include "sky_pixel.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void refresh_transponder_info ( void ) ;
Sky_ReturnVal_E read_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
Sky_ReturnVal_E IsSatelliteNameValid ( char * name ) ;
void insert_satellite_name_into_entry ( void ) ;

void
data_menu_SATBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void Compare_LNB_OldInfo_Set ( LNB_STRUCTURE * old_lnb ) ;
Sky_ReturnVal_E Compare_Old_With_LNB_NewInfo ( LNB_STRUCTURE * new_lnb ) ;
void SAT_ValChg_Set ( void ) ;
Sky_ReturnVal_E DoSameSatelliteExist ( unsigned char * name ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
tuner_satImportBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_satNewBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_satEditBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_scanBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_lockBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_satAutoScanBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
addsat_shell_popupCallback ( Widget w, XtPointer client_data, XtPointer call_data ) ;
void
addsat_shell_popdownCallback ( Widget w, XtPointer client_data, XtPointer call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void SetLabelSatelliteSubShell ( char * lab ) ;
void ClearSatelliteSubShell ( void ) ;

Sky_ReturnVal_E GetToneBurstWidget ( u8 tone, Widget * w ) ;
Sky_ReturnVal_E GetDiseqcWidget ( u8 diseqc, Widget * w ) ;
Sky_ReturnVal_E GetToneBurstValue ( u8 * tone ) ;
Sky_ReturnVal_E GetDiseqcValue ( u8 * diseqc ) ;

void SetValSatelliteSubShell ( LNB_STRUCTURE * lnb_p ) ;
Sky_ReturnVal_E GetValSatelliteShell ( LNB_STRUCTURE * tp_p ) ;
Sky_ReturnVal_E SetNewSatelliteInfo ( LNB_STRUCTURE * lnb_p ) ;
Sky_ReturnVal_E SetChangedSatelliteInfo ( LNB_STRUCTURE * lnb_p ) ;
SAT_B_Select_E GetAddEditSatellite ( void ) ;
void LocalOscillatorList_Compare ( int newVal ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
extern PixelResources_t pixel_resources ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
tuner_satImportBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
}
void
tuner_satNewBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{

  SetLabelSatelliteSubShell ( SATELLITE_ADD_TITLE_STR ) ;
  ClearSatelliteSubShell ( ) ;
  XtManageChild ( addsat_board ) ;

#if 0
  XtSetSensitive ( data_shell, False ) ;
#endif

}
void
tuner_satEditBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  LNB_STRUCTURE lnb_p ;
  int           pos ;

  #ifdef DEBUG
  printf ( "%s : %d : tuner_satEditBCallback ()\n", __FILE__, __LINE__ ) ; fflush ( stdout ) ;
  #endif

  SetLabelSatelliteSubShell ( SATELLITE_EDIT_TITLE_STR ) ;
  ClearSatelliteSubShell ( ) ;

  /*--------------------------------------------------------*/
  /* get satellite information from driver */
  /*--------------------------------------------------------*/
  memset ( & lnb_p, 0, sizeof ( lnb_p ) ) ;
  if ( Sky_GetSatelliteInformation ( & lnb_p ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_GetSatelliteInformation () Fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*--------------------------------------------------------*/
  /* setting satellite info */
  /*--------------------------------------------------------*/
  SetValSatelliteSubShell ( & lnb_p ) ;

  /*--------------------------------------------------------*/
  /* Set satellite name uneditable */
  /*--------------------------------------------------------*/
  /*XmTextSetEditable ( addsat_nameT, False ) ;*/
  XtVaSetValues ( addsat_nameT, XmNeditable, False, 0 ) ;

  #ifdef DEBUG
  printf ( "%s : %d : tuner_satEditBCallback () \n", __FILE__, __LINE__ ) ; fflush ( stdout ) ;
  #endif

  XtVaSetValues ( addsat_nameT, XmNcursorPositionVisible, False, NULL ) ;

  #ifdef DEBUG
  printf ( "%s : %d : tuner_satEditBCallback () \n", __FILE__, __LINE__ ) ; fflush ( stdout ) ;
  #endif

  /*--------------------------------------------------------*/
  /* display addsat_shell */
  /*--------------------------------------------------------*/
  XtManageChild ( addsat_board ) ;
#if 0
  XtSetSensitive ( data_shell, False ) ;
#endif

  /*--------------------------------------------------------*/
  /* Save Old Satellite Information */
  /*--------------------------------------------------------*/
  Compare_LNB_OldInfo_Set ( & lnb_p ) ;

}

void
addsat_shell_popupCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
#if 0
  XtSetSensitive ( data_shell, False ) ;
#endif
}
void
addsat_shell_popdownCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
#if 0
  XtSetSensitive ( data_shell, True ) ;
#endif
}

void
addsat_okBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  LNB_STRUCTURE  tp_p ;
  int            numOfTrans ;
  int            pos ;

  memset ( & tp_p, 0, sizeof ( tp_p ) ) ;

  /*----------------------------------------------------------*/
  /* Get Add/Edit Satellite Value from Pannel */
  /*----------------------------------------------------------*/
  if ( GetValSatelliteShell ( & tp_p ) != SKY_SUCCESS ) {
    XtSetSensitive ( data_shell, True ) ;
    return ;
  } 

  /*----------------------------------------------------------*/
  /* Add or Edit ? */
  /*----------------------------------------------------------*/
  switch ( GetAddEditSatellite ( ) ) {
    case SAT_ADD_E :
      /* New Satellite Add */
      SetNewSatelliteInfo ( & tp_p ) ;
      /* Oscillator value Add or not */
      LocalOscillatorList_Compare ( tp_p . LocalOscillator ) ;
      break ;

    case SAT_EDIT_E :
      SetChangedSatelliteInfo ( & tp_p ) ; 

      /*------------------------------------------------------*/
      /* Compare Old LNB & New LNB */
      /*------------------------------------------------------*/
      if ( Compare_Old_With_LNB_NewInfo ( & tp_p ) == SKY_SUCCESS ) {
        SAT_ValChg_Set ( ) ;
      }
      /* Oscillator value Add or not */
      LocalOscillatorList_Compare ( tp_p . LocalOscillator ) ;

      break ;

    default :
    
      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Invalid Satellite Add/Edit?\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
  }

  /*------------------------------------------------------*/
  /* Re-Display Satellite Info */
  /*------------------------------------------------------*/
  data_menu_SATBCallback ( NULL, NULL, NULL ) ;

}

Sky_ReturnVal_E GetValSatelliteShell ( LNB_STRUCTURE * tp_p ) 
{
  char * txt ;
  u8     tone ;
  u8     diseqc ;
  float  f_tmp ;

  /*-------------------------------------------------------*/
  /* get satellite name value */
  /*-------------------------------------------------------*/
  txt = ( char * ) XmTextGetString ( addsat_nameT ) ;
  if ( *txt == ( char )NULL ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : empty satellite name\n", __FILE__, __LINE__ ) ;
    #endif

    return SKY_FAIL ;
  }
  strcpy ( ( char * ) tp_p -> SatelliteName, txt ) ;
  XtFree ( txt ) ; 

  if ( IsSatelliteNameValid ( ( char * ) tp_p -> SatelliteName ) != SKY_SUCCESS ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Invalid Satellite Name [%s]\n", __FILE__, __LINE__,  tp_p -> SatelliteName ) ;
     #endif

     return SKY_FAIL ;
   }

  
  /*-------------------------------------------------------*/
  /* get local Oscillator value */
  /*-------------------------------------------------------*/
  txt = ( char * ) XmTextGetString ( addsat_oscT ) ;
  if ( *txt == ( char )NULL ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : empty Local Oscillator\n", __FILE__, __LINE__ ) ;
    #endif

    return SKY_FAIL ;
  }
  sscanf ( txt, "%f", & f_tmp ) ;
  tp_p -> LocalOscillator = f_tmp * 1000 ;
  XtFree ( txt ) ; 
  
  /*-------------------------------------------------------*/
  /* Get Tone Burst */
  /*-------------------------------------------------------*/
  if ( GetToneBurstValue ( & tone ) != SKY_SUCCESS ) {
    return SKY_FAIL ;
  }
  tp_p -> ToneBurst = tone ;

  /*-------------------------------------------------------*/
  /* Get DiSEqC */
  /*-------------------------------------------------------*/
  if ( GetDiseqcValue ( & diseqc ) != SKY_SUCCESS ) {
    return SKY_FAIL ;
  }
  tp_p -> DiseqcPort = diseqc ;

  /*-------------------------------------------------------*/
  /* get switch on/off */
  /*-------------------------------------------------------*/
  if ( XmToggleButtonGetState ( addsat_22kB1 ) == True ) {
    tp_p -> SwitchOnOff = LNB_HIGH ;
  } else if ( XmToggleButtonGetState ( addsat_22kB2 ) == True ) {
    tp_p -> SwitchOnOff = LNB_LOW ;
  } else {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : empty switch on/off\n", __FILE__, __LINE__ ) ;
    #endif

    return SKY_FAIL ;
  }

  /*-------------------------------------------------------*/
  /* get lnb power */
  /*-------------------------------------------------------*/
  if ( XmToggleButtonGetState ( addsat_lnbB1 ) == True ) {
    tp_p -> LNBPower = LNB_POWER_ON ;
  } else if ( XmToggleButtonGetState ( addsat_lnbB2 ) == True ) {
    tp_p -> LNBPower = LNB_POWER_OFF ;
  } else {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : empty switch on/off\n", __FILE__, __LINE__ ) ;
    #endif

    return SKY_FAIL ;
  }

  return SKY_SUCCESS ;
}
void SetValSatelliteSubShell ( LNB_STRUCTURE * lnb_p ) 
{
  char    txt [ 100 ] ;
  Widget  w ;

  /*-------------------------------------------------------*/
  /* satellite name */
  /*-------------------------------------------------------*/
  XmTextSetString ( addsat_nameT, lnb_p -> SatelliteName ) ;
  #ifdef DEBUG
  printf ( "%s : %d : sat name = %s\n", __FILE__, __LINE__, lnb_p -> SatelliteName ) ;
  #endif

  /*-------------------------------------------------------*/
  /* local oscillator */
  /*-------------------------------------------------------*/
  ConvertInt2Str_LocalOscillator_2 ( ( lnb_p -> LocalOscillator / 1000 ), txt ) ;
  XmTextSetString ( addsat_oscT, txt ) ;

  /*-------------------------------------------------------*/
  /* Tone burst */
  /*-------------------------------------------------------*/
  if ( GetToneBurstWidget ( lnb_p -> ToneBurst, & w ) != SKY_SUCCESS ) {
    return ;
  }
  XtVaSetValues ( addsat_toneOpt, XmNmenuHistory, w, NULL ) ;

  /*-------------------------------------------------------*/
  /* Diseqc */
  /*-------------------------------------------------------*/
  if ( GetDiseqcWidget ( lnb_p -> DiseqcPort, & w ) != SKY_SUCCESS ) {
    return ;
  }
  XtVaSetValues ( addsat_diseqcOpt, XmNmenuHistory, w, NULL ) ;

  /*-------------------------------------------------------*/
  /* switch on/off */
  /*-------------------------------------------------------*/
  EnableWidget ( addsat_22kRB ) ;
  if ( ! lnb_p -> LocalOscillator ) {
    DisableWidget ( addsat_22kRB ) ;
  } else if ( lnb_p -> SwitchOnOff == LNB_HIGH ) {
    XmToggleButtonSetState ( addsat_22kB1, True, False ) ;
    XmToggleButtonSetState ( addsat_22kB2, False, False ) ;
  } else if ( lnb_p -> SwitchOnOff == LNB_LOW ) {
    XmToggleButtonSetState ( addsat_22kB1, False, False ) ;
    XmToggleButtonSetState ( addsat_22kB2, True, False ) ;
  } else {
    XmToggleButtonSetState ( addsat_22kB1, False, False ) ;
    XmToggleButtonSetState ( addsat_22kB2, False, False ) ;
  }

  /*-------------------------------------------------------*/
  /* Lnb power */
  /*-------------------------------------------------------*/
  if ( lnb_p -> LNBPower == LNB_POWER_ON ) {
    XmToggleButtonSetState ( addsat_lnbB1, True, False ) ;
    XmToggleButtonSetState ( addsat_lnbB2, False, False ) ;
  } else if ( lnb_p -> LNBPower == LNB_POWER_OFF ) {
    XmToggleButtonSetState ( addsat_lnbB1, False, False ) ;
    XmToggleButtonSetState ( addsat_lnbB2, True, False ) ;
  } else {
    XmToggleButtonSetState ( addsat_lnbB1, False, False ) ;
    XmToggleButtonSetState ( addsat_lnbB2, False, False ) ;
  }
}

void SetLabelSatelliteSubShell ( char * lab )
{
  XmString   str ;

  str = ( XmString ) strToXmString ( lab ) ;

  /* addtrans_shell title setting */
  XtVaSetValues ( addsat_shell, XmNtitle, lab, NULL ) ;

  /* addtrans_label setting */
  XtVaSetValues ( addsat_lab, XmNlabelString, str, NULL ) ;

  XmStringFree ( str ) ;
}
void ClearSatelliteSubShell ( void )
{
  #ifdef DEBUG
  printf ( "%s : %d : ClearSatelliteSubShell ()\n", __FILE__, __LINE__ ) ; fflush ( stdout ) ;
  #endif

  /*--------------------------------------------------------*/
  /* Set satellite name editable & cursor visible */
  /*--------------------------------------------------------*/
  /*XmTextSetEditable ( addsat_nameT, True ) ;*/
  XtVaSetValues ( addsat_nameT, XmNeditable, True, 0 ) ;
  XtVaSetValues ( addsat_nameT, XmNcursorPositionVisible, True, NULL ) ;

  /*--------------------------------------------------------*/
  /* Set satellite info clear */
  /*--------------------------------------------------------*/
  XmTextSetString ( addsat_nameT, "" ) ;
  XmTextSetString ( addsat_oscT, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ;
  XtVaSetValues ( addsat_toneOpt, XmNmenuHistory, addsat_toneB1, NULL ) ;
  XtVaSetValues ( addsat_diseqcOpt, XmNmenuHistory, addsat_diseqcB1, NULL ) ;
  XmToggleButtonSetState ( addsat_22kB1, True, False ) ;
  XmToggleButtonSetState ( addsat_22kB2, False, False ) ;
  XmToggleButtonSetState ( addsat_lnbB1, True, False ) ;
  XmToggleButtonSetState ( addsat_lnbB2, False, False ) ;
}


SAT_B_Select_E GetAddEditSatellite ( void ) 
{
  XmString   str ;
  Boolean    rt ;
  char     * txt ;

  XtVaGetValues ( addsat_lab, XmNlabelString, & str, NULL ) ;
  rt = XmStringGetLtoR ( str, XmFONTLIST_DEFAULT_TAG, & txt ) ;
  XmStringFree ( str ) ;
  if ( rt != True ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : XmStringGetLtoR() fail\n", __FILE__, __LINE__ ) ;
    #endif

    return SAT_UNKNOWN_E ;
  }

  if ( ! strcmp ( txt, SATELLITE_ADD_TITLE_STR ) ) {
    return SAT_ADD_E ;
  } else if ( ! strcmp ( txt, SATELLITE_EDIT_TITLE_STR ) ) {
    return SAT_EDIT_E ;
  } else {
    return SAT_UNKNOWN_E ;
  }
}

Sky_ReturnVal_E GetToneBurstWidget ( u8 tone, Widget * w ) 
{
  switch ( tone ) {
    case TONE_BURST_DISABLE :
      *w = addsat_toneB1 ;
      break ;
    case TONE_BURST_A :
      *w = addsat_toneB2 ;
      break ;
    case TONE_BURST_B :
      *w = addsat_toneB3 ;
      break ;
    default :
      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Invalid Tone Burst Value = %d\n", __FILE__, __LINE__, tone ) ;
      #endif
      return SKY_FAIL ;
  }

  return SKY_SUCCESS ;
}
Sky_ReturnVal_E GetDiseqcWidget ( u8 diseqc, Widget * w ) 
{
  switch ( diseqc ) {
    case DISEQC_DISABLE :
      *w = addsat_diseqcB1 ;
      break ;
    case DISEQC_PORT1 :
      *w = addsat_diseqcB2 ;
      break ;
    case DISEQC_PORT2 :
      *w = addsat_diseqcB3 ;
      break ;
    case DISEQC_PORT3 :
      *w = addsat_diseqcB4 ;
      break ;
    case DISEQC_PORT4 :
      *w = addsat_diseqcB5 ;
      break ;
    default :
      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Invalid Diseqc Value = %d\n", __FILE__, __LINE__, diseqc ) ;
      #endif
      return SKY_FAIL ;
  }

  return SKY_SUCCESS ;
}
Sky_ReturnVal_E GetToneBurstValue ( u8 * tone ) 
{
  Widget    w ;

  XtVaGetValues ( addsat_toneOpt, XmNmenuHistory, & w, NULL ) ;
  if ( w == addsat_toneB1 ) {
    *tone = TONE_BURST_DISABLE ;
  } else if ( w == addsat_toneB2 ) {
    *tone = TONE_BURST_A ;
  } else if ( w == addsat_toneB3 ) {
    *tone = TONE_BURST_B ;
  } else {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Invalid Tone Burst Widget = %x\n", __FILE__, __LINE__, w ) ;
      #endif

      return SKY_FAIL ;
  }

  return SKY_SUCCESS ;
}
Sky_ReturnVal_E GetDiseqcValue ( u8 * diseqc ) 
{
  Widget    w ;
  XmString  str ;

  XtVaGetValues ( addsat_diseqcOpt, XmNmenuHistory, & w, NULL ) ;
  if ( w == addsat_diseqcB1 ) {
    *diseqc = DISEQC_DISABLE ;
  } else if ( w == addsat_diseqcB2 ) {
    *diseqc = DISEQC_PORT1 ;
  } else if ( w == addsat_diseqcB3 ) {
    *diseqc = DISEQC_PORT2 ;
  } else if ( w == addsat_diseqcB4 ) {
    *diseqc = DISEQC_PORT3 ;
  } else if ( w == addsat_diseqcB5 ) {
    *diseqc = DISEQC_PORT4 ;

  } else {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Invalid Diseqc Widget\n", __FILE__, __LINE__ ) ;
     #endif
     return SKY_FAIL ;
  }

  return SKY_SUCCESS ;
}
Sky_ReturnVal_E SetNewSatelliteInfo ( LNB_STRUCTURE * lnb_p ) 
{
  char       filename [ MAX_FILE_NAME_LEN ] ;

  /*----------------------------------------------------------------*/
  /* Check Same satellite name */
  /*----------------------------------------------------------------*/
  if ( DoSameSatelliteExist ( lnb_p -> SatelliteName ) == SKY_SUCCESS ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : The satellite name(%s) already exist\n", __FILE__, __LINE__, lnb_p -> SatelliteName ) ;
    #endif

    DisplayErrorDialog ( tuner_satAddNewB, "Warnig !\nAlready exsit satellite name" ) ;
    return SKY_FAIL ;
  }

  /*----------------------------------------------------------------*/
  /* Reset Transponder Info */
  /*----------------------------------------------------------------*/
  if ( Sky_FreeTransponderStructure ( ) != FALSE ) {

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_FreeTransponderStructure () Fail\n", __FILE__, __LINE__ ) ;
     #endif

     /*return SKY_FAIL ;*/
  }

  /*----------------------------------------------------------------*/
  /* Set New Satellite */
  /*----------------------------------------------------------------*/
  if ( Sky_SetNewLNBInformation ( * lnb_p ) != FALSE ) {

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_SetNewLNBInformation () Fail [%s]\n", __FILE__, __LINE__, lnb_p -> SatelliteName ) ;
     #endif

     return SKY_FAIL ;
  }

  /*----------------------------------------------------------------*/
  /* Get New Satellite File Name */
  /*----------------------------------------------------------------*/
  if ( get_file_name_from_new_satellite ( lnb_p -> SatelliteName, filename ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : get_file_name_from_new_satellite () Fail\n", __FILE__, __LINE__ ) ;
     #endif

     return SKY_FAIL ;
  }

  /*----------------------------------------------------------------*/
   /* initfile Update : Satellite List */
  /*----------------------------------------------------------------*/
  if ( insert_new_satellite_to_initfile ( lnb_p -> SatelliteName, filename ) != SKY_SUCCESS ) {

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : insert_new_satellite_to_initfile () Fail\n", __FILE__, __LINE__ ) ;
     #endif

     return SKY_FAIL ;
  }

  /*----------------------------------------------------------------*/
  /* Create & Open New Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileCreate ( filename ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileCreate () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }

  /*----------------------------------------------------------------*/
  /* Write To Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileWrite ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileWrite () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }
  /*----------------------------------------------------------------*/
  /* Close Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileClose () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }

  /*----------------------------------------------------------------------*/
  /* Insert Satellite Entry  */
  /*----------------------------------------------------------------------*/
  insert_satellite_name_into_entry ( ) ;

  return SKY_SUCCESS ;

}
Sky_ReturnVal_E SetChangedSatelliteInfo ( LNB_STRUCTURE * lnb_p ) 
{
  char            filename [ MAX_FILE_NAME_LEN ] ;
  Sky_LastInfo_S  sky_p ;

  read_initfile ( & sky_p ) ;
  strcpy ( filename, sky_p . satellite_file ) ;

  /*----------------------------------------------------------------*/
  /* Set New Satellite */
  /*----------------------------------------------------------------*/
  if ( Sky_SetNewLNBInformation ( * lnb_p ) != FALSE ) {

     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_SetNewLNBInformation () Fail [%s]\n", __FILE__, __LINE__, lnb_p -> SatelliteName ) ;
     #endif

     return SKY_FAIL ;
  }
#if 0
  /*----------------------------------------------------------------*/
  /* Open Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileOpen ( filename ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileOpen () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }
  /*----------------------------------------------------------------*/
  /* Write To Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileWrite ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileWrite () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }
  /*----------------------------------------------------------------*/
  /* Close Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileClose () Fail [%s]\n", __FILE__, __LINE__, filename ) ;
     #endif

     return SKY_FAIL ;
  }
#endif
  return SKY_SUCCESS ;
}

