/* 
 * Source Name : sky_sat_chg.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
void
data_menu_SATBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data );
Sky_ReturnVal_E read_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
Sky_ReturnVal_E write_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
Sky_ReturnVal_E get_filename ( char * inp, char * outp ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
tuner_satNameSelectBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
Sky_ReturnVal_E GetSatelliteNameFromWidget ( Widget w, char * satname ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
tuner_satNameSelectBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  char             old_satname [ MAX_SATELLITE_NAME_LEN + 1 ] ;
  char             new_satname [ MAX_SATELLITE_NAME_LEN + 1 ] ;
  char             satname [ MAX_SATELLITE_NAME_LEN + 1 ] ;
  char             filename [ MAX_FILE_NAME_LEN + 1 ] ;
  Sky_LastInfo_S   sky_p ;
  int              error ;
  int              i ;
  int              j ;

  /*-----------------------------------------------------------------------*/
  /* Get Satellite Name */
  /*-----------------------------------------------------------------------*/
  memset ( satname, 0, sizeof ( satname ) );
  if ( GetSatelliteNameFromWidget ( w, satname ) != SKY_SUCCESS ) {
    return ;
  }

  read_initfile ( & sky_p ) ;

#if 0
  /*-----------------------------------------------------------------------*/
  /* Write Init File */
  /*-----------------------------------------------------------------------*/
  if ( get_filename ( satname, filename ) != SKY_SUCCESS ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : get_filename () Fail [%s]\n", __FILE__, __LINE__, satname ) ;
     #endif

     return ;
  }
  strcpy ( sky_p . satellite_name, satname ) ;
  strcpy ( sky_p . satellite_file, filename ) ;
  sky_p . transponder_id = 0 ;
  write_initfile ( & sky_p ) ;

  /*-----------------------------------------------------------------------*/
  /* Open & Read & Close Channel */
  /*-----------------------------------------------------------------------*/
  /*----------------------------------------------------------------*/
  /* Open New Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileOpen ( filename ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileOpen () Fail [%s]\n", __FILE__, __LINE__, satname ) ;
     #endif

     return ;
  }
  /*----------------------------------------------------------------*/
  /* Read Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileRead ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileRead () Fail [%s]\n", __FILE__, __LINE__, satname ) ;
     #endif

     return ;
  }
  /*----------------------------------------------------------------*/
  /* Close Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileClose () Fail [%s]\n", __FILE__, __LINE__, satname ) ;
     #endif

     return ;
  }
#else

  strcpy ( old_satname, sky_p.satellite_name ) ;

  /*----------------------------------------------------------------*/
  /* Reset Transponder Info */
  /*----------------------------------------------------------------*/
  if ( Sky_FreeTransponderStructure ( ) != FALSE ) {
    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_FreeTransponderStructure () Fail\n", __FILE__, __LINE__ ) ;
    #endif
  }
   for ( i = 0, error = 0 ; ; i ++ ) { 

     /*---------------------------------------*/
     /*   ̸ */
     /*---------------------------------------*/
     if ( i > 1 ) { 
       if ( Get_NextSatelliteFileName ( filename ) != SKY_SUCCESS ) {

          #ifdef DEBUG_FAIL
          printf ( "%s : %d : get_filename () Fail. SatName = %s\n", __FILE__, __LINE__, sky_p . satellite_name ) ;
          #endif

          error = 1 ;
          break ;
       } else {
          read_initfile ( & sky_p ) ;
          strcpy ( new_satname, sky_p.satellite_name ) ;
       }
     } else if ( i == 1 ) {
       if ( get_filename ( old_satname, filename ) != SKY_SUCCESS ) {

          #ifdef DEBUG_FAIL
          printf ( "%s : %d : get_filename () Fail. SatName = %s\n", __FILE__, __LINE__, sky_p . satellite_name ) ;
          #endif

          continue ;
       }
       strcpy ( new_satname, old_satname ) ;
       strcpy ( sky_p . satellite_name, old_satname ) ;
       strcpy ( sky_p . satellite_file, filename ) ;
       sky_p . transponder_id = 0 ;
       write_initfile ( & sky_p ) ;
     } else {
       if ( get_filename ( satname, filename ) != SKY_SUCCESS ) {

          #ifdef DEBUG_FAIL
          printf ( "%s : %d : get_filename () Fail. SatName = %s\n", __FILE__, __LINE__, sky_p . satellite_name ) ;
          #endif

          continue ;
       }
       strcpy ( new_satname, satname ) ;
       strcpy ( sky_p . satellite_name, satname ) ;
       strcpy ( sky_p . satellite_file, filename ) ;
       sky_p . transponder_id = 0 ;
       write_initfile ( & sky_p ) ;
     }

     #ifdef DEBUG
     printf ( "%s : %d : fileName = %s\n", __FILE__, __LINE__, filename ) ;
     #endif

     read_initfile ( & sky_p ) ;
     if ( Sky_ChannelFileOpen ( filename ) != FALSE ) {

        #ifdef DEBUG_FAIL
        printf ( "%s : %d : Sky_ChannelFileOpen () Fail. File = %s\n", __FILE__, __LINE__, filename ) ;
        #endif

        /*-----------------------------------------------------------------------*/
        /* compare satellite name */
        /*-----------------------------------------------------------------------*/
        for ( j = 0 ; j < MAX_SATELLITE_NUM ; j ++ ) {
          if ( ! strcmp ( new_satname, sky_p . sat_list [ j ] . satellite_name ) ) {
            memset ( & sky_p. sat_list [ j ], 0, sizeof ( Sky_SatelliteEntry_S ) );
            sky_p . num_of_satellite = sky_p . num_of_satellite - 1 ;
            break ;
          }
        }

        continue ;
     }
     if ( Sky_ChannelFileRead ( ) != FALSE ) {

        #ifdef DEBUG_FAIL
        printf ( "%s : %d : Sky_ChannelFileRead () Fail. \n", __FILE__, __LINE__ ) ;
        #endif

        /*-----------------------------------------------------------------------*/
        /* compare satellite name */
        /*-----------------------------------------------------------------------*/
        for ( j = 0 ; j < MAX_SATELLITE_NUM ; j ++ ) {
          if ( ! strcmp ( new_satname, sky_p . sat_list [ j ] . satellite_name ) ) {
            memset ( & sky_p. sat_list [ j ], 0, sizeof ( Sky_SatelliteEntry_S ) );
            sky_p . num_of_satellite = sky_p . num_of_satellite - 1 ;
            break ;
          }
        }

        continue ;
     }
     if ( Sky_ChannelFileClose ( ) != FALSE ) {

        #ifdef DEBUG_FAIL
        printf ( "%s : %d : Sky_ChannelFileClose () Fail.\n", __FILE__, __LINE__ ) ;
        #endif

        /*-----------------------------------------------------------------------*/
        /* compare satellite name */
        /*-----------------------------------------------------------------------*/
        for ( j = 0 ; j < MAX_SATELLITE_NUM ; j ++ ) {
          if ( ! strcmp ( new_satname, sky_p . sat_list [ j ] . satellite_name ) ) {
            memset ( & sky_p. sat_list [ j ], 0, sizeof ( Sky_SatelliteEntry_S ) );
            sky_p . num_of_satellite = sky_p . num_of_satellite - 1 ;
            break ;
          }
        }

        continue ;
     }

     break ;
   }

   if ( error ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : There is No Satellite File\n", __FILE__, __LINE__ ) ;
    #endif

   }
#endif

  /*-----------------------------------------------------------------------*/
  /* Refresh New Satellite */
  /*-----------------------------------------------------------------------*/
  data_menu_SATBCallback ( NULL, NULL, NULL ) ;
}

Sky_ReturnVal_E GetSatelliteNameFromWidget ( Widget w, char * satname ) 
{
  XmString   str ;
  char     * txt ;
  Boolean    rt ;

  XtVaGetValues ( w, XmNlabelString, & str, NULL ) ;
  rt = XmStringGetLtoR (  str, XmFONTLIST_DEFAULT_TAG, & txt ) ;
  if ( rt != True ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : XmStringGetLtoR () Fail\n", __FILE__, __LINE__ ) ;
    #endif

    XmStringFree ( str ) ;
    return SKY_FAIL ;
  }

  strcpy ( satname, txt ) ;
  XtFree ( txt ) ;

  return SKY_SUCCESS ;
}
