/* 
 * Source Name : sky_sat_compare.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void Compare_LNB_Info_Reset ( void ) ;
void Compare_LNB_OldInfo_Set ( LNB_STRUCTURE * old_lnb ) ;
Sky_ReturnVal_E Compare_Old_With_LNB_NewInfo ( LNB_STRUCTURE * new_lnb ) ;

void Compare_TP_Info_Reset ( void ) ;
void Compare_TP_OldInfo_Set ( int count, TR_STRUCTURE * old_tp ) ;
Sky_ReturnVal_E Compare_Old_With_TP_NewInfo ( int count, TR_STRUCTURE * new_tp ) ;

void Compare_DATA_Info_Reset ( void ) ;
void Compare_DATA_OldInfo_Set ( int count, DATAPROGRAM_STRUCTURE * old_tp ) ;
Sky_ReturnVal_E Compare_Old_With_DATA_NewInfo ( int count, DATAPROGRAM_STRUCTURE * new_tp ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static LNB_STRUCTURE g_lnb_info ;

static int           g_num_of_tp ;
static TR_STRUCTURE  g_tp_info [ MAX_TRANSPONDER_NUM ] ;

static int                    g_num_of_data ;
static DATAPROGRAM_STRUCTURE  g_data_info [ MAX_DATA_NUM ] ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void Compare_LNB_Info_Reset ( void )
{
  memset ( & g_lnb_info, 0, sizeof ( LNB_STRUCTURE ) ) ; 
}
void Compare_LNB_OldInfo_Set ( LNB_STRUCTURE * old_lnb )
{
  memset ( & g_lnb_info, 0, sizeof ( LNB_STRUCTURE ) ) ; 
  memcpy ( & g_lnb_info, old_lnb, sizeof ( LNB_STRUCTURE ) );
}
Sky_ReturnVal_E Compare_Old_With_LNB_NewInfo ( LNB_STRUCTURE * new_lnb )
{
  /*     return SKY_SUCCESS, otherwise, return SKY_FAIL */
  if ( g_lnb_info . LocalOscillator != new_lnb -> LocalOscillator ) {
    return SKY_SUCCESS ;
  } else if ( g_lnb_info . SwitchOnOff != new_lnb -> SwitchOnOff ) {
    return SKY_SUCCESS ;
  } else if ( g_lnb_info . LNBPower != new_lnb -> LNBPower ) {
    return SKY_SUCCESS ;
  } else if ( g_lnb_info . ToneBurst != new_lnb -> ToneBurst ) {
    return SKY_SUCCESS ;
  } else if ( g_lnb_info . DiseqcPort != new_lnb -> DiseqcPort ) {
    return SKY_SUCCESS ;
  } else {
    return SKY_FAIL ;
  }
}
void Compare_TP_Info_Reset ( void )
{
  g_num_of_tp = 0 ;
  memset ( & g_tp_info, 0, sizeof ( g_tp_info ) ) ; 
}
void Compare_TP_OldInfo_Set ( int count, TR_STRUCTURE * old_tp )
{
  g_num_of_tp = 0 ;
  memset ( & g_tp_info, 0, sizeof ( g_tp_info ) ) ; 

  g_num_of_tp = count ;
  memcpy ( & g_tp_info, old_tp, sizeof ( TR_STRUCTURE ) * count );
}
Sky_ReturnVal_E Compare_Old_With_TP_NewInfo ( int count, TR_STRUCTURE * new_tp )
{
  int    i ;

  /*     return SKY_SUCCESS, otherwise, return SKY_FAIL */

  if ( g_num_of_tp != count ) {
    return SKY_SUCCESS ;
  }

  for ( i = 0 ; i < count ; i ++ ) {
    if ( g_tp_info [ i ] . DownlinkFrequency != new_tp [ i ] . DownlinkFrequency ) {
      return SKY_SUCCESS ;
    } else if ( g_tp_info [ i ] . SymbolRate != new_tp [ i ] . SymbolRate ) {
      return SKY_SUCCESS ;
    } else if ( g_tp_info [ i ] . SignalType != new_tp [ i ] . SignalType ) {
      return SKY_SUCCESS ;
    } 
  }
  return SKY_FAIL ;
}
void Compare_DATA_Info_Reset ( void )
{
  g_num_of_data = 0 ;
  memset ( & g_data_info, 0, sizeof ( g_data_info ) ) ; 
}
void Compare_DATA_OldInfo_Set ( int count, DATAPROGRAM_STRUCTURE * old_data )
{
  g_num_of_data = 0 ;
  memset ( & g_data_info, 0, sizeof ( g_data_info ) ) ; 

  g_num_of_data = count ;
  memcpy ( & g_data_info, old_data, sizeof ( DATAPROGRAM_STRUCTURE ) * count );
}
Sky_ReturnVal_E Compare_Old_With_DATA_NewInfo ( int count, DATAPROGRAM_STRUCTURE * new_data )
{
  int    i ;

  /*     return SKY_SUCCESS, otherwise, return SKY_FAIL */

  if ( g_num_of_data != count ) {
    return SKY_SUCCESS ;
  }

  for ( i = 0 ; i < count ; i ++ ) {
    if ( g_data_info [ i ] . DataPID != new_data [ i ] . DataPID ) {
      return SKY_SUCCESS ;
    } else if ( g_data_info [ i ] . Encapsulation != new_data [ i ] . Encapsulation ) {
      return SKY_SUCCESS ;
    }
  }
  return SKY_FAIL ;
}
