/* 
 * Source Name : sky_sat_osc_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif
#include <stdio.h>
#include <errno.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_sat_osc.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
Sky_ReturnVal_E LocalOscillatorInfo_Read ( int * count, int * osc_val ) ;
Sky_ReturnVal_E LocalOscillatorInfo_Write (  int count, int * osc_val ) ;

void OscillatorLister_Reset ( void ) ;
rcl_return_code OscillatorLister_Add ( int newId, char * );
rcl_return_code OscillatorLister_Remove ( int newId );
rcl_return_code OscillatorLister_GetPosById ( int newId, int * pos );
rcl_return_code OscillatorLister_GetIdByPos ( int pos, int * newId ) ;
void ConvertInt2Str_LocalOscillator_2 ( unsigned int x, char * txt ) ;
void List_SelectedScrolled ( Widget w, int pos, int page ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
addsat_oscUpBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void LocalOscillatorList_FirstDisplay ( void ) ;
void LocalOscillatorList_SecondDisplay ( void ) ;
void LocalOscillatorList_Compare ( int newVal ) ;
Sky_ReturnVal_E LocalOscillatorList_CompareDefaultValue ( int newVal ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
addsat_oscUpBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  #ifdef DEBUG
  printf ( "addsat_oscUpBCallback\n" ) ;
  #endif

  XtMapWidget ( addsat_oscForm ) ;
  LocalOscillatorList_SecondDisplay ( ) ;
}
void
addsat_oscDelBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  char  * txt ;
  float   f_tmp ;
  int     newVal ;
  int     pos ;
  char    buf [ 20 ] ;

  /*--------------------------------------------------------*/
  /* Get LocalOscillator Value */
  /*--------------------------------------------------------*/
  txt = ( char * ) XmTextGetString ( addsat_oscT ) ;
  if ( *txt == ( char ) NULL ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : empty Local Oscillator\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  if ( ! strcmp ( txt, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Local Oscillator Default Value ( %s )-> not Delete\n", __FILE__, __LINE__, txt ) ;
    #endif

    XtFree ( txt ) ;
    return ;
  } 

  sscanf ( txt, "%f", & f_tmp ) ;
  newVal = f_tmp * 1000 ;
  XtFree ( txt ) ;

  /*--------------------------------------------------------*/
  /* Default ̸  ʴ´. */
  /*--------------------------------------------------------*/
  /*  -> return SKY_FAIL, otherwise, return SKY_SUCCESS */
  if ( LocalOscillatorList_CompareDefaultValue ( newVal ) != SKY_SUCCESS ) {
    return ;
  }

  /*--------------------------------------------------------*
   * List Ͽ  Ѵ, 
   *   List Item ϰ, ũѸϰ,
   * Text Value   Ѵ.
   *--------------------------------------------------------*/
  if ( OscillatorLister_GetPosById ( newVal, & pos ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : This Oscillator value ( %d ) doesn't exist in list\n", __FILE__, __LINE__, newVal ) ;
    #endif

    return ;
  }
  if ( OscillatorLister_Remove ( newVal ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : OscillatorLister_Remove () Fail. val = %d\n",  __FILE__, __LINE__, newVal ) ;
    #endif

    return ;
  }

  if ( OscillatorLister_GetIdByPos ( pos - 1, & newVal ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : This Oscillator pos ( %d ) doesn't exist in list\n", __FILE__, __LINE__, pos ) ;
    #endif

    return ;
  }
#ifdef LINUX_VERSION
  List_SelectedScrolled ( addsat_oscListLister, pos, 5 ) ;
#else
  List_SelectedScrolled ( addsat_oscListLister, pos, 5 ) ;
#endif

  /* Set Text Value */
  ConvertInt2Str_LocalOscillator_2 ( (newVal/1000 ), buf ) ;
  XmTextSetString ( addsat_oscT, buf ) ;

}
void
addsat_oscListListerBrowseCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmListCallbackStruct *call_data;
{
  XmListCallbackStruct * data = call_data ;
  int                    position ;
  XmString               xmStr ; 
  char                 * txt ;
  Boolean                rt ;

  /*--------------------------------------------------------*/
  /* Get Position/String */
  /*--------------------------------------------------------*/
  position = data -> item_position ;
  xmStr = data -> item ;

  /*--------------------------------------------------------*/
  /* Unmap Oscillator List Widget */
  /*--------------------------------------------------------*/
  XtUnmapWidget ( addsat_oscForm ) ;
  
  /*--------------------------------------------------------*/
  /* Get String from XmString */
  /*--------------------------------------------------------*/
  rt = XmStringGetLtoR ( xmStr, XmFONTLIST_DEFAULT_TAG, & txt ) ;
  if ( rt != True ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : XmStringGetLtoR () Fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  #ifdef DEBUG
  printf ( "%s : %d : text = %s\n", __FILE__, __LINE__, txt ) ;
  #endif

  /*--------------------------------------------------------*/
  /* Set String To Oscillator Text */
  /*--------------------------------------------------------*/
  XmTextSetString ( addsat_oscT, txt ) ;
  XtFree ( txt ) ;

  /*--------------------------------------------------------*/
  /* Oscillator Value == 0̸, diable, ƴϸ enable Ѵ. */
  /*--------------------------------------------------------*/
  if ( !strcmp ( txt, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ) {
    DisableWidget ( addsat_22kRB ) ;
  } else {
    EnableWidget ( addsat_22kRB ) ;
  }
}
void LocalOscillatorList_FirstDisplay ( void )
{
  int  i ;
  int  count ;
  int  osc_val [ MAX_LOCAL_OSCILATOR_NUM ] ;
  char txt [ 20 ] ;

  #ifdef DEBUG
  printf ( "%s : %d : LocalOscillatorList_Display ()\n", __FILE__, __LINE__ ) ;
  #endif

  LocalOscillatorInfo_Read ( & count, osc_val ) ;

  #ifdef DEBUG
  printf ( "%s : %d : count = %d, osc_val [ 0 ] = %d\n", __FILE__, __LINE__, count, osc_val [ 0 ] ) ;
  #endif

  OscillatorLister_Reset ( ) ;
  
  for ( i = 0 ; i < count ; i ++ ) {
    ConvertInt2Str_LocalOscillator_2 ( ( osc_val [ i ] / 1000 ) , txt ) ;
    OscillatorLister_Add ( osc_val [ i ], txt );
  }
}

void LocalOscillatorList_Compare ( int newVal )
{
  int    i ;
  int    pos ;
  char   txt [ 20 ] ;

  /*--------------------------------------------------------*/
  /*  -> ο LocalOscillator ̸ list Ѵ. */
  /*--------------------------------------------------------*/
  if ( OscillatorLister_GetPosById ( newVal, & pos ) == RCL_SUCCESS ) {

    #ifdef DEBUG
    printf ( "%s : %d : OscillatorLister_GetPosById () succ. newVal = %d, pos = %d\n", __FILE__, __LINE__, newVal, pos ) ;
    #endif

    return ;
  }

  /*--------------------------------------------------------*/
  /* List Add Ѵ. */
  /*--------------------------------------------------------*/
  ConvertInt2Str_LocalOscillator_2 ( newVal / 1000, txt ) ;
  OscillatorLister_Add ( newVal, txt );
  
  if ( OscillatorLister_GetPosById ( newVal, & pos ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : OscillatorLister_GetPosById () Fail, newval = %d\n", __FILE__, __LINE__, newVal ) ;
    #endif

    return ;
  }

  #ifdef DEBUG_FAIL
  printf ( "%s : %d : OscillatorLister_GetPosById () Succ. Val = %d, pos = %d\n", __FILE__, __LINE__, newVal, pos ) ;
  #endif


  XmListSelectPos ( addsat_oscListLister, pos + 1, False ) ;
  XmListSetHorizPos ( addsat_oscListLister, pos + 1 ) ;
}

/* Default Value ̸-> return SKY_FAIL, otherwise, return SKY_SUCCESS */
Sky_ReturnVal_E LocalOscillatorList_CompareDefaultValue ( int newVal ) 
{
  if ( newVal == LOCAL_OSCILLATOR_DEFAULT_VAL_1 ) {
    return SKY_FAIL ;
  } else if ( newVal == LOCAL_OSCILLATOR_DEFAULT_VAL_2 ) {
    return SKY_FAIL ;
  } else if ( newVal == LOCAL_OSCILLATOR_DEFAULT_VAL_3 ) {
    return SKY_FAIL ;
  } else if ( newVal == LOCAL_OSCILLATOR_DEFAULT_VAL_4 ) {
    return SKY_FAIL ;
  } else if ( newVal == LOCAL_OSCILLATOR_DEFAULT_VAL_5 ) {
    return SKY_FAIL ;
  } else {
    return SKY_SUCCESS ;
  }
}

void LocalOscillatorList_SecondDisplay ( void )
{
  char  *  txt ;
  int      val ;
  int      pos ;
  float    f_tmp ;

  #ifdef DEBUG
  printf ( "LocalOscillatorList_SecondDisplay ()\n" ) ;
  #endif

  /*  addsat_oscT ִ  list Ͽ    select/scrollingѴ. */
  txt = ( char * ) XmTextGetString ( addsat_oscT ) ;
  if ( * txt == ( char ) NULL ) {

    #ifdef DEBUG
    printf ( "%s : %d : empty oscillator value\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  if ( ! strcmp ( txt, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ) {
    val = 0 ;
  } else {
    sscanf ( txt, "%f", & f_tmp ) ;
    val = ( int ) ( f_tmp * 1000.0 ) ;
  }
  XtFree ( txt ) ;

  if ( OscillatorLister_GetPosById ( val, & pos ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : This Oscillator value ( %d ) doesn't exist in list\n", __FILE__, __LINE__, val ) ;
    #endif

    return ;
  }
#ifdef LINUX_VERSION
  List_SelectedScrolled ( addsat_oscListLister, pos + 1, 5 ) ;
#else
  List_SelectedScrolled ( addsat_oscListLister, pos + 1, 5 ) ;
#endif
}
