/* 
 * Source Name : sky_sat_osc_file.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"
#include "sky_sat_osc.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
rcl_return_code OscillatorLister_GetIdByPos ( int pos, int * newId ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
Sky_ReturnVal_E LocalOscillatorInfo_Read ( int * count, int * osc_val ) ;
Sky_ReturnVal_E LocalOscillatorInfo_Write ( void ) ; 

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
Sky_ReturnVal_E LocalOscillatorInfo_Read ( int * count, int * osc_val )
{
  int            i ;
  int            rt ;
  int            fd ;
  char         * name = LOCAL_OSCILATOR_FILE_NAME ;
  struct stat    fileSts ;

  #ifdef DEBUG_FAIL
  printf ( "%s : %d : LocalOscillatorInfo_Read ()\n", __FILE__, __LINE__ ) ;
  #endif

  /* Default Value Setting */
  * count = 5 ;
  osc_val [ 0 ] = LOCAL_OSCILLATOR_DEFAULT_VAL_1 ;
  osc_val [ 1 ] = LOCAL_OSCILLATOR_DEFAULT_VAL_2 ;
  osc_val [ 2 ] = LOCAL_OSCILLATOR_DEFAULT_VAL_3 ;
  osc_val [ 3 ] = LOCAL_OSCILLATOR_DEFAULT_VAL_4 ;
  osc_val [ 4 ] = LOCAL_OSCILLATOR_DEFAULT_VAL_5 ;

  fd = open ( name, O_RDWR ) ;
  if ( fd < 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : open ( %s , ) fail. errno = %d\n", __FILE__, __LINE__, name, errno ) ;
    #endif

    fd = open ( name, O_CREAT | O_WRONLY, 0644 ) ;
    if ( fd < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s creat fail. errno = %d\n", name, errno ) ;
      #endif

    } else {
      write ( fd, ( char * ) & osc_val, sizeof ( int ) * ( * count ) ) ;
      close ( fd ) ;
    }

  } else {
    rt = fstat ( fd, & fileSts ) ;  
    if ( rt < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : fstate ( %s ) fail. errno = %d\n", __FILE__, __LINE__, name, errno ) ;
      #endif

    } else {
      *count = fileSts.st_size/sizeof ( int ) ;
      for ( i = 0 ; i < * count ; i ++ ) {
        rt = read ( fd, & osc_val [ i ], sizeof ( int ) ) ;
        if ( rt < 0 ) {

          #ifdef DEBUG_FAIL
          printf ( "%s : %d : read ( %s ) fail. errno = %d\n", __FILE__, __LINE__, name, errno ) ;
          #endif

          break ;
        } 
      }
    }
    close ( fd ) ;
  }

  return SKY_SUCCESS ;
}
Sky_ReturnVal_E LocalOscillatorInfo_Write ( void )
{
  int            i ;
  int            newVal ;
  int            rt ;
  int            fd ;
  char         * name = LOCAL_OSCILATOR_FILE_NAME ;

  fd = open ( name, O_CREAT | O_RDWR, 0644 ) ;
  if ( fd < 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : open/create ( %s ) fail. errno = %d\n", __FILE__, __LINE__, name, errno ) ;
    #endif

    return SKY_FAIL ;
  } 

  for ( i = 0 ; ; i ++ ) { 

    if ( OscillatorLister_GetIdByPos ( i, & newVal ) != RCL_SUCCESS ) {
      break ;
    }
    rt = write ( fd, & newVal, sizeof ( int ) ) ;

    if ( rt < 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : [ %d ] write ( %s ) fail. errno = %d\n", __FILE__, __LINE__, i, name, errno ) ;
      #endif

      break ;
    } else {

      #ifdef DEBUG
      printf ( "%s : %d : [ %d ] write ( %s ) succ. errno = %d\n", __FILE__, __LINE__, i, name, errno ) ;
      #endif

    }
  }
  close ( fd ) ;
  return SKY_SUCCESS ;
}
