/* 
 * Source Name : sky_oscilator_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_sat_osc.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void Oscillator_SetCurrentOscillatorId ( int newId ) ;
int Oscillator_GetCurrentOscillatorId ( void ) ;

void OscillatorLister_Reset ( void ) ;
rcl_return_code OscillatorLister_Add ( int newId, char * );
rcl_return_code OscillatorLister_Remove ( int newId );
rcl_return_code OscillatorLister_GetPosById ( int newId, int * pos );
rcl_return_code OscillatorLister_GetIdByPos ( int pos, int * newId ) ;
rcl_return_code OscillatorList_SetNotSelected ( int numPos, int * pos_list ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int                   current_oscilator_id_g = 0 ;
static OscillatorList_S     * oscilatorList_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentOscillatorId ( int newId )
{
   current_oscilator_id_g = newId ;
}
int GetCurrentOscillatorId ( void )
{
   return current_oscilator_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void OscillatorLister_Reset ( void ) 
{
   OscillatorList_S  *  curr ;
   OscillatorList_S  *  next ;

   XmListDeleteAllItems ( addsat_oscListLister ) ;

   if ( oscilatorList_g == NULL ) {
      return ;
   }

   curr = oscilatorList_g ;
   next = oscilatorList_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   oscilatorList_g = NULL ;
}
rcl_return_code OscillatorLister_Add ( int newId, char * txt )
{
   OscillatorList_S  ** curr ;
   XmString        xmstr ;

   xmstr = ( XmString ) strToXmString ( txt ) ;
 
   if ( oscilatorList_g == NULL ) {
      oscilatorList_g = ( OscillatorList_S * ) malloc ( sizeof ( OscillatorList_S ) ) ;
      oscilatorList_g -> id = newId ;
      oscilatorList_g -> next = NULL ;

      XmListAddItem ( addsat_oscListLister, xmstr, 0 ) ;
      XmStringFree ( xmstr ) ;

      return RCL_SUCCESS ;
   }

   curr = & oscilatorList_g ;
   while ( 1 ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( OscillatorList_S * ) malloc ( sizeof ( OscillatorList_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         XmListAddItem ( addsat_oscListLister, xmstr, 0 ) ;
         XmStringFree ( xmstr ) ;

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;
   }
   return RCL_SUCCESS ;
}
rcl_return_code OscillatorLister_Remove ( int newId )
{
   OscillatorList_S  ** prev ;
   OscillatorList_S  ** curr ;
   int                  pos = 0 ;
   
   if ( oscilatorList_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( oscilatorList_g -> next == NULL ) {
      if ( oscilatorList_g -> id == newId ) {
         free ( oscilatorList_g ) ;
         oscilatorList_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & oscilatorList_g ;
   curr = & oscilatorList_g ;
   while ( ( *curr ) ) {

      if ( ( * curr ) -> id == newId ) {

         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;

         XmListDeletePos ( addsat_oscListLister, pos + 1 ) ;
         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;

      pos ++ ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code OscillatorLister_GetPosById ( int newId, int * pos )
{
   int               i ;
   OscillatorList_S  * curr ;
   
   if ( oscilatorList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = oscilatorList_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code OscillatorLister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   OscillatorList_S  * curr ;
   
   if ( oscilatorList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = oscilatorList_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}
rcl_return_code OscillatorList_SetNotSelected ( int numPos, int * pos_list ) 

{
   int               i ;
   OscillatorList_S  * curr ;
   
   if ( oscilatorList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   for ( i = 0 ; i < numPos ; i ++ ) {
     XmListDeselectPos ( addsat_oscListLister, pos_list [ i ] + 1 ) ; 
   }

   return RCL_SUCCESS ;
}

