/* 
 * Source Name : sky_save.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include <sky_type.h>

#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
Sky_ReturnVal_E read_initfile ( Sky_LastInfo_S * lastinfo_p ) ;
void SAT_ValChg_Reset ( void ) ;

int DoMsgBoxPopUp ( void ) ;
void SetMsgBoxPopUp ( void ) ;
void ResetMsgBoxPopUp ( void ) ;

void LockStatus_DisplayToTitle ( char * , TR_STRUCTURE * tp_p, int sts ) ;
void LockStatus_ResetDisplayToTile ( char * ) ;
void ResetNetworkPIDs ( int tp_id ) ;
Sky_ReturnVal_E APP_Sky_ReleaseAllNetworkPID ( int tp_id ) ;

void data_menu_SATBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void data_menu_PEBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void SaveIpAddressInform ( void ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
data_okBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_satAddCC_CCCallback ( Widget w, XtPointer client_data, XmAnyCallbackStruct *call_data ) ;
void
tuner_satInfoB1_1Callback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;
void
tuner_satOtherBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;


/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void SaveSatelliteInfo ( Widget w ) ;
void TunerLockTest_BeforeMinimize ( Widget w ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
extern PixmapResources_t pixmap_resources;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
data_okBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  XIconifyWindow ( XtDisplay ( data_shell ), XtWindow ( data_shell ), 0 ) ;
  SaveIpAddressInform ( ) ; /* add 1999.11.23 */
  SaveSatelliteInfo ( w ) ;
  TunerLockTest_BeforeMinimize ( w ) ;
}
void SaveSatelliteInfo ( Widget w ) 
{
  Sky_LastInfo_S   sky_r ;

  /*------------------------------------------------------------------------*/
  /* Save Satellite & Transponder Information */
  /*------------------------------------------------------------------------*/
  read_initfile ( & sky_r ) ;

  /*----------------------------------------------------------------*/
  /* Open Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileOpen ( sky_r . satellite_file ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileOpen () Fail [%s]\n", __FILE__, __LINE__, sky_r . satellite_file  ) ;
     #endif

     return ;
  }
  /*----------------------------------------------------------------*/
  /* Read Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileWrite ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileWrite () Fail [%s]\n", __FILE__, __LINE__, sky_r . satellite_file  ) ;
     #endif

     return ;
  }
  /*----------------------------------------------------------------*/
  /* Close Satellite File */
  /*----------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : Sky_ChannelFileClose () Fail [%s]\n", __FILE__, __LINE__, sky_r . satellite_file  ) ;
     #endif

     return ;
  }

  /*------------------------------------------------------------------------*/
  /* Changed Information All Reset */
  /*------------------------------------------------------------------------*/
  SAT_ValChg_Reset ( ) ;
}

void
data_cancelBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  Sky_LastInfo_S   sky_r ;

  /*------------------------------------------------------------------------*/
  /* Changed Information All Reset */
  /*------------------------------------------------------------------------*/
  SAT_ValChg_Reset ( ) ;

  /*------------------------------------------------------------------------*/
  /* Iconify */
  /*------------------------------------------------------------------------*/
  XIconifyWindow ( XtDisplay ( data_shell ), XtWindow ( data_shell ), 0 ) ;

  /*------------------------------------------------------------------------*/
  /* initfile */
  /*------------------------------------------------------------------------*/
  read_initfile ( & sky_r ) ;

  /*------------------------------------------------------------------------*/
  /* Free Queue */
  /*------------------------------------------------------------------------*/
  if ( Sky_FreeTransponderStructure ( ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_FreeTransponderStructure () Fail. File = %s\n", __FILE__, __LINE__, sky_r . satellite_file ) ;
    #endif

  }
  /*------------------------------------------------------------------------*/
  /* Satellite File Re-Load */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileOpen ( sky_r . satellite_file ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ChannelFileOpen () Fail. File = %s\n", __FILE__, __LINE__, sky_r . satellite_file ) ;
      #endif

      return ;

   }

  /*------------------------------------------------------------------------*/
  /* Open Read */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileRead ( ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ChannelFileRead () Fail. File = %s\n", __FILE__, __LINE__, sky_r . satellite_file) ;
      #endif

      Sky_ChannelFileClose () ;
      return ;
  }

  /*------------------------------------------------------------------------*/
  /* Close */
  /*------------------------------------------------------------------------*/
  if ( Sky_ChannelFileClose ( ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_ChannelFileClose () Fail. File = %s\n", __FILE__, __LINE__, sky_r . satellite_file ) ;
    #endif

    return ;
  }

  /*------------------------------------------------------------------------*/
  /* Re-Display Satellite Inform */
  /* Re-Display Data Inform */
  /*------------------------------------------------------------------------*/
  data_menu_SATBCallback ( NULL, NULL, NULL ) ;
  data_menu_PEBCallback ( NULL, NULL, NULL ) ;

}

void TunerLockTest_BeforeMinimize ( Widget w ) 
{
  LNB_STRUCTURE  lnbinfo ;
  TR_STRUCTURE   tpinfo ;
  int            ret2 ;

  Sky_LastInfo_S   sky_r ;

  /*------------------------------------------------------------------------*/
  /* Save Satellite & Transponder Information */
  /*------------------------------------------------------------------------*/
  read_initfile ( & sky_r ) ;

  /*------------------------------------------------------------------------*/
  /* Lock Test Current Transponder */
  /*------------------------------------------------------------------------*/
   if ( Sky_GetSatelliteInformation ( & lnbinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetSatelliteInformation () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }

   if ( ( sky_r . transponder_id <= 0 ) && Sky_GetNumOfTransponder () ) {
     sky_r . transponder_id = 1 ;
   }

   if ( Sky_GetInfoTransponder ( sky_r . transponder_id, & tpinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      return ;
   }

   /*if ( APP_Sky_ReleaseAllNetworkPID ( sky_r . transponder_id ) != SKY_SUCCESS ) {*/
   if ( Sky_ReleaseAllNetworkPID ( ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ReleaseAllNetworkPID () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

   }

   ret2 = Sky_TunerLockTest ( lnbinfo, tpinfo ) ;
   if ( ret2 != SKY_OK ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_TunerLockTest () Fail.\n", __FILE__, __LINE__ ) ;
      #endif
   } else {
      /*---------------------------------------------------------------*/
      /* Reset Network PID */
      /*---------------------------------------------------------------*/
      ResetNetworkPIDs ( tpinfo . TransponderNumber ) ;
   }

  /*------------------------------------------------------------------------*/
  /* Display Title */
  /*------------------------------------------------------------------------*/
  LockStatus_DisplayToTitle ( ( char * )lnbinfo.SatelliteName, & tpinfo, ret2 ) ;

  /*------------------------------------------------------------------------*/
  /* Set Icon Pixmap*/
  /*------------------------------------------------------------------------*/
  if ( ret2 != SKY_OK ) {
    XtVaSetValues ( data_shell, XmNiconPixmap, pixmap_resources . unlock_icon, NULL ) ;
  } else {
    XtVaSetValues ( data_shell, XmNiconPixmap, pixmap_resources . lock_icon, NULL ) ;
  }
}
