#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include "sky_extern.h"
#include "sky_type.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void LockStatus_DisplayToTitle ( char * , TR_STRUCTURE * tp_p, int sts ) ;
void ResetAllNetworkPIDs ( SI_STRUCTURE * si_p ) ;

void my_SetWatchCursor ( Widget w1 ) ;
void my_ResetCursor ( Widget w1 ) ;

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void scan_channel ( Widget w, int ) ;
void display_unlock_scan_shell ( void )  ;
void display_lock_scan_shell ( void )  ;
void disappear_scan_shell ( XtPointer x, XtIntervalId * interval ) ;
void ResetNetworkPIDs ( int tp_id ) ;

/**************************************************************************
 * Global Variable
 **************************************************************************/
extern PixmapResources_t pixmap_resources;

void scan_channel ( Widget w, int TransponderNumber )
{
   LNB_STRUCTURE   lnbinfo ;
   TR_STRUCTURE    tpinfo ;
   SI_STRUCTURE    siinfo ;

   Sky_LastInfo_S  sky_p ;
   int             rt ;

   if ( Sky_GetSatelliteInformation ( & lnbinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetSatelliteInformation () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( w, "Warning!!\nFail to get Satellte information" ) ;
      return ;
   }

   if ( Sky_GetInfoTransponder ( TransponderNumber , & tpinfo ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_GetInfoTransponder () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( w, "Warning!!\nFail to get Transponder information" ) ;
      return ;
   }

   /*if ( APP_Sky_ReleaseAllNetworkPID ( TransponderNumber ) != SKY_SUCCESS ) {*/
   if ( Sky_ReleaseAllNetworkPID ( ) != FALSE ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : Sky_ReleaseAllNetworkPID () Fail.\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( w, "Warning!!\nFail to release All Network PID" ) ;
      return ;
   }
   /*----------------------------------------------------------------------*/
   /* Initfile read */
   /*----------------------------------------------------------------------*/
   read_initfile ( & sky_p ) ;

   rt = Sky_TunerLockTest ( lnbinfo, tpinfo )  ;

   sky_p . transponder_id = tpinfo . TransponderNumber ;
   write_initfile ( & sky_p ) ;

   /*----------------------------------------------------------------------*/
   /* Map Scan Shell */
   /*----------------------------------------------------------------------*/
   XtManageChild ( scan_shell ) ;

   if ( rt != SKY_OK ) {
      display_unlock_scan_shell ( ) ;
   } else {

      if ( Sky_SearchChannel ( _WAIT, & siinfo ) != FALSE ) {
         display_unlock_scan_shell ( ) ;

         #ifdef DEBUG
         printf ( "%s : %d : before ResetNetworkPIDs().\n", __FILE__, __LINE__ ) ;
         #endif

         ResetNetworkPIDs ( TransponderNumber ) ;
      } else {
         display_lock_scan_shell ( ) ;

         #ifdef DEBUG
         printf ( "%s : %d : before ResetAllNetworkPIDs().\n", __FILE__, __LINE__ ) ;
         #endif

         ResetAllNetworkPIDs ( & siinfo ) ;
      }

   }

   /*----------------------------------------------------------------------*/
   /* Display Lock Status to Title */
   /*----------------------------------------------------------------------*/
   LockStatus_DisplayToTitle ( ( char * ) lnbinfo.SatelliteName, & tpinfo, rt ) ;
}
void display_unlock_scan_shell ( void ) 
{
   Arg      args [ 2 ] ;
   int      ac ;
   XmString xmstr ;

   /* TITLE */
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNtitle, WD_SCAN_ERROR_TITLE ) ;
   ac ++ ;
   XtSetValues ( scan_shell, args, ac ) ;

   /* TUner Status */
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Red ) ;
   ac ++ ;
   XtSetValues ( scan_tunerSts, args, ac ) ;
   XtSetValues ( scan_qpskSts, args, ac ) ;
   XtSetValues ( scan_viterbiSts, args, ac ) ;
   XtSetValues ( scan_frameSts, args, ac ) ;

   /* Label */
   xmstr = XmStringCreate ( WD_SCAN_ERROR_LABEL, XmFONTLIST_DEFAULT_TAG ) ;
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
   ac ++ ;
   XtSetValues ( scan_msg, args, ac ) ;
   XmStringFree ( xmstr ) ;

   /* Button */
   xmstr = XmStringCreate ( WD_CANCEL_BUTTON_LABEL, XmFONTLIST_DEFAULT_TAG ) ;
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
   ac ++ ;
   XtSetValues ( scan_okB, args, ac ) ;
   XmStringFree ( xmstr ) ;
}
void display_lock_scan_shell ( void ) 
{
   Arg           args [ 2 ] ;
   int           ac ;
   XmString      xmstr ;
   XtIntervalId  interval ;
   char          txt [ 100 ] ;

   /* TITLE */
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNtitle, WD_SCAN_OK_TITLE ) ;
   ac ++ ;
   XtSetValues ( scan_shell, args, ac ) ;

   /* TUner Status */
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelPixmap, pixmap_resources . Green ) ;
   ac ++ ;
   XtSetValues ( scan_tunerSts, args, ac ) ;
   XtSetValues ( scan_qpskSts, args, ac ) ;
   XtSetValues ( scan_viterbiSts, args, ac ) ;
   XtSetValues ( scan_frameSts, args, ac ) ;

   /* Label */
   xmstr = XmStringCreate ( WD_SCAN_OK_LABEL, XmFONTLIST_DEFAULT_TAG ) ;
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
   ac ++ ;
   XtSetValues ( scan_msg, args, ac ) ;
   XmStringFree ( xmstr ) ;

   /* Button */
   sprintf ( txt, "    %s    ", WD_OK_BUTTON_LABEL ) ;
   xmstr = XmStringCreate ( txt, XmFONTLIST_DEFAULT_TAG ) ;
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ;
   ac ++ ;
   XtSetValues ( scan_okB, args, ac ) ;
   XmStringFree ( xmstr ) ;

   interval = XtAppAddTimeOut ( XtWidgetToApplicationContext ( scan_shell ), SCAN_TEST_TIME_OUT, & disappear_scan_shell, NULL ) ;

}
void disappear_scan_shell ( XtPointer x, XtIntervalId * interval )
{
   #ifdef DEBUG
   printf ( "disappear_scan_shell ()\n" ) ;
   #endif

   XtUnmanageChild ( scan_board ) ;
}

void
tuner_scanBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
   int       TransponderNumber ;
   Boolean   rt ;
   int     * position ;
   int       count ;

   #ifdef DEBUG
   printf ( "tuner_scanBCallback\n" ) ;
   #endif

   if ( ! Sky_GetNumOfTransponder () ) {
     DisplayErrorDialog ( w, "Warning !!\nThere is No Transponder\n" ) ;
     return ;
   }

#if 1
   rt = XmListGetSelectedPos ( tuner_satinfo_FqSyList_list, & position, & count ) ;
   if ( rt != True ) {
     #ifdef DEBUG_FAIL
     printf ( "%s : %d : XmListGetSelectedPos () Fail\n", __FILE__, __LINE__ ) ;
     #endif

     DisplayErrorDialog ( w, "Warning !!\nSelect a Transponder" ) ;
     return ;

   } else if ( count == 0 ) {
     DisplayErrorDialog ( w, "Warning !!\nSelect a Transponder" ) ;
     return ;
   }
   TransponderNumber = *position ;
#else
   TransponderNumber = XmListGetKbdItemPos ( tuner_satinfo_FqSyList_list ) ;
   if ( TransponderNumber <= 0 ) {

      #ifdef DEBUG_FAIL
      printf ( "%s : %d : UnSelected Transponder\n", __FILE__, __LINE__ ) ;
      #endif

      DisplayErrorDialog ( w, "Warning!!\nFail to get Transponder ID" ) ;
      return ;
   }
#endif

   #ifdef DEBUG_FAIL
   printf ( "%s : %d : Selected Transponder = %d\n", __FILE__, __LINE__, TransponderNumber ) ;
   #endif

   XtManageChild ( scan_board ) ;

   scan_channel ( w, TransponderNumber ) ;
}

void
scan_shell_popupCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
#if 0
  DisableWidget ( data_board ) ;
#else
  my_SetWatchCursor ( data_shell ) ;
#endif
}
void
scan_shell_popdownCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
#if 0
  EnableWidget ( data_board ) ;
#else
  my_ResetCursor ( data_shell ) ;
#endif
}

