/* 
 * Source Name : sky_setdata_add_del.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_data.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/
rcl_return_code AvailableDataLister_GetIdByPos ( int pos, int * newId ) ;
rcl_return_code SetDataLister_GetPosById ( int newId, int * pos );

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/
void
data_PE_addBCallback ( Widget w, XtPointer client_data, XmPushButtonCallbackStruct *call_data ) ;

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void
data_PE_addBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  int     pos ;
  int     count ;
  int   * pos_p ;
  int     DataNumber ;
  int     setdata_pos ;

  DATAPROGRAM_STRUCTURE  data_r ;
  SETDATA_STRUCTURE      setdata_r ;

  /*-------------------------------------------------*/
  /* Get selected data PID */
  /*-------------------------------------------------*/
  XmListGetSelectedPos ( data_PE_form1_listLister, & pos_p, & count ) ;
  if ( count == 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Unselected Data\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  pos = * pos_p ;
  
  /*-------------------------------------------------*/
  /* Compare Data Number : Already exist ? */
  /*-------------------------------------------------*/
  if ( AvailableDataLister_GetIdByPos ( pos - 1, & DataNumber ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : AvailableDataLister_GetIdByPos () fail \n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  if ( SetDataLister_GetPosById ( DataNumber, & setdata_pos ) == RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : SetDataLister_GetPosById () succ -> already exist data pid \n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Get Data Information */
  /*-------------------------------------------------*/
  memset ( & data_r, 0, sizeof ( data_r ) ) ;
  data_r . DataNumber = DataNumber ;
  if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_GetInfoData () succ -> already exist data pid \n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Make Message */
  /*-------------------------------------------------*/
  memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
  setdata_r . DataPID       = data_r . DataPID ;
  setdata_r . Encapsulation = data_r . Encapsulation ;
  setdata_r . pgnumber      = data_r . pgnumber ;
  setdata_r . PmtPID        = data_r . PmtPID ;

  /*-------------------------------------------------*/
  /* Set network PID */
  /*-------------------------------------------------*/
  if ( Sky_SetNetworkPID ( setdata_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_SetNetworkPID () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Update Data */
  /*-------------------------------------------------*/
  data_r . SetFlag = 1 ;
  if ( Sky_UpdateData ( data_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_UpdateData () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  
  /*-------------------------------------------------*/
  /* Re-Fresh Data Panel */
  /*-------------------------------------------------*/
  DisplayDataInfoToPanel ( data_menu_PE, data_r . DataNumber / 100 ) ;

  /*------------------------------------------------------*/
  /* Satellite Information Changed Flag Setting */
  /*------------------------------------------------------*/
  SAT_ValChg_Set ( ) ;

  /*------------------------------------------------------*/
  /* Select Available Data List Position */
  /*------------------------------------------------------*/
  XmListSelectPos ( data_PE_form1_listLister, pos, False ) ;

  /*------------------------------------------------------*/
  /* Select Set Data List Position */
  /*------------------------------------------------------*/
  if ( SetDataLister_GetPosById ( DataNumber, & setdata_pos ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : SetDataLister_GetPosById () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  XmListSelectPos ( data_PE_form3_list_lister, setdata_pos + 1, False ) ;

 
}
void
data_PE_removeBCallback (w, client_data, call_data )
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *call_data;
{
  int     pos ;
  int     count ;
  int   * pos_p ;
  int     DataNumber ;
  int     setdata_pos ;

  DATAPROGRAM_STRUCTURE  data_r ;
  SETDATA_STRUCTURE      setdata_r ;

  /*-------------------------------------------------*/
  /* Get selected data PID */
  /*-------------------------------------------------*/
  XmListGetSelectedPos ( data_PE_form3_list_lister, & pos_p, & count ) ;
  if ( count == 0 ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Unselected Data\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  pos = * pos_p ;
  
  /*-------------------------------------------------*/
  /* Compare Data Number : Already exist ? */
  /*-------------------------------------------------*/
  if ( SetDataLister_GetIdByPos ( pos - 1, & DataNumber ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : AvailableDataLister_GetIdByPos () fail \n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Get Data Information */
  /*-------------------------------------------------*/
  memset ( & data_r, 0, sizeof ( data_r ) ) ;
  data_r . DataNumber = DataNumber ;
  if ( Sky_GetInfoData ( & data_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_GetInfoData () succ -> already exist data pid \n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Make Message */
  /*-------------------------------------------------*/
  memset ( & setdata_r, 0, sizeof ( setdata_r ) ) ;
  setdata_r . DataPID       = data_r . DataPID ;
  setdata_r . Encapsulation = data_r . Encapsulation ;
  setdata_r . pgnumber      = data_r . pgnumber ;
  setdata_r . PmtPID        = data_r . PmtPID ;

  /*-------------------------------------------------*/
  /* Release network PID */
  /*-------------------------------------------------*/
  if ( Sky_ReleaseNetworkPID ( setdata_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_ReleaseNetworkPID () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  /*-------------------------------------------------*/
  /* Update Data */
  /*-------------------------------------------------*/
  data_r . SetFlag = 0 ;
  if ( Sky_UpdateData ( data_r ) != FALSE ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : Sky_UpdateData () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }

  
  /*-------------------------------------------------*/
  /* Re-Fresh Data Panel */
  /*-------------------------------------------------*/
  DisplayDataInfoToPanel ( data_menu_PE, data_r . DataNumber / 100 ) ;

  /*------------------------------------------------------*/
  /* Satellite Information Changed Flag Setting */
  /*------------------------------------------------------*/
  SAT_ValChg_Set ( ) ;

  /*------------------------------------------------------*/
  /* Select List Position */
  /*------------------------------------------------------*/
  if ( AvailableDataLister_GetPosById ( DataNumber, & setdata_pos ) != RCL_SUCCESS ) {

    #ifdef DEBUG_FAIL
    printf ( "%s : %d : SetDataLister_GetPosById () fail\n", __FILE__, __LINE__ ) ;
    #endif

    return ;
  }
  XmListSelectPos ( data_PE_form1_listLister, setdata_pos + 1, False ) ;
  XmListSelectPos ( data_PE_form3_list_lister, 1, False ) ;
}
