/* 
 * Source Name : sky_setdata_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_data.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void SetData_SetCurrentSetDataId ( int newId ) ;
int SetData_GetCurrentSetDataId ( void ) ;

void SetDataLister_Reset ( void ) ;
rcl_return_code SetDataLister_Add ( int newId, char * );
rcl_return_code SetDataLister_Remove ( int newId );
rcl_return_code SetDataLister_GetPosById ( int newId, int * pos );
rcl_return_code SetDataLister_GetIdByPos ( int pos, int * newId ) ;
void SetDatList_Format ( u16 pid, u16 type, char * txt ) ;

rcl_return_code SetDataList_SetNotSelected ( int numPos, int * pos_list ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int               current_setdata_id_g = 0 ;
static SetDataList_S     * setdataList_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentSetDataId ( int newId )
{
   current_setdata_id_g = newId ;
}
int GetCurrentSetDataId ( void )
{
   return current_setdata_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetDataLister_Reset ( void ) 
{
   SetDataList_S  *  curr ;
   SetDataList_S  *  next ;

   if ( setdataList_g == NULL ) {
      return ;
   }

   curr = setdataList_g ;
   next = setdataList_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   setdataList_g = NULL ;
}
rcl_return_code SetDataLister_Add ( int newId, char * txt )
{
   SetDataList_S  ** curr ;
   XmString        xmstr ;

   xmstr = ( XmString ) strToXmString ( txt ) ;
 
   if ( setdataList_g == NULL ) {
      setdataList_g = ( SetDataList_S * ) malloc ( sizeof ( SetDataList_S ) ) ;
      setdataList_g -> id = newId ;
      setdataList_g -> next = NULL ;

      XmListAddItem ( data_PE_form3_list_lister, xmstr, 0 ) ;
      XmStringFree ( xmstr ) ;

      return RCL_SUCCESS ;
   }

   curr = & setdataList_g ;
   while ( 1 ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( SetDataList_S * ) malloc ( sizeof ( SetDataList_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         XmListAddItem ( data_PE_form3_list_lister, xmstr, 0 ) ;
         XmStringFree ( xmstr ) ;

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;
   }
   return RCL_SUCCESS ;
}
rcl_return_code SetDataLister_Remove ( int newId )
{
   SetDataList_S  ** prev ;
   SetDataList_S  ** curr ;
   
   if ( setdataList_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( setdataList_g -> next == NULL ) {
      if ( setdataList_g -> id == newId ) {
         free ( setdataList_g ) ;
         setdataList_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & setdataList_g ;
   curr = & setdataList_g ;
   while ( ( *curr ) ) {
      if ( ( * curr ) -> id == newId ) {
         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;
         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code SetDataLister_GetPosById ( int newId, int * pos )
{
   int               i ;
   SetDataList_S  * curr ;
   
   if ( setdataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = setdataList_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code SetDataLister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   SetDataList_S  * curr ;
   
   if ( setdataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = setdataList_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}
rcl_return_code SetDataList_SetNotSelected ( int numPos, int * pos_list ) 

{
   int               i ;
   SetDataList_S  * curr ;
   
   if ( setdataList_g == NULL ) {
      return RCL_FAILURE ;
   } 

   for ( i = 0 ; i < numPos ; i ++ ) {
     XmListDeselectPos ( data_PE_form3_list_lister, pos_list [ i ] + 1 ) ; 
   }

   return RCL_SUCCESS ;
}

