#ifndef _sky_skysub_h_
#define _sky_skysub_h_

#define MAX_SATELLITE_NUM        8 
#define MAX_SATELLITE_NAME_LEN   50 
#define MAX_FILE_NAME_LEN        1024 
#define MAX_CHANNEL_NAME_LEN     50
#define MAX_TRANSPONDER_NUM      100

#ifndef MAX_PROGRAM_NUM
#define MAX_PROGRAM_NUM          80
#endif

#define MAX_CHANNEL_NUM          MAX_PROGRAM_NUM

#ifndef MAX_DATA_NUM
#define MAX_DATA_NUM             80
#endif

#define MAX_DATA_PID_VAL         0x1FFFF

#define MAX_VOLUME_NUM           20
#define MAX_AGC_NUM              20
#define MAX_BER_NUM              20
#define MAX_FULL_PATH_LEN        100
#define MAX_FAVORITE_CHANNEL_NUM MAX_CHANNEL_NUM

#define DATAFILE_PATH                    "./datafiles"
#define INITIAL_INFO_FILE_NAME           DATAFILE_PATH"/.info.ini"
#define LOCAL_OSCILATOR_FILE_NAME        DATAFILE_PATH"/.info_osc.ini"
#define IP_ADDRESS_LIST_FILE_NAME        DATAFILE_PATH"/.info_ip.ini"

#if 1
#define SATELLITE_FILE_NAME_EXTENSION    "smd"
#define KOREA_SAT_FILE_NAME              "KoreaSat."SATELLITE_FILE_NAME_EXTENSION
#define ASIA_SAT_2_FILE_NAME             "AsiaSat2."SATELLITE_FILE_NAME_EXTENSION
#define HOTBIRD_FILE_NAME                "HotBird."SATELLITE_FILE_NAME_EXTENSION
#define ASTRA_FILE_NAME                  "Astra."SATELLITE_FILE_NAME_EXTENSION
#endif

#define FIRMWARE_FILE_NAME               "skymedia.bin"

/* DATA Service Type String */

#define SERVICE_TYPE_INT_1     1
#define SERVICE_TYPE_INT_2     2
#define SERVICE_TYPE_INT_3     3
#define SERVICE_TYPE_INT_4     5

#define SERVICE_TYPE_1         "Data Piping"
#define SERVICE_TYPE_2         "Data Streaming"
#define SERVICE_TYPE_3         "Multiprotocol Encapsulation"
#define SERVICE_TYPE_4         "Data Streaming PES 1"
#define SERVICE_TYPE_5         "Data Carousel"

typedef struct {
   int           flag ; /* 0 : empty 1 : entry */
   char          satellite_name [ MAX_SATELLITE_NAME_LEN ] ;
   char          satellite_file [ MAX_FILE_NAME_LEN ] ;
} Sky_SatelliteEntry_S ;

struct Sky_LocalOscilator_Struct {
   unsigned int                        value ;   
   struct Sky_LocalOscilator_Struct  * next ;
} Sky_LocalOscilator_S ;

typedef enum {
   SKY_NETCARD_IP = 0 ,
   SKY_NETCARD_MAC = 1
} Sky_SelectNetCard_E ;

typedef struct {
   char                  satellite_name [ MAX_SATELLITE_NAME_LEN ] ;
   char                  satellite_file [ MAX_FILE_NAME_LEN ] ;
   int                   transponder_id ;
   int                   num_of_satellite ;
   Sky_SatelliteEntry_S  sat_list [ MAX_SATELLITE_NUM ] ; 
   Sky_SelectNetCard_E   ip_or_mac ; /* ip : 0, mac 1 */
} Sky_LastInfo_S ;

typedef struct {
   char          satellite_name [ MAX_SATELLITE_NAME_LEN ] ;
   char          satellite_file [ MAX_FILE_NAME_LEN ] ;
   int           cur_transponder_id ;
   int           num_of_transponder ;

   LNB_STRUCTURE lnb ;
   TR_STRUCTURE  transponder [ MAX_TRANSPONDER_NUM ] ;
   int                   num_of_data [ MAX_TRANSPONDER_NUM ] ;
   DATAPROGRAM_STRUCTURE data [ MAX_TRANSPONDER_NUM ] [ MAX_DATA_NUM ] ;
} Sky_SatelliteInfo_S ;

/* start to add by hochoi 1999. 10. 2 */
                              /* 123456789012345 */
#define TONE_BURST_DISABLE_STR  "    Disable"
#define TONE_BURST_A_STR        " Tone Burst A"
#define TONE_BURST_B_STR        " Tone Burst B"
#define DISEQC_DISABLE_STR      "    Disable"
#define DISEQC_PORT1_STR        "    Port 1"
#define DISEQC_PORT2_STR        "    Port 2"
#define DISEQC_PORT3_STR        "    Port 3"
#define DISEQC_PORT4_STR        "    Port 4"
#define DISEQC_PORT5_STR        "    Port 5"
#define DISEQC_PORT6_STR        "    Port 6"
#define DISEQC_PORT7_STR        "    Port 7"
#define DISEQC_PORT8_STR        "    Port 8"
#define DISEQC_PORT9_STR        "    Port 9"
#define DISEQC_PORT10_STR       "    Port 10"
#define DISEQC_PORT11_STR       "    Port 11"
#define DISEQC_PORT12_STR       "    Port 12"
#define DISEQC_PORT13_STR       "    Port 13"
#define DISEQC_PORT14_STR       "    Port 14"
#define DISEQC_PORT15_STR       "    Port 15"
#define DISEQC_PORT16_STR       "    Port 16"

                              /* 123456789012345 */
#define LNB_SWITCH_OFF_STR      "      22KHz Off"
#define LNB_SWITCH_ON_STR       "       22KHz On"
#define LNB_SWITCH_LIMIT_STR    "    22KHz Limit"

#define LNB_POWER_ON_STR        "   LNB Power On"
#define LNB_POWER_OFF_STR       "  LNB Power Off"
#define LNB_POWER_LIMIT_STR     "LNB Power Limit"

#define TR_SIGNAL_TYPE_VERTICAL_STR    'V'
#define TR_SIGNAL_TYPE_HORIZONTAL_STR  'H'
#define TR_SIGNAL_TYPE_LIMIT_STR       'L'

typedef unsigned short  rcl_return_code ;
#ifndef RCL_SUCCESS
#define RCL_SUCCESS 0
#endif
#ifndef RCL_FAILURE
#define RCL_FAILURE 1
#endif

#define MAX_PROGRESS_BAR_NUM       35
#define MAX_LOCAL_OSCILATOR_NUM    30

#endif
