/* 
 * Source Name : sky_trans2_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_trans.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void Trans2_SetCurrentTrans2Id ( int newId ) ;
int Trans2_GetCurrentTrans2Id ( void ) ;

void Trans2Lister_Reset ( void ) ;
rcl_return_code Trans2Lister_Add ( int newId, char * );
rcl_return_code Trans2Lister_Remove ( int newId );
rcl_return_code Trans2Lister_GetPosById ( int newId, int * pos );
rcl_return_code Trans2Lister_GetIdByPos ( int pos, int * newId ) ;
void Trans2List_Format ( int num, u32 freq, u32 symRate, SIGNAL_TYPE type, char * txt ) ;

rcl_return_code Trans2List_SetNotSelected ( int numPos, int * pos_list ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int               current_trans2_id_g = 0 ;
static Trans2List_S     * trans2List_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentTrans2Id ( int newId )
{
   current_trans2_id_g = newId ;
}
int GetCurrentTrans2Id ( void )
{
   return current_trans2_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void Trans2Lister_Reset ( void ) 
{
   Trans2List_S  *  curr ;
   Trans2List_S  *  next ;

   if ( trans2List_g == NULL ) {
      return ;
   }

   curr = trans2List_g ;
   next = trans2List_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   trans2List_g = NULL ;
}
rcl_return_code Trans2Lister_Add ( int newId, char * txt )
{
   Trans2List_S  ** curr ;
   XmString        xmstr ;

   xmstr = ( XmString ) strToXmString ( txt ) ;
 
   if ( trans2List_g == NULL ) {
      trans2List_g = ( Trans2List_S * ) malloc ( sizeof ( Trans2List_S ) ) ;
      trans2List_g -> id = newId ;
      trans2List_g -> next = NULL ;

      XmListAddItem ( import_transListLister, xmstr, 0 ) ;
      XmStringFree ( xmstr ) ;

      return RCL_SUCCESS ;
   }

   curr = & trans2List_g ;
   while ( 1 ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( Trans2List_S * ) malloc ( sizeof ( Trans2List_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         XmListAddItem ( import_transListLister, xmstr, 0 ) ;
         XmStringFree ( xmstr ) ;

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;
   }
   return RCL_SUCCESS ;
}
rcl_return_code Trans2Lister_Remove ( int newId )
{
   Trans2List_S  ** prev ;
   Trans2List_S  ** curr ;
   
   if ( trans2List_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( trans2List_g -> next == NULL ) {
      if ( trans2List_g -> id == newId ) {
         free ( trans2List_g ) ;
         trans2List_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & trans2List_g ;
   curr = & trans2List_g ;
   while ( ( *curr ) ) {
      if ( ( * curr ) -> id == newId ) {
         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;
         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code Trans2Lister_GetPosById ( int newId, int * pos )
{
   int               i ;
   Trans2List_S  * curr ;
   
   if ( trans2List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = trans2List_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code Trans2Lister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   Trans2List_S  * curr ;
   
   if ( trans2List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = trans2List_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}
rcl_return_code Trans2List_SetNotSelected ( int numPos, int * pos_list ) 

{
   int               i ;
   Trans2List_S  * curr ;
   
   if ( trans2List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   for ( i = 0 ; i < numPos ; i ++ ) {
     XmListDeselectPos ( import_transListLister, pos_list [ i ] + 1 ) ; 
   }

   return RCL_SUCCESS ;
}
void Trans2List_Format ( int num, unsigned int freq, unsigned int symRate, SIGNAL_TYPE type, char * txt )
{
  char   buf ;

  switch ( type ) {
    case SIGNAL_VERTICAL :
      buf = TR_SIGNAL_TYPE_VERTICAL_STR ; 
      break ;
    case SIGNAL_HORIZONTAL :
      buf = TR_SIGNAL_TYPE_HORIZONTAL_STR ; 
      break ;
    case SIGNAL_LIMIT :
      buf = TR_SIGNAL_TYPE_LIMIT_STR ; 
      break ;
    default :
      buf = 'U' ; break ;
  }
  #ifdef DEBUG
  printf ( "%s : %d : freq = %d, symbol rate = %d, signal type = %d ( %c )\n", 
           __FILE__, __LINE__, ( float ) freq, ( float ) symRate, type, buf ) ;
  #endif
  /* Frequency & Symbol Rate */
  sprintf ( txt, "  %-2d  %-16.3f %-23.3f %c", num, ( ( float ) freq / 1000.0 ) , ( ( float ) symRate / 1000.0 ) , buf ) ;

  #ifdef DEBUG
  printf ( "%s : %d : Freq + Symbol = %s\n", __FILE__, __LINE__, txt ) ;
  #endif

}

