/* 
 * Source Name : sky_trans3_list.c
 * Description : 
 *
 */

/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#ifdef SMARTCARD_VERSION
#include <rpc/rpc.h>
#include <rpc/svc.h>
#endif

#include <sky_extern.h>
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_inc.h"
#include "sky_trans.h"

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES ( CallBack Functions )
*****************************************************************************/

/*****************************************************************************
** INTERAL FUNCTION PROTOTYPES 
*****************************************************************************/
void Trans3_SetCurrentTrans3Id ( int newId ) ;
int Trans3_GetCurrentTrans3Id ( void ) ;

void Trans3Lister_Reset ( void ) ;
rcl_return_code Trans3Lister_Add ( int newId, char * );
rcl_return_code Trans3Lister_Remove ( int newId );
rcl_return_code Trans3Lister_GetPosById ( int newId, int * pos );
rcl_return_code Trans3Lister_GetIdByPos ( int pos, int * newId ) ;
void Trans3List_Format ( int num, u32 freq, u32 symRate, char * txt ) ;
void Trans3List_Format_2 ( int num, unsigned int freq, unsigned int symRate, int lock, char * txt ) ;
void Trans3List_Format_3 ( int num, unsigned int freq, unsigned int symRate, char * txt ) ;

rcl_return_code Trans3List_SetNotSelected ( int numPos, int * pos_list ) ;
void TransponderLockStatus ( int lock, char * txt ) ;

/*****************************************************************************
** GLOBAL VARIABLES
*****************************************************************************/
static int               current_trans3_id_g = 0 ;
static Trans3List_S     * trans3List_g = NULL ;

/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void SetCurrentTrans3Id ( int newId )
{
   current_trans3_id_g = newId ;
}
int GetCurrentTrans3Id ( void )
{
   return current_trans3_id_g ;
}
/*****************************************************************************
** FUNCTION NAME : 
** DESCRIPTION   :
** PARAMETERS    : [in]
**                 [out]
** RETURN VALUE  : void
*****************************************************************************/
void Trans3Lister_Reset ( void ) 
{
   Trans3List_S  *  curr ;
   Trans3List_S  *  next ;

   if ( trans3List_g == NULL ) {
      return ;
   }

   curr = trans3List_g ;
   next = trans3List_g -> next ;
   while ( 1 ) {
      if ( next == NULL ) {
         free ( curr ) ; 
         break ;
      }

      free ( curr ) ;

      curr = next ;
      next = curr -> next ;
   }
   
   trans3List_g = NULL ;
}
rcl_return_code Trans3Lister_Add ( int newId, char * txt )
{
   Trans3List_S  ** curr ;
   XmString        xmstr ;

   xmstr = ( XmString ) strToXmString ( txt ) ;
 
   if ( trans3List_g == NULL ) {
      trans3List_g = ( Trans3List_S * ) malloc ( sizeof ( Trans3List_S ) ) ;
      trans3List_g -> id = newId ;
      trans3List_g -> next = NULL ;

      XmListAddItem ( autoscan_listLister, xmstr, 0 ) ;
      XmStringFree ( xmstr ) ;

      return RCL_SUCCESS ;
   }

   curr = & trans3List_g ;
   while ( 1 ) {
      if ( ( * curr ) -> next == NULL ) {
         ( * curr ) -> next = ( Trans3List_S * ) malloc ( sizeof ( Trans3List_S ) ) ; 
         ( ( * curr ) -> next ) -> id = newId ;
         ( ( * curr ) -> next ) -> next = NULL ;

         XmListAddItem ( autoscan_listLister, xmstr, 0 ) ;
         XmStringFree ( xmstr ) ;

         break ;
      }
 
      curr = & ( ( * curr ) -> next ) ;
   }
   return RCL_SUCCESS ;
}
rcl_return_code Trans3Lister_Remove ( int newId )
{
   Trans3List_S  ** prev ;
   Trans3List_S  ** curr ;
   
   if ( trans3List_g == NULL ) {
      return RCL_FAILURE ;
   } else if ( trans3List_g -> next == NULL ) {
      if ( trans3List_g -> id == newId ) {
         free ( trans3List_g ) ;
         trans3List_g = NULL ;
         return RCL_SUCCESS ;
      } else {
         return RCL_FAILURE ;
      }
   }

   prev = & trans3List_g ;
   curr = & trans3List_g ;
   while ( ( *curr ) ) {
      if ( ( * curr ) -> id == newId ) {
         ( * prev ) -> next = ( * curr ) -> next ; 
         free ( * curr ) ;
         * curr = NULL ;
         return RCL_SUCCESS ;
      }
      prev = curr ;
      curr = & ( ( * curr ) -> next ) ;
   }

   return RCL_FAILURE ;
}
/* position : 0,1,2,..... */
rcl_return_code Trans3Lister_GetPosById ( int newId, int * pos )
{
   int               i ;
   Trans3List_S  * curr ;
   
   if ( trans3List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = trans3List_g ;
   for ( i = 0 ; curr ; i ++ ) {
      if ( curr -> id == newId ) {
         * pos = i ;
         return RCL_SUCCESS ;
      }
      curr = curr -> next ;
   }

   return RCL_FAILURE ;
}
rcl_return_code Trans3Lister_GetIdByPos ( int pos, int * newId )
{
   int               i ;
   Trans3List_S  * curr ;
   
   if ( trans3List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   curr = trans3List_g ;
   for ( i = 0 ; i < pos ; i ++ ) {

      if ( curr == NULL ) {
        break ;
      } else {
        curr = curr -> next ;
      }
   }

   if ( curr == NULL ) {
     return RCL_FAILURE ;
   } else {
      * newId = curr -> id ;
      return RCL_SUCCESS ;
   }
}
rcl_return_code Trans3List_SetNotSelected ( int numPos, int * pos_list ) 

{
   int               i ;
   Trans3List_S  * curr ;
   
   if ( trans3List_g == NULL ) {
      return RCL_FAILURE ;
   } 

   for ( i = 0 ; i < numPos ; i ++ ) {
     XmListDeselectPos ( autoscan_listLister, pos_list [ i ] + 1 ) ; 
   }

   return RCL_SUCCESS ;
}

void Trans3List_Format ( int num, unsigned int freq, unsigned int symRate, char * txt )
{
  #ifdef DEBUG
  printf ( "%s : %d : freq = %d, symbol rate = %d\n", 
           __FILE__, __LINE__, ( float ) freq, ( float ) symRate ) ;
  #endif

  /* Frequency & Symbol Rate */
  sprintf ( txt, " %2d   %-11.3f %-13.3f", num, ( ( float ) freq / 1000.0 ) , ( ( float ) symRate / 1000.0 ) ) ;

  #ifdef DEBUG
  printf ( "%s : %d : Freq + Symbol = %s\n", __FILE__, __LINE__, txt ) ;
  #endif

}

void Trans3List_Format_2 ( int num, unsigned int freq, unsigned int symRate, int lock, char * txt )
{
  char    buf [ 20 ] ;

  if ( lock == SKY_OK ) {
    sprintf ( buf, "Lock" ) ;
  } else {
    sprintf ( buf, "Unlock" ) ;
  }
  #ifdef DEBUG
  printf ( "%s : %d : freq = %d, symbol rate = %d, lock = %s\n", 
           __FILE__, __LINE__, freq, symRate, buf ) ;
  #endif

  /* Frequency & Symbol Rate */
  sprintf ( txt, " %2d   %-11.3f %-13.3f %s", num, ( ( float ) freq / 1000.0 ) , ( ( float ) symRate / 1000.0 ), buf ) ;

}

void Trans3List_Format_3 ( int num, unsigned int freq, unsigned int symRate, char * txt )
{
  char    * buf = "Scanning" ;

  #ifdef DEBUG
  printf ( "%s : %d : freq = %d, symbol rate = %d\n", 
           __FILE__, __LINE__, freq, symRate ) ;
  #endif

  sprintf ( txt, "%s [%d] %.3f[MHz], %.3f[MSymbol]", buf, num, ( ( float ) freq / 1000.0 ) , ( ( float ) symRate / 1000.0 ) ) ;

}

void TransponderLockStatus ( int lock, char * txt )
{
  if ( lock == SKY_OK ) {
    strcpy ( txt, "Lock" ) ;
  } else {
    strcpy ( txt, "Unlock" ) ;
  }
}
