#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include "sky_extern.h"
#include "sky_pixmap.h"
#include "sky_windows.h"
#include "sm2_lib.h"
#include "skymedia.h"
#include "sky_skysub.h"
#include "sky_app.h"
#include "sky_inc.h"
#include "sky_sat_osc.h"

/**************************************************************************
 * EXTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void TransponderList_Format ( u32 freq, u32 symRate, SIGNAL_TYPE type, char * txt ) ;
void List_SelectedScrolled ( Widget w, int pos, int page ) ;
void LockStatus_DisplayToTitle ( char * , TR_STRUCTURE * tp_p, int sts ) ;
void LockStatus_ResetDisplayToTile ( char * ) ;

/**************************************************************************
 * INTERNAL FUNCTION PROTOTYPES
 **************************************************************************/
void display_satellite_name_list ( Sky_LastInfo_S * sky_p ) ;
void refresh_transponder_info ( void ) ;
void display_satellite_lnb_info ( LNB_STRUCTURE * lnb_p ) ;
void allClearTunerPanel ( void ) ;
void ConvertInt2Str_LocalOscillator ( unsigned int x, char * txt ) ;
void ConvertInt2Str_LocalOscillator_2 ( unsigned int x, char * txt ) ;
void ConvertInt2Str_ToneBurst ( unsigned int ToneBurst, char * txt ) ;
void ConvertInt2Str_SwitchOnOff ( LNB_HILOW SwitchOnOff , char * txt ) ;
void ConvertInt2Str_DiseqcPort ( unsigned char DiseqcPort , char * txt ) ;
void ConvertInt2Str_LNBPower ( LNB_POWER LNBPower , char * txt ) ;

void display_satellite_name_list ( Sky_LastInfo_S * sky_p ) 
{
   Arg        args [ 2 ] ;
   int        ac ;
   XmString   xmstr ;

   int        i ;
   int        j ;

   #ifdef DEBUG
   printf ( "display_satellite_name_list (  )\n" ) ;
   #endif

   /* Clear */
   xmstr = XmStringCreate ( " ", XmFONTLIST_DEFAULT_TAG ) ; 
   ac = 0 ;
   XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
   ac ++ ;

   for ( i = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {
      switch ( i ) {
         case 0 :
            XtSetValues ( tuner_satMenu_pdB1, args, ac ) ;
            break ;
         case 1 :
            XtSetValues ( tuner_satMenu_pdB2, args, ac ) ;
            break ;
         case 2 :
            XtSetValues ( tuner_satMenu_pdB3, args, ac ) ;
            break ;
         case 3 :
            XtSetValues ( tuner_satMenu_pdB4, args, ac ) ;
            break ;
         case 4 :
            XtSetValues ( tuner_satMenu_pdB5, args, ac ) ;
            break ;
         case 5 :
            XtSetValues ( tuner_satMenu_pdB6, args, ac ) ;
            break ;
         case 6 :
            XtSetValues ( tuner_satMenu_pdB7, args, ac ) ;
            break ;
         case 7 :
            XtSetValues ( tuner_satMenu_pdB8, args, ac ) ;
            break ;
      }
   }

   /* Setting */
   for ( i = 0, j = 0 ; i < MAX_SATELLITE_NUM ; i ++ ) {

      #ifdef DEBUG
      printf ( "Satellite_name [ %d ] = %s\n", i, sky_p -> sat_list [ i ] . satellite_name ) ;
      #endif

      if ( sky_p -> sat_list [ i ] . flag == 0 ) {
        continue ;
      } else if ( j == sky_p -> num_of_satellite ) {
        break ;
      }

      xmstr = XmStringCreate ( sky_p -> sat_list [ i ] . satellite_name, XmFONTLIST_DEFAULT_TAG ) ; 
      ac = 0 ;
      XtSetArg ( args [ ac ], XmNlabelString, xmstr ) ; 
      ac ++ ;

      switch ( j ) {
         case 0 :
            XtSetValues ( tuner_satMenu_pdB1, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB1 ) ;
            break ;
         case 1 :
            XtSetValues ( tuner_satMenu_pdB2, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB2 ) ;
            break ;
         case 2 :
            XtSetValues ( tuner_satMenu_pdB3, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB3 ) ;
            break ;
         case 3 :
            XtSetValues ( tuner_satMenu_pdB4, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB4 ) ;
            break ;
         case 4 :
            XtSetValues ( tuner_satMenu_pdB5, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB5 ) ;
            break ;
         case 5 :
            XtSetValues ( tuner_satMenu_pdB6, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB6 ) ;
            break ;
         case 6 :
            XtSetValues ( tuner_satMenu_pdB7, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB7 ) ;
            break ;
         case 7 :
            XtSetValues ( tuner_satMenu_pdB8, args, ac ) ;
            XtManageChild ( tuner_satMenu_pdB8 ) ;
            break ;
      }
      XmStringFree ( xmstr ) ;
      j ++ ;
   }
   for ( i = sky_p -> num_of_satellite ; i < MAX_SATELLITE_NUM ; i ++ ) {
      switch ( i ) {
         case 0 :
            XtUnmanageChild ( tuner_satMenu_pdB1 ) ;
            break ;
         case 1 :
            XtUnmanageChild ( tuner_satMenu_pdB2 ) ;
            break ;
         case 2 :
            XtUnmanageChild ( tuner_satMenu_pdB3 ) ;
            break ;
         case 3 :
            XtUnmanageChild ( tuner_satMenu_pdB4 ) ;
            break ;
         case 4 :
            XtUnmanageChild ( tuner_satMenu_pdB5 ) ;
            break ;
         case 5 :
            XtUnmanageChild ( tuner_satMenu_pdB6 ) ;
            break ;
         case 6 :
            XtUnmanageChild ( tuner_satMenu_pdB7 ) ;
            break ;
         case 7 :
            XtUnmanageChild ( tuner_satMenu_pdB8 ) ;
            break ;
      }
      
   }
}
void refresh_transponder_info ( void ) 
{
   XmString   str ;
   Arg        args [ 2 ] ;
   int        ac ;
   char       tmp [ 200 ] ;

   int        i ;
   int        j ;
   int        find = 0 ;

   int        numOfT ;

   Sky_ReturnVal_E  ret ;
   Sky_LastInfo_S   sky_p ;

#if 0
   TP_INFORM        tplist ;
#else
   TR_STRUCTURE     tplist ;
#endif

   ret = read_initfile ( & sky_p ) ;
   /***************************************************************************
   ** Current transponder id setting
   ***************************************************************************/
   numOfT = Sky_GetNumOfTransponder ( ) ;
   if ( !numOfT ) {
     sky_p . transponder_id = 1 ;
   } else if ( sky_p . transponder_id <= 0 ) {
     sky_p . transponder_id = 1 ;
   }
   write_initfile ( & sky_p ) ;

   /*-------------------------------------------------------------*/
   /* all clear */
   /*-------------------------------------------------------------*/
   XmListDeleteAllItems ( tuner_satinfo_FqSyList_list ) ;

   /*-------------------------------------------------------------*/
   /* Get tansponder information */
   /*-------------------------------------------------------------*/
   TransLister_Reset () ;
   for ( i = 0 ; i < numOfT ; i ++ ) {

      if ( Sky_GetInfoTransponder ( i + 1, & tplist ) != FALSE ) {

         #ifdef DEBUG_FAIL
         printf ( "%s : %d : Sky_GetInfoTransponder () Fail\n", __FILE__, __LINE__ ) ;
         #endif

         return ;
      }
      #ifdef DEBUG
      printf ( "[ %d ] . TransponderNumber = %d\n", i, tplist . TransponderNumber ) ;
      printf ( "[ %d ] . DownlinkFrequency = %d\n", i, tplist . DownlinkFrequency ) ;
      printf ( "[ %d ] . SymbolRate        = %d\n", i, tplist . SymbolRate ) ;
      printf ( "[ %d ] . SignalType        = %d\n", i, tplist . SignalType ) ;
      printf ( "[ %d ] . TransportID       = %d\n", i, tplist . TransportID ) ;
      #endif

      memset ( tmp, 0, sizeof ( tmp ) ) ;
      TransponderList_Format ( tplist . DownlinkFrequency, 
                               tplist . SymbolRate, 
                               tplist . SignalType, 
                               tmp ) ;
      /* delete by hoc 1999.10.2 
      str = XmStringCreate ( tmp, XmFONTLIST_DEFAULT_TAG ) ;
      XmListAddItem ( tuner_satinfo_FqSyList_list, str, 0 ) ;
      XmStringFree ( str ) ;
      */

      TransLister_Add ( i + 1, tmp ) ;

      if ( sky_p . transponder_id == ( i + 1 ) ) {
        LockStatus_DisplayToTitle ( sky_p.satellite_name, & tplist, 0 ) ;
      }
   }
   if ( i ) {
#ifdef LINUX_VERSION
      List_SelectedScrolled ( tuner_satinfo_FqSyList_list, sky_p . transponder_id, 6 ) ;
#else
      List_SelectedScrolled ( tuner_satinfo_FqSyList_list, sky_p . transponder_id, 5 ) ;
#endif
   } else {
      LockStatus_ResetDisplayToTitle ( ( char * )sky_p.satellite_name ) ;
   }
}

void display_satellite_lnb_info ( LNB_STRUCTURE * lnb_p ) 
{
  char    txt [ 100 ] ;

  /*----------------------------------------------------------*/
  /* Display LNB Information */
  /*----------------------------------------------------------*/
  /*----------------------------------------------------------*/
  /* Local Oscillator */  
  /*----------------------------------------------------------*/
  /*sprintf ( txt, "%d", ( lnb_p -> LocalOscillator / 1000 ) ) ;*/
  ConvertInt2Str_LocalOscillator ( ( lnb_p -> LocalOscillator / 1000 ) , txt ) ;
  XmTextSetString ( tuner_oscTxt, txt ) ;

  /*----------------------------------------------------------*/
  /* ToneBurst */
  /*----------------------------------------------------------*/
  ConvertInt2Str_ToneBurst ( lnb_p -> ToneBurst, txt ) ;
  XmTextSetString ( tuner_toneTxt1, txt ) ;

  /*----------------------------------------------------------*/
  /* Switch On/Off */
  /*----------------------------------------------------------*/
  if ( lnb_p -> LocalOscillator ) {
    ConvertInt2Str_SwitchOnOff ( lnb_p -> SwitchOnOff , txt ) ;
    XmTextSetString ( tuner_22kTxt, txt ) ;
  } else {
    XmTextSetString ( tuner_22kTxt, NULL ) ;
  }

  /*----------------------------------------------------------*/
  /* DiseqcPort */
  /*----------------------------------------------------------*/
  ConvertInt2Str_DiseqcPort ( lnb_p -> DiseqcPort , txt ) ;
  XmTextSetString ( tuner_diseqcTxt, txt ) ;

  /*----------------------------------------------------------*/
  /* LNB Power */
  /*----------------------------------------------------------*/
  ConvertInt2Str_LNBPower ( lnb_p -> LNBPower , txt ) ;
  switch ( lnb_p -> LNBPower ) {
    case LNB_POWER_ON :
      strcpy ( txt, LNB_POWER_ON_STR ) ;
      break ;
    case LNB_POWER_OFF :
      strcpy ( txt, LNB_POWER_OFF_STR ) ;
      break ;
    case LNB_POWER_LIMIT :
      strcpy ( txt, LNB_POWER_LIMIT_STR ) ;
      break ;
    default :
      strcpy ( txt, "Unknown" ) ;
      break ;
  }
  XmTextSetString ( tuner_lnbTxt, txt ) ;
}

void allClearTunerPanel ( void )    
{
   Arg              args [ 2 ] ;
   int              ac ;

   /* satellite name */
   XmTextSetString ( tuner_satName, NULL ) ;
   /* Local oscillator */
   XmTextSetString ( tuner_oscTxt, NULL ) ;
   /* tone burst */
   XmTextSetString ( tuner_toneTxt1, NULL ) ;
   /* switch on/off */
   XmTextSetString ( tuner_22kTxt, NULL ) ;
   /* diseqc */
   XmTextSetString ( tuner_diseqcTxt, NULL ) ;
   /* LNB power */
   XmTextSetString ( tuner_lnbTxt, NULL ) ;

   /* transponder list */
   XmListDeleteAllItems ( tuner_satinfo_FqSyList_list ) ;
}
void ConvertInt2Str_LocalOscillator ( unsigned int x, char * txt ) 
{
  int          i ;
  unsigned int y = 1 ;
  unsigned int z = x ;
  char         buf [ 8 ] ;

  if ( x == 0 ) {
    strcpy ( txt, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ;
    return ;
  }

  memset ( buf, ' ', sizeof ( buf ) ) ;
  buf [ 7 ] = 0 ;
  for ( i = 1 ; i <= 10 ; i ++ ) {
    y = y * 10 ;
    z = z / y ;
    if ( z == 0 ) {
      break ;
    }
    if ( ! ( i % 2 ) ) {  
      buf [ 8 - i ] = 0 ;
    }
  }
  sprintf ( txt, "%s%d", buf, x ) ;
}
void ConvertInt2Str_LocalOscillator_2 ( unsigned int x, char * txt ) 
{
  if ( x == 0 ) {
    strcpy ( txt, LOCAL_OSCILLATOR_ZERO_VAL_STR ) ;
  } else {
    sprintf ( txt, "%d", x ) ;
  }
}
void ConvertInt2Str_ToneBurst ( unsigned int ToneBurst, char * txt ) 
{
  switch ( ToneBurst ) {
    case TONE_BURST_DISABLE :
      strcpy ( txt, TONE_BURST_DISABLE_STR ) ;
      break ;
    case TONE_BURST_A :
      strcpy ( txt, TONE_BURST_A_STR ) ;
      break ;
    case TONE_BURST_B :
      strcpy ( txt, TONE_BURST_B_STR ) ;
      break ;
    default :
      strcpy ( txt, "Unknown" ) ;
      break ;
  }
}
void  ConvertInt2Str_SwitchOnOff ( LNB_HILOW SwitchOnOff , char * txt ) 
{
  switch ( SwitchOnOff ) {
    case LNB_HIGH :
      strcpy ( txt, LNB_SWITCH_ON_STR ) ;
      break ;
    case LNB_LOW :
      strcpy ( txt, LNB_SWITCH_OFF_STR ) ;
      break ;
    case LNB_LIMIT :
      strcpy ( txt, LNB_SWITCH_LIMIT_STR ) ;
      break ;
    default :
      strcpy ( txt, "Unknown" ) ;
      break ;
  }
}
void ConvertInt2Str_DiseqcPort ( unsigned char DiseqcPort , char * txt ) 
{
  switch ( DiseqcPort ) {
    case DISEQC_DISABLE :
      strcpy ( txt, DISEQC_DISABLE_STR ) ;
      break ;
    case DISEQC_PORT1 :
      strcpy ( txt, DISEQC_PORT1_STR ) ;
      break ;
    case DISEQC_PORT2 :
      strcpy ( txt, DISEQC_PORT2_STR ) ;
      break ;
    case DISEQC_PORT3 :
      strcpy ( txt, DISEQC_PORT3_STR ) ;
      break ;
    case DISEQC_PORT4 :
      strcpy ( txt, DISEQC_PORT4_STR ) ;
      break ;
    case DISEQC_PORT5 :
      strcpy ( txt, DISEQC_PORT5_STR ) ;
      break ;
    case DISEQC_PORT6 :
      strcpy ( txt, DISEQC_PORT6_STR ) ;
      break ;
    case DISEQC_PORT7 :
      strcpy ( txt, DISEQC_PORT7_STR ) ;
      break ;
    case DISEQC_PORT8 :
      strcpy ( txt, DISEQC_PORT8_STR ) ;
      break ;
    case DISEQC_PORT9 :
      strcpy ( txt, DISEQC_PORT9_STR ) ;
      break ;
    case DISEQC_PORT10 :
      strcpy ( txt, DISEQC_PORT10_STR ) ;
      break ;
    case DISEQC_PORT11 :
      strcpy ( txt, DISEQC_PORT11_STR ) ;
      break ;
    case DISEQC_PORT12 :
      strcpy ( txt, DISEQC_PORT12_STR ) ;
      break ;
    case DISEQC_PORT13 :
      strcpy ( txt, DISEQC_PORT13_STR ) ;
      break ;
    case DISEQC_PORT14 :
      strcpy ( txt, DISEQC_PORT14_STR ) ;
      break ;
    case DISEQC_PORT15 :
      strcpy ( txt, DISEQC_PORT15_STR ) ;
      break ;
    case DISEQC_PORT16 :
      strcpy ( txt, DISEQC_PORT16_STR ) ;
      break ;
    default :
      strcpy ( txt, "Unknown" ) ;
      break ;
  }
}
void ConvertInt2Str_LNBPower ( LNB_POWER LNBPower , char * txt ) 
{
  switch ( LNBPower ) {
    case LNB_POWER_ON :
      strcpy ( txt, LNB_POWER_ON_STR ) ;
      break ;
    case LNB_POWER_OFF :
      strcpy ( txt, LNB_POWER_OFF_STR ) ;
      break ;
    case LNB_POWER_LIMIT :
      strcpy ( txt, LNB_POWER_LIMIT_STR ) ;
      break ;
    default :
      strcpy ( txt, "Unknown" ) ;
      break ;
  }
}
